@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/calma/CalmaRdcl.c,v 1.4 2001/07/23 17:49:49 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/calma/CalmaRdcl.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -276,9 +276,6 @@ calmaParseStructure(filename)
     bool was_called;
     CellDef *def;
 
-    if (CalmaReadOnly)
-	filepos = ftell(calmaInputFile);
-
     /* Make sure this is a structure; if not, let the caller know we're done */
     PEEKRH(nbytes, rtype);
     if (nbytes <= 0 || rtype != CALMA_BGNSTR)
@@ -289,6 +286,9 @@ calmaParseStructure(filename)
     if (!calmaReadStringRecord(CALMA_STRNAME, &strname)) goto syntaxerror;
     TxPrintf("Reading \"%s\".\n", strname);
 
+    if (CalmaReadOnly)
+	filepos = ftell(calmaInputFile);
+
     /* Set up the cell definition */
     he = HashFind(&calmaDefInitHash, strname);
     if ((def = (CellDef *)HashGetValue(he)) != NULL)
@@ -320,6 +320,10 @@ calmaParseStructure(filename)
     /* Done with strname */
     if (strname != NULL) freeMagic(strname);
 
+    /* For read-only cells, set flag in def */
+    if (CalmaReadOnly)
+	cifReadCellDef->cd_flags |= CDVENDORGDS;
+
     /* Skip CALMA_STRCLASS or CALMA_STRTYPE */
     calmaSkipSet(structs);
 
@@ -359,7 +363,7 @@ calmaParseStructure(filename)
 	char *fpcopy = (char *)mallocMagic(20);
 	char *fncopy = StrDup(NULL, filename);
 	sprintf(fpcopy, "%d", filepos);
-	DBPropPut(cifReadCellDef, "GDS_BEGIN", (ClientData)fpcopy);
+	DBPropPut(cifReadCellDef, "GDS_START", (ClientData)fpcopy);
 
 	fpcopy = (char *)mallocMagic(20);
 	filepos = ftell(calmaInputFile);
@@ -368,8 +372,8 @@ calmaParseStructure(filename)
 
 	DBPropPut(cifReadCellDef, "GDS_FILE", (ClientData)fncopy);
 
-	/* Don't want to do this, or we can't make "fake" versions	*/
-	/* of cells like pads we want to read in from a GDS database	*/
+	/* Do not lock the cell, or else we can't save the	*/
+	/* magic cell with its GDS pointers to disk. . .	*/
 	/* cifReadCellDef->cd_flags |= CDNOEDIT; */
     }
 
@@ -398,9 +402,12 @@ calmaParseStructure(filename)
 	      (cifCurReadStyle->crs_flags & CRF_NO_RECONNECT_LABELS) ? 1 : 0);
     DBReComputeBbox(cifReadCellDef);
 
-    /* Don't bother to register with DRC if we're going to delete the cell */
+    /* Don't bother to register with DRC if we're going to delete the	*/
+    /* cell, or if the cell is read-only.				*/
+
     if (!CalmaFlattenUses || (npaths >= 10) || (nsrefs != 0))
-	DRCCheckThis(cifReadCellDef, TT_CHECKPAINT, &cifReadCellDef->cd_bbox);
+	if (!CalmaReadOnly)
+	    DRCCheckThis(cifReadCellDef, TT_CHECKPAINT, &cifReadCellDef->cd_bbox);
 
     DBWAreaChanged(cifReadCellDef, &cifReadCellDef->cd_bbox,
 	DBW_ALLWINDOWS, &DBAllButSpaceBits);
