@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic-7.5/calma/CalmaWrite.c,v 1.8 2007/10/10 18:59:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic-7.5/calma/CalmaWrite.c,v 1.9 2008/01/26 04:19:22 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -2304,7 +2304,7 @@ calmaOutStringRecord(type, str, f)
 	    else
 	    {
 		if (((unsigned char)table[c] != c) && (origstr == NULL))
-		    origstr = StrDup(NULL, locstr);
+		    origstr = StrDup(NULL, str);
 
 		c = table[c];
 		locstrprv[0] = c;
@@ -2318,7 +2318,7 @@ calmaOutStringRecord(type, str, f)
     if (origstr != NULL)
     {
 	TxError("Warning: characters changed in string \'%s\'; "
-		"modified string is \'%s\'\n", origstr, locstr);
+		"modified string is \'%s\'\n", origstr, str);
 	freeMagic(origstr);
     }
 }
