@@ -18,15 +18,13 @@
  *     *********************************************************************
  *
  *
- * rcsid "$Header: /ufs/repository/magic/cif/CIFread.h,v 1.5 2001/07/23 17:49:50 tim Exp $
+ * rcsid "$Header: /usr/cvsroot/magic-7.5/cif/CIFread.h,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $
  */
 
-#ifndef	_DATABASE
-int err1 = Need_to_include_database/database_header;
-#endif	/* _DATABASE */
-#ifndef	_CIFINT
-int err1 = Need_to_include_cifint_header;
-#endif	/* _CIFINT */
+#ifndef _CIFREAD_H
+#define _CIFREAD_H
+
+#include "cif/CIFint.h"
 
 /* The structures below are built up by CIFreadtech.c to describe
  * various styles for reading CIF.
@@ -150,12 +148,17 @@ extern bool CIFParseFinish(), CIFParseDelete(), CIFParseUser();
 extern bool CIFParseCall(), CIFParseTransform(), CIFParseInteger();
 extern bool CIFParsePath(), CIFParsePoint(), CIFParseSInteger();
 extern void CIFSkipToSemi(), CIFSkipSep(), CIFSkipBlanks();
-extern void CIFReadError(), CIFFreePath(), CIFCleanPath();
+extern void CIFFreePath(), CIFCleanPath();
 extern void CIFReadCellInit(), CIFReadCellCleanup();
 extern LinkedRect *CIFPolyToRects();
 extern Transform *CIFDirectionToTrans();
 extern int CIFReadNameToType();
 
+/* Variable argument procedures require complete prototype */
+
+extern void CIFReadError(char *format, ...);
+extern void CIFReadWarning(char *format, ...);
+
 /* Variables shared by the CIF-reading modules, see CIFreadutils.c
  * for more details:
  */
@@ -183,4 +186,4 @@ extern int cifParseLaChar;
 		? (cifParseLaAvail = FALSE, cifParseLaChar) \
 		: (cifParseLaChar = getc(cifInputFile)))
 
-/* constants */
+#endif /* _CIFREAD_H */
