@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/cif/CIFsee.c,v 1.3 2001/03/29 19:50:04 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/cif/CIFsee.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -43,7 +43,12 @@ static CellDef *cifSeeDef;
 /* Verbosity of warning messages */
 global int CIFWarningLevel = CIF_WARN_DEFAULT;
 
-
+typedef struct {
+   char *text;
+   int  layer;
+   int  style;
+} SeeLayerData;
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -64,11 +69,12 @@ global int CIFWarningLevel = CIF_WARN_DEFAULT;
  */
 
 int
-cifSeeFunc(tile, text)
+cifSeeFunc(tile, sld)
     Tile *tile;			/* Tile to be entered as feedback. */
-    char *text;			/* Explanation for the feedback. */
+    SeeLayerData *sld;		/* Layer and explanation for the feedback. */
 {
     Rect area;
+
     TiToRect(tile, &area);
 
     if (((area.r_xbot & 0x1) || (area.r_ybot & 0x1))
@@ -79,13 +85,13 @@ cifSeeFunc(tile, text)
 		(float)area.r_ybot / (float)CIFCurStyle->cs_scaleFactor);
     }
 
-    DBWFeedbackAdd(&area, text, cifSeeDef, CIFCurStyle->cs_scaleFactor,
+    DBWFeedbackAdd(&area, sld->text, cifSeeDef, CIFCurStyle->cs_scaleFactor,
 #ifdef NONMANHATTAN 
-        STYLE_PALEHIGHLIGHTS |
+	sld->style |
                 (TiGetTypeExact(tile) & (TT_DIAGONAL | TT_DIRECTION | TT_SIDE)));
         /* (preserve information about the geometry of a diagonal tile) */
 #else
-	STYLE_PALEHIGHLIGHTS);
+	sld->style);
 #endif
     return 0;
 }
@@ -118,6 +124,7 @@ CIFSeeLayer(rootDef, area, layer)
     int oldCount, i;
     char msg[100];
     SearchContext scx;
+    SeeLayerData sld;
     TileTypeBitMask mask;
 
     /* Make sure the desired layer exists. */
@@ -152,11 +159,18 @@ CIFSeeLayer(rootDef, area, layer)
 
     (void) sprintf(msg, "CIF layer \"%s\"", layer);
     cifSeeDef = rootDef;
+    sld.text = msg;
     for (i = 0; i < CIFCurStyle->cs_nLayers; i++)
     {
 	if (!TTMaskHasType(&mask, i)) continue;
-	(void) DBSrPaintArea((Tile *) NULL, CIFPlanes[i], &TiPlaneRect,
-	    &CIFSolidBits, cifSeeFunc, (ClientData) msg);
+	sld.layer = i;
+#ifdef THREE_D
+	sld.style = CIFCurStyle->cs_layers[i]->cl_renderStyle + TECHBEGINSTYLES;
+#else
+	sld.style = STYLE_PALEHIGHLIGHTS;
+#endif
+	(void) DBSrPaintArea((Tile *)NULL, CIFPlanes[i], &TiPlaneRect,
+	    &CIFSolidBits, cifSeeFunc, (ClientData)&sld);
     }
     UndoEnable();
 }
@@ -192,6 +206,7 @@ CIFSeeHierLayer(rootDef, area, layer, arrays, subcells)
     bool subcells;		/* TRUE means show subcell interactions. */
 {
     int i, oldCount;
+    SeeLayerData sld;
     char msg[100];
     TileTypeBitMask mask;
 
@@ -217,11 +232,18 @@ CIFSeeHierLayer(rootDef, area, layer, arrays, subcells)
     
     (void) sprintf(msg, "CIF layer \"%s\"", layer);
     cifSeeDef = rootDef;
+    sld.text = msg;
     for (i = 0; i < CIFCurStyle->cs_nLayers; i++)
     {
 	if (!TTMaskHasType(&mask, i)) continue;
-	(void) DBSrPaintArea((Tile *) NULL, CIFPlanes[i], &TiPlaneRect,
-	    &CIFSolidBits, cifSeeFunc, (ClientData) msg);
+	sld.layer = i;
+#ifdef THREE_D
+	sld.style = CIFCurStyle->cs_layers[i]->cl_renderStyle + TECHBEGINSTYLES;
+#else
+	sld.style = STYLE_PALEHIGHLIGHTS;
+#endif
+	(void) DBSrPaintArea((Tile *)NULL, CIFPlanes[i], &TiPlaneRect,
+	    &CIFSolidBits, cifSeeFunc, (ClientData)&sld);
     }
 }
 
