@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/cif/CIFtech.c,v 1.7 2007/10/10 18:59:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/cif/CIFtech.c,v 1.9 2008/01/09 00:51:55 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -945,8 +945,10 @@ CIFTechLine(sectionName, argc, argv)
 	if (argc < 2) goto wrongNumArgs;
 	for (i = 1; i < argc; i++)
 	{
-	    if(strcmp(argv[i], "calma-permissive-labels") == 0)
+	    if (strcmp(argv[i], "calma-permissive-labels") == 0)
 		CIFCurStyle->cs_flags |= CWF_PERMISSIVE_LABELS;
+	    else if (strcmp(argv[i], "grow-euclidean") == 0)
+		CIFCurStyle->cs_flags |= CWF_GROW_EUCLIDEAN;
 	}
 	return TRUE;
     }
@@ -2207,4 +2209,3 @@ CIFTechOutputScale(n, d)
     /* Recompute value of cs_radius */
     cifComputeHalo(ostyle);
 }
-
