@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdCD.c,v 1.6 2007/10/12 22:14:36 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdCD.c,v 1.11 2008/01/17 22:26:54 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -568,6 +568,8 @@ CmdCellname(w, cmd)
 	"create		create a new cell definition",
 	"delete		delete the named cell definition",
 	"flags		list option flags of the indicated cell definition",
+	"lock		lock the named cell (prevent changes to cell use)",
+	"unlock		unlock the named cell (allow changes to cell use)",
 	"property	list or set cell definition properties",
 	"rename		rename the indicated cell",
 	"writeable	make the cell definition read-only or read-write",
@@ -576,8 +578,8 @@ CmdCellname(w, cmd)
     typedef enum { IDX_CHILDREN, IDX_PARENTS, IDX_EXISTS, IDX_SELF,
 		   IDX_INSTANCE, IDX_CHILDINST, IDX_CELLDEF, IDX_ALLCELLS,
 		   IDX_TOPCELLS, IDX_IN_WINDOW, IDX_CREATE,
-		   IDX_DELETE, IDX_FLAGS, IDX_PROPERTY, IDX_RENAME,
-		   IDX_READWRITE } optionType;
+		   IDX_DELETE, IDX_FLAGS, IDX_LOCK, IDX_UNLOCK,
+		   IDX_PROPERTY, IDX_RENAME, IDX_READWRITE } optionType;
 
     if (strstr(cmd->tx_argv[0], "in"))
 	func = DBUsePrint;
@@ -615,10 +617,20 @@ CmdCellname(w, cmd)
 	}
     }
 
-    /* These functions only work width instances */
-
-    if (func == DBUsePrint) 
+    if (func != DBUsePrint) 
     {
+	/* These functions only work with cell uses (instances) */
+	switch (option) {
+	    case IDX_LOCK:
+	    case IDX_UNLOCK:
+		TxError("Cell definitions cannot be locked.  Use \"instance\"?\n");
+		TxError("  or do you mean \"cellname writeable\"?\n");
+		return;
+	}
+    }
+    else 
+    {
+	/* These functions only work with cell definitions */
 	switch (option) {
 	    case IDX_TOPCELLS:
 		TxError("Instances do not have a top level.  Use \"cellname\"?\n");
@@ -812,6 +824,12 @@ CmdCellname(w, cmd)
 		DBCellSetAvail(newDef);
 	    }
 	    break;
+	case IDX_LOCK:
+	    DBLockUse(cellname, TRUE);
+	    break;
+	case IDX_UNLOCK:
+	    DBLockUse(cellname, FALSE);
+	    break;
     }
     return;
 
@@ -821,6 +839,7 @@ badusage:
     TxError("or:    %s [list] allcells|topcells|window\n", cmd->tx_argv[0]);
     TxError("or:    %s create name\n", cmd->tx_argv[0]);
     TxError("or:    %s rename name newname\n", cmd->tx_argv[0]);
+    TxError("or:    %s [un]lock [name]\n", cmd->tx_argv[0]);
     TxError("or:    %s writeable [name] [true|false]\n", cmd->tx_argv[0]);
     return;
 }
@@ -1508,6 +1527,7 @@ CmdContact(w, cmd)
 	while (ccs.lhead != NULL)
 	{
 	    TTMaskSetOnlyType(&smask, type);
+	    TTMaskAndMask(&smask, &DBActiveLayerBits);
 	    DBPaintMask(EditCellUse->cu_def, &ccs.lhead->r_r, &smask);
 	    freeMagic(ccs.lhead);
 	    ccs.lhead = ccs.lhead->r_next;
@@ -1739,7 +1759,6 @@ copyToPoint:
 	GeoTransRect(&t, &rootBox, &newBox);
 	DBWSetBox(rootDef, &newBox);
     }
-    
     SelectCopy(&t);
 }
 
@@ -3388,6 +3407,12 @@ cmdDumpParseArgs(cmdName, w, cmd, dummy, scx)
 	goto usage;
     }
 
+    if (EditCellUse == NULL)
+    {
+	TxError("No cell def being edited; cannot place cell use!\n");
+	return FALSE;
+    }
+
     /* Locate the cell specified by the command */
     if (CmdIllegalChars(cmd->tx_argv[1], "", "Cell name"))
 	return (FALSE);
@@ -3395,7 +3420,7 @@ cmdDumpParseArgs(cmdName, w, cmd, dummy, scx)
     /* If the name still has ".mag" attached, then strip it. */
     clen = strlen(cmd->tx_argv[1]);
     if ((clen > 4) && !strcmp(cmd->tx_argv[1] + clen - 4, ".mag"))
-	*(cmd->tx_argv[1] - 4) = '\0';
+	*(cmd->tx_argv[1] + clen - 4) = '\0';
 
     def = DBCellLookDef(cmd->tx_argv[1]);
     if (def == (CellDef *) NULL)
