@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdLQ.c,v 1.3 2007/02/12 03:52:13 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdLQ.c,v 1.11 2008/02/03 21:23:56 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -33,6 +33,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
+#include "database/fonts.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
 #include "utils/main.h"
@@ -74,7 +75,7 @@ void CmdPaintEraseButton();
 void
 CmdLabelProc(text, pos, type)
     char *text;			/* Text for label. */
-    int pos;			/* Position for text relative to text. -1
+    int pos;			/* Justification of text relative to text. -1
 				 * means "pick a nice one for me."
 				 */
     TileType type;		/* Type of material label is to be attached
@@ -120,7 +121,8 @@ CmdLabelProc(text, pos, type)
  * Place a label at a specific point on a specific type in EditCell
  *
  * Usage:
- *	label text [direction [layer]]
+ *	label <text> [<direction> [<layer>]]
+ *	label font <filename>
  *
  * Direction may be one of:
  *	right left top bottom
@@ -162,7 +164,6 @@ CmdLabel(w, cmd)
 
     p = cmd->tx_argv[1];
 
-
     /*
      * Find and check validity of type parameter
      */
@@ -183,13 +184,13 @@ CmdLabel(w, cmd)
 
     if (cmd->tx_argc > 2)
     {
-	pos = GeoNameToPos(cmd->tx_argv[2], FALSE, TRUE);
+	pos = GeoNameToPos(cmd->tx_argv[2], FALSE, FALSE);
 	if (pos < 0)
 	    return;
-        pos = GeoTransPos(&RootToEditTransform, pos);
+	pos = GeoTransPos(&RootToEditTransform, pos);
     }
     else pos = -1;
-    
+
     CmdLabelProc(p, pos, type);
 }
 
@@ -574,7 +575,7 @@ CmdPaint(w, cmd)
     }
 
     TTMaskClearType(&mask, TT_SPACE);
-    DBPaintMask(EditCellUse->cu_def, &editRect, &mask);
+    DBPaintValid(EditCellUse->cu_def, &editRect, &mask, 0);
     DBAdjustLabels(EditCellUse->cu_def, &editRect);
     SelectClear();
     DBWAreaChanged(EditCellUse->cu_def, &editRect, DBW_ALLWINDOWS, &mask);
@@ -621,13 +622,16 @@ CmdPaintEraseButton(w, butPoint, isPaint)
 
     DBSeeTypesAll(((CellUse *)w->w_surfaceID), &rootRect, 
 	    crec->dbw_bitmask, &mask);
-    TTMaskAndMask(&mask, &DBAllButSpaceAndDRCBits);
+    TTMaskAndMask(&mask, &DBActiveLayerBits);
     TTMaskAndMask(&mask, &crec->dbw_visibleLayers);
+    TTMaskClearType(&mask, TT_SPACE);
 
     if (!ToolGetEditBox(&editRect)) return;
 
     if (TTMaskEqual(&mask, &DBZeroTypeBits))
     {
+	TileTypeBitMask eraseMask;
+
 	TTMaskAndMask3(&mask, &CmdYMAllButSpace, &crec->dbw_visibleLayers);
 
 	/* A little extra bit of cleverness:  if the box is zero size
@@ -639,14 +643,15 @@ CmdPaintEraseButton(w, butPoint, isPaint)
 	if (GEO_RECTNULL(&editRect))
 	    TTMaskSetType(&mask, L_LABEL);
 
-	DBEraseMask(EditCellUse->cu_def, &editRect, &crec->dbw_visibleLayers);
+	TTMaskAndMask3(&eraseMask, &crec->dbw_visibleLayers, &DBActiveLayerBits);
+	DBEraseValid(EditCellUse->cu_def, &editRect, &eraseMask, 0);
 	(void) DBEraseLabel(EditCellUse->cu_def, &editRect, &mask);
     }
     else if (isPaint)
-	DBPaintMask(EditCellUse->cu_def, &editRect, &mask);
+	DBPaintValid(EditCellUse->cu_def, &editRect, &mask, 0);
     else
     {
-	DBEraseMask(EditCellUse->cu_def, &editRect, &mask);
+	DBEraseValid(EditCellUse->cu_def, &editRect, &mask, 0);
 	DBEraseLabel(EditCellUse->cu_def, &editRect, &mask);
     }
     SelectClear();
@@ -1197,7 +1202,7 @@ CmdPort(w, cmd)
 	    case PORT_REMOVE:
 	        lab->lab_flags = 0;
 		DBWLabelChanged(editDef, lab->lab_text, &editBox,
-			lab->lab_pos, DBW_ALLWINDOWS);
+			lab->lab_just, DBW_ALLWINDOWS);
 		editDef->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
 		break;
 
@@ -1276,7 +1281,7 @@ parsepositions:
     {
 	/* Get position from label */
 
-	pos = lab->lab_pos;
+	pos = lab->lab_just;
         switch (pos)
 	{
 	    case GEO_NORTH:
@@ -1339,7 +1344,7 @@ parsepositions:
     lab->lab_flags &= ~PORT_DIR_MASK;
     lab->lab_flags |= dirmask;
 
-    DBWLabelChanged(editDef, lab->lab_text, &editBox, lab->lab_pos, DBW_ALLWINDOWS);
+    DBWLabelChanged(editDef, lab->lab_text, &editBox, lab->lab_just, DBW_ALLWINDOWS);
     editDef->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
 }
 
