@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdRS.c,v 1.8 2007/03/30 21:35:40 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdRS.c,v 1.9 2007/10/21 10:20:08 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -411,6 +411,8 @@ CmdSee(w, cmd)
 
     if (TTMaskHasType(&mask, L_LABEL))
 	flags |= DBW_SEELABELS;
+    if (TTMaskHasType(&mask, L_CELL))
+	flags |= DBW_SEECELLS;
     TTMaskClearType(&mask, L_LABEL);
     TTMaskClearType(&mask, L_CELL);
     TTMaskClearType(&mask, TT_SPACE);
