@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdTZ.c,v 1.8 2007/08/23 21:22:53 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdTZ.c,v 1.18 2008/02/07 06:22:44 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -56,6 +56,49 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 
 extern void DisplayWindow();
 
+/* Trivial function that returns 1 if called */
+
+int
+existFunc(tile)
+    Tile *tile;
+{
+    return 1;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * checkForPaintFunc---
+ *
+ * Simple routine that checks to see if a cell has any paint in it.
+ *
+ * Results:
+ *	Returns 1 on the first non-space tile found.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+checkForPaintFunc(cellDef, arg)
+    CellDef *cellDef;
+    ClientData arg;
+{
+    int numPlanes = *((int *)arg);
+    int pNum, result;
+   
+    if (cellDef->cd_flags & CDINTERNAL) return 0;
+
+    for (pNum = PL_SELECTBASE; pNum < numPlanes; pNum++)
+    {
+	if (DBSrPaintArea((Tile *)NULL, cellDef->cd_planes[pNum],
+		&TiPlaneRect, &DBAllButSpaceAndDRCBits,
+		existFunc, NULL))
+	    return 1;
+    }
+
+    return 0;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -116,7 +159,8 @@ CmdTech(w, cmd)
 	"version		Show current technology version",
 	"lambda			Show internal units per lambda", 
 	"planes			Show defined planes",
-	"layers			Show defined layers",
+	"layers	[lock|unlock <layer...>]\n\
+				Show (or lock and unlock) defined layers",
 	"drc <rule> <layer...>	Query DRC ruleset",
 	NULL
     };
@@ -201,15 +245,70 @@ CmdTech(w, cmd)
 #endif
 
 	case TECH_LAYERS:
-	    if (cmd->tx_argc == 3)
+	    if (cmd->tx_argc == 4)
+	    {
+		TileTypeBitMask lockmask;
+		TileType ctype;
+
+		if (!strcmp(cmd->tx_argv[3], "*"))
+		    TTMaskSetMask(&lockmask, &DBUserLayerBits);
+		else
+		    DBTechNoisyNameMask(cmd->tx_argv[3], &lockmask);
+		if (!strcmp(cmd->tx_argv[2], "lock"))
+		{
+		    TTMaskClearMask(&DBActiveLayerBits, &lockmask);
+		    for (ctype = TT_TECHDEPBASE; ctype < DBNumUserLayers; ctype++)
+			if (TTMaskHasType(&lockmask, ctype))
+			    if (DBIsContact(ctype))
+				DBLockContact(ctype);
+		    for (ctype = DBNumUserLayers; ctype < DBNumTypes; ctype++)
+			if (TTMaskIntersect(&lockmask, DBResidueMask(ctype)))
+			{
+			    TTMaskClearType(&DBActiveLayerBits, ctype);
+			    DBLockContact(ctype);
+			}
+		}
+		else if (!strcmp(cmd->tx_argv[2], "unlock"))
+		{
+		    TTMaskSetMask(&DBActiveLayerBits, &lockmask);
+		    for (ctype = TT_TECHDEPBASE; ctype < DBNumUserLayers; ctype++)
+			if (TTMaskHasType(&lockmask, ctype))
+			    if (DBIsContact(ctype))
+				DBUnlockContact(ctype);
+
+		    for (ctype = DBNumUserLayers; ctype < DBNumTypes; ctype++)
+		    {
+			TileTypeBitMask testmask;
+			TTMaskAndMask3(&testmask, &DBActiveLayerBits,
+					DBResidueMask(ctype));
+			if (TTMaskEqual(&testmask, DBResidueMask(ctype)))
+			{
+			    TTMaskSetType(&DBActiveLayerBits, ctype);
+			    DBUnlockContact(ctype);
+			}
+		    }
+		    TTMaskAndMask(&DBActiveLayerBits, &DBAllButSpaceAndDRCBits);
+		}
+		else
+		    TxError("Unknown operation %s; use lock or unlock\n",
+				cmd->tx_argv[2]);
+	    }
+	    else if (cmd->tx_argc == 3)
 	    {
 		if (!strcmp(cmd->tx_argv[2], "*"))
-		    DBTechPrintTypes(TRUE);
+		    DBTechPrintTypes(&DBAllButSpaceAndDRCBits, TRUE);
+		else if (!strcmp(cmd->tx_argv[2], "locked"))
+		{
+		    TileTypeBitMask lockedLayers;
+		    TTMaskCom2(&lockedLayers, &DBActiveLayerBits);
+		    TTMaskAndMask(&lockedLayers, &DBAllButSpaceAndDRCBits);
+		    DBTechPrintTypes(&lockedLayers, TRUE);
+		}
 		else
 		    DBTechPrintCanonicalType(cmd->tx_argv[2]);
 	    }
 	    else if (cmd->tx_argc == 2)
-		DBTechPrintTypes(FALSE);
+		DBTechPrintTypes(&DBAllButSpaceAndDRCBits, FALSE);
 	    else
 		goto wrongNumArgs;
 	    break;
@@ -315,13 +414,16 @@ CmdTech(w, cmd)
 	    /* all tech stuff set up such that multiple technologies	*/
 	    /* can be present at the same time.				*/
 
-	    /* For now: warn without checking */
+	    /* For now: check and warn, but do nothing to the layout */
 
 	    if (!noprompt)
 	    {
-		action = TxDialog("Technology file (re)loading may invalidate"
-			" any existing layout.  Continue? ", actionNames, 0);
-		if (action == 0) return;
+		if (DBCellSrDefs(0, checkForPaintFunc, (ClientData)&DBNumPlanes))
+		{
+		    action = TxDialog("Technology file (re)loading may invalidate"
+				" the existing layout.  Continue? ", actionNames, 0);
+		    if (action == 0) return;
+		}
 	    }
 	
 	    /* Changing number of planes requires handling on every	*/
@@ -389,12 +491,15 @@ CmdTech(w, cmd)
 #endif
 	    PlowAfterTech();
 
-	    if (saveNumPlanes != DBNumPlanes)
+	    if (DBCellSrDefs(0, checkForPaintFunc, (ClientData)&saveNumPlanes))
 	    {
-		TxError("Warning:  Number of planes has changed.  Any"
-			" existing layout may be invalid.\n");
-		(void) DBCellSrDefs(0, changePlanesFunc, (ClientData) &saveNumPlanes);
+		if (saveNumPlanes != DBNumPlanes)
+		    TxError("Warning:  Number of planes has changed.  ");
+		TxError("Existing layout may be invalid.\n");
 	    }
+	    if (saveNumPlanes != DBNumPlanes)
+		DBCellSrDefs(0, changePlanesFunc, (ClientData) &saveNumPlanes);
+
 	    break;
 
 	case TECH_HELP:
@@ -470,7 +575,6 @@ changePlanesFunc(cellDef, arg)
     return 0;
 }
 
-
 /*
  * ----------------------------------------------------------------------------
  *
@@ -643,7 +747,55 @@ CmdUpsidedown(w, cmd)
 
     return;
 }
-
+
+/*
+ * ----------------------------------------------------------------------------
+ * cmdWhatPrintCell --
+ *
+ * This command is used to print the cell id of the types encountered in the
+ * what command. For each type it only prints the name once.  The routine
+ * must be initialized by calling it with NULL args everytime the tile type
+ * changes and we start a new search.  (BSI)
+ *
+ * Results:
+ *	Returns 0 to keep the search going.
+ *
+ * Side Effects:
+ *	Prints cell names to the terminal.
+ * ----------------------------------------------------------------------------
+ */
+
+int cmdWhatPrintCell(tile, cxp)
+   Tile *tile;
+   TreeContext *cxp;
+{
+    char *CurrCellName;
+    char **whatCell = (char **)cxp->tc_filter->tf_arg;
+
+    // Use the id of the cell or its name if top cell
+
+    CurrCellName = cxp->tc_scx->scx_use->cu_id;
+    if ((CurrCellName == NULL) || (CurrCellName[0] == '\0'))
+	CurrCellName = cxp->tc_scx->scx_use->cu_def->cd_name;
+
+    if (*whatCell != CurrCellName)
+    {
+	TxPrintf(" %s ", CurrCellName);
+	*whatCell = CurrCellName;
+    }
+    return 0;
+}
+
+typedef struct labelstore
+{
+    TileType lab_type;
+    char *lab_text;
+    char *cell_name;
+} LabelStore;
+
+static int moreLabelEntries, labelEntryCount;
+static LabelStore *labelBlockTop, *labelEntry;
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -665,15 +817,6 @@ CmdUpsidedown(w, cmd)
  *
  * ----------------------------------------------------------------------------
  */
-typedef struct labelstore
-{
-    TileType lab_type;
-    char *lab_text;
-    char *cell_name;
-} LabelStore;
-
-static int moreLabelEntries, labelEntryCount;
-static LabelStore *labelBlockTop, *labelEntry;
 
 void
 CmdWhat(w, cmd)
@@ -683,7 +826,10 @@ CmdWhat(w, cmd)
     int i, locargc;
     bool foundAny;
     bool doList = FALSE;
-    TileTypeBitMask layers;
+    TileTypeBitMask layers, maskBits;
+    SearchContext scx;
+    CellUse *CheckUse;
+    char *whatCell;
 
 #ifdef MAGIC_WRAPPER
     Tcl_Obj *lobj, *paintobj, *labelobj, *cellobj;
@@ -751,11 +897,42 @@ CmdWhat(w, cmd)
 	else
 	{
 #endif
-	    TxPrintf("Selected mask layers:\n");
-	    for (i = TT_SELECTBASE; i < DBNumUserLayers; i++)
+	    CheckUse = NULL;
+	    if (EditRootDef == SelectRootDef)
+		CheckUse = EditCellUse;
+	    if (CheckUse == NULL)
 	    {
-		if (TTMaskHasType(&layers, i))
-		    TxPrintf("    %s\n", DBTypeLongName(i));
+		if (w == (MagWindow *)NULL)
+		    windCheckOnlyWindow(&w, DBWclientID);
+		if (w) CheckUse = (CellUse *)w->w_surfaceID;
+	    }
+	    if ((CheckUse != NULL) && (CheckUse->cu_def == SelectRootDef))
+	    {
+		scx.scx_use = CheckUse;
+		scx.scx_area = SelectUse->cu_bbox;	// BSI
+		scx.scx_trans = GeoIdentityTransform;	// BSI
+
+		TxPrintf("Selected mask layers:\n");
+		for (i = TT_SELECTBASE; i < DBNumUserLayers; i++)
+		{
+		    if (TTMaskHasType(&layers, i))
+		    {
+			whatCell = NULL;
+			TxPrintf("    %-8s (", DBTypeLongName(i));
+			TTMaskSetOnlyType(&maskBits, i);
+			if (DBIsContact(i)) DBMaskAddStacking(&maskBits);
+			DBTreeSrTiles(&scx, &maskBits, 0, cmdWhatPrintCell,
+				(ClientData)&whatCell);
+			TxPrintf(")\n");
+		    }
+		}
+	    }
+	    else
+	    {
+		TxPrintf("Selected mask layers:\n");
+		for (i = TT_SELECTBASE; i < DBNumUserLayers; i++)
+		    if (TTMaskHasType(&layers, i))
+			TxPrintf("    %s\n", DBTypeLongName(i));
 	    }
 #ifdef MAGIC_WRAPPER
 	}
@@ -895,7 +1072,7 @@ cmdWhatLabelPreFunc(label, cellUse, transform, foundAny)
     labelEntry->lab_text = label->lab_text;
     if (!cellUse->cu_id)
 	labelEntry->cell_name = NULL;
-    else if (!strcmp(cellDef->cd_name, EditRootDef->cd_name))
+    else if (EditRootDef && (!strcmp(cellDef->cd_name, EditRootDef->cd_name)))
 	/* This is a hack to get around an apparently bogus cellUse entry. . .*/
 	labelEntry->cell_name = NULL;
     else
@@ -930,7 +1107,12 @@ cmdWhatLabelFunc(entry, foundAny)
     if (entry->cell_name == NULL)
     {
 	isDef = TRUE;
-	entry->cell_name = EditRootDef->cd_name;
+	if (SelectRootDef)
+	    entry->cell_name = SelectRootDef->cd_name;
+	else if (EditRootDef)
+	    entry->cell_name = EditRootDef->cd_name;
+	else
+	    entry->cell_name = "(unknown)";
     }
 
     if ((last_name && (strcmp(entry->lab_text, last_name) !=0 ||
