@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBcellcopy.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBcellcopy.c,v 1.7 2008/01/29 02:41:57 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -48,11 +48,7 @@ static PaintResultType (*dbCurPaintTbl)[NT][NT] = DBPaintResultTbl;
  * such as the design-rule checker that need to use, for example,
  * DBPaintPlaneMergeOnce instead of the standard version.
  */
-#ifdef NONMANHATTAN
-static void (*dbCurPaintPlane)() = DBNMPaintPlane;
-#else
-static void (*dbCurPaintPlane)() = DBPaintPlane;
-#endif
+static void (*dbCurPaintPlane)() = DBPaintPlaneWrapper;
 
     /* Structure passed to DBTreeSrTiles() */
 struct copyAllArg
@@ -75,7 +71,7 @@ struct copyArg
     Transform		*ca_trans;	/* Transform to target */
 };
 
-    /* Structure passed to DBTreeSrLabels to hold informatio about
+    /* Structure passed to DBTreeSrLabels to hold information about
      * copying labels.
      */
 
@@ -88,6 +84,90 @@ struct copyLabelArg
 					 */
 };
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBPaintPlaneWrapper --
+ *
+ *    Simple wrapper to DBPaintPlane.  Argument "type" is ignored.
+ *    Note that this function is passed as a pointer on occasion, so
+ *    it cannot be replaced with a macro!
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DBPaintPlaneWrapper(def, pNum, type, area, undo)
+    CellDef *def;
+    int pNum;
+    TileType type;
+    Rect *area;
+    PaintUndoInfo *undo;
+{
+#ifdef NONMANHATTAN
+    TileType loctype = type & TT_LEFTMASK;
+#else
+    TileType loctype = type;
+#endif
+
+    undo->pu_pNum = pNum;
+#ifdef NONMANHATTAN
+    DBNMPaintPlane(def->cd_planes[pNum], type, area,
+		dbCurPaintTbl[pNum][loctype], undo);
+#else
+    DBPaintPlane(def->cd_planes[pNum], area,
+		dbCurPaintTbl[pNum][loctype], undo);
+#endif
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBPaintPlaneActive ---
+ *
+ *    This function calls DBPaintPlane, but first checks if the type
+ *    being painted is an active layer.  If the type is a contact,
+ *    then the residues are checked to see if they are active layers. 
+ *    Painting proceeds accordingly.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DBPaintPlaneActive(def, pNum, type, area, undo)
+    CellDef *def;
+    int pNum;
+    TileType type;
+    Rect *area;
+    PaintUndoInfo *undo;
+{
+#ifdef NONMANHATTAN
+    TileType loctype = type & TT_LEFTMASK;
+#else
+    TileType loctype = type;
+#endif
+    TileType t;
+
+    if (DBIsContact(loctype))
+    {
+	TileTypeBitMask tmask;
+
+	TTMaskAndMask3(&tmask, DBResidueMask(loctype), &DBActiveLayerBits);
+	if (!TTMaskEqual(&tmask, DBResidueMask(loctype)))
+	{
+	    if (!TTMaskIsZero(&tmask))
+		for (t = TT_TECHDEPBASE; t < DBNumUserLayers; t++)
+		    if (TTMaskHasType(&tmask, t))
+			DBPaintPlaneWrapper(def, pNum, t | (type &
+				(TT_SIDE | TT_DIRECTION | TT_DIAGONAL)),
+				area, undo);
+	    return;
+	}
+    }
+    if (TTMaskHasType(&DBActiveLayerBits, loctype))
+	DBPaintPlaneWrapper(def, pNum, type, area, undo);
+}
+
 #ifdef NONMANHATTAN
 /*
  *-----------------------------------------------------------------------------
@@ -158,6 +238,7 @@ DBCellCopyAllPaint(scx, mask, xMask, targetUse)
     int xMask;			/* Expansion state mask to be used in search */
     CellUse *targetUse;		/* Cell into which material is to be stuffed */
 {
+    TileTypeBitMask locMask;
     struct copyAllArg arg;
     int dbCopyAllPaint();
 
@@ -165,7 +246,11 @@ DBCellCopyAllPaint(scx, mask, xMask, targetUse)
     arg.caa_targetUse = targetUse;
     GEOTRANSRECT(&scx->scx_trans, &scx->scx_area, &arg.caa_rect);
 
-    (void) DBTreeSrTiles(scx, mask, xMask, dbCopyAllPaint, (ClientData) &arg);
+    /* Add any stacking types for the search (but not to mask passed as arg!) */
+    locMask = *mask;
+    DBMaskAddStacking(&locMask);
+
+    (void) DBTreeSrTiles(scx, &locMask, xMask, dbCopyAllPaint, (ClientData) &arg);
 }
 
 /*
@@ -237,7 +322,7 @@ dbCopyAllLabels(scx, lab, tpath, arg)
     def = arg->cla_targetUse->cu_def;
     if (!GEO_LABEL_IN_AREA(&lab->lab_rect, &(scx->scx_area))) return 0;
     GeoTransRect(&scx->scx_trans, &lab->lab_rect, &labTargetRect);
-    targetPos = GeoTransPos(&scx->scx_trans, lab->lab_pos);
+    targetPos = GeoTransPos(&scx->scx_trans, lab->lab_just);
 
     /* Eliminate duplicate labels.  Don't pay any attention to layers
      * in deciding on duplicates:  if text and position match, it's a
@@ -371,7 +456,7 @@ DBCellCopyLabels(scx, mask, xMask, targetUse, pArea)
 		|| TTMaskHasType(mask, L_LABEL)))
 	{
 	    GeoTransRect(&scx->scx_trans, &lab->lab_rect, &labTargetRect);
-	    targetPos = GeoTransPos(&scx->scx_trans, lab->lab_pos);
+	    targetPos = GeoTransPos(&scx->scx_trans, lab->lab_just);
 
 	    /* Eliminate duplicate labels.  Don't pay any attention to
 	     * type when deciding on duplicates, since types can change
@@ -404,7 +489,7 @@ dbCopyManhattanPaint(tile, cxp)
     PaintUndoInfo ui;
     CellDef *def;
     TileType type;
-    int pNum;
+    int pNum = cxp->tc_plane;
 
     /*
      * Don't copy space tiles -- this copy is additive.
@@ -431,13 +516,7 @@ dbCopyManhattanPaint(tile, cxp)
     /* Clip against the target area */
     GEOCLIP(&targetRect, &arg->caa_rect);
 
-    for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
-	if (DBPaintOnPlane(type, pNum))
-	{
-	    ui.pu_pNum = pNum;
-	    (*dbCurPaintPlane)(def->cd_planes[pNum], type,
-			&targetRect, dbCurPaintTbl[pNum][type], &ui);
-	}
+    (*dbCurPaintPlane)(def, pNum, type, &targetRect, &ui);
     return (0);
 }
 #endif
@@ -458,7 +537,8 @@ dbCopyAllPaint(tile, cxp)
     PaintUndoInfo ui;
     CellDef *def;
     TileType type = TiGetTypeExact(tile);
-    int pNum;
+    int pNum = cxp->tc_plane;
+    TileTypeBitMask *typeMask;
 
     /*
      * Don't copy space tiles -- this copy is additive.
@@ -484,6 +564,37 @@ dbCopyAllPaint(tile, cxp)
 	return 0;
 
     arg = (struct copyAllArg *) cxp->tc_filter->tf_arg;
+    typeMask = arg->caa_mask;
+
+    /* Resolve what type we're going to paint, based on the type and mask */
+    if (!TTMaskHasType(typeMask, type))
+    {
+	TileTypeBitMask rMask;
+
+	/* Simple case---typeMask has a residue of type on pNum */
+	TTMaskAndMask3(&rMask, typeMask, DBResidueMask(type));
+	TTMaskAndMask(&rMask, &DBPlaneTypes[pNum]);
+	if (!TTMaskIsZero(&rMask))
+	{
+	    for (type = TT_TECHDEPBASE; type < DBNumUserLayers; type++)
+		if (TTMaskHasType(&rMask, type))
+		    break;
+	    if (type == DBNumUserLayers) return 0;	/* shouldn't happen */
+
+	    /* Hopefully there's always just one type here---sanity check */
+	    TTMaskClearType(&rMask, type);
+	    if (!TTMaskIsZero(&rMask))
+	    {
+		/* Diagnostic */
+		TxError("Bad assumption:  Multiple types to paint!  Fix me!\n");
+	    }
+	}
+	else
+	{
+	    type = DBPlaneToResidue(type, pNum);
+	    if (!TTMaskHasType(typeMask, type)) return 0;
+	}
+    }
 
     /* Construct the rect for the tile in source coordinates */
     TITORECT(tile, &sourceRect);
@@ -562,13 +673,7 @@ dbCopyAllPaint(tile, cxp)
 		else
 		    goto topbottom;
 
-		for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
-		    if (DBPaintOnPlane(type, pNum))
-		    {
-			ui.pu_pNum = pNum;
-			(*dbCurPaintPlane)(def->cd_planes[pNum], type,
-				&rrect, dbCurPaintTbl[pNum][type], &ui);
-		    }
+		(*dbCurPaintPlane)(def, pNum, type, &rrect, &ui);
 
 topbottom:
 		/* Rectangle to top or bottom */
@@ -587,13 +692,7 @@ topbottom:
 		else
 		    goto splitdone;
 
-		for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
-		    if (DBPaintOnPlane(type, pNum))
-		    {
-			ui.pu_pNum = pNum;
-			(*dbCurPaintPlane)(def->cd_planes[pNum], type,
-				&rrect, dbCurPaintTbl[pNum][type], &ui);
-		    }
+		(*dbCurPaintPlane)(def, pNum, type, &rrect, &ui);
 	    }
 	}
     }
@@ -605,19 +704,7 @@ topbottom:
 
 splitdone:
 
-    /* Should this just be replaced with DBPaint()? */
-    for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
-	if (DBPaintOnPlane(type, pNum))
-	{
-	    ui.pu_pNum = pNum;
-#ifdef NONMANHATTAN
-	    (*dbCurPaintPlane)(def->cd_planes[pNum], dinfo,
-			&targetRect, dbCurPaintTbl[pNum][type], &ui);
-#else
-	    (*dbCurPaintPlane)(def->cd_planes[pNum], &targetRect,
-			dbCurPaintTbl[pNum][type], &ui);
-#endif
-	}
+    (*dbCurPaintPlane)(def, pNum, dinfo | type, &targetRect, &ui);
 
     return (0);
 }
@@ -789,6 +876,7 @@ dbCellCopyCellsFunc(scx, arg)
 	(void) DBLinkCell(newUse, arg->caa_targetUse->cu_def);
     }
     newUse->cu_expandMask = use->cu_expandMask;
+    newUse->cu_flags = use->cu_flags;
 
     /* The translation stuff is funny, since we got one element of
      * the array, but not necessarily the lower-left element.  To
