@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBcellname.c,v 1.4 2007/04/20 17:56:59 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBcellname.c,v 1.6 2008/01/04 20:34:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -1046,7 +1046,115 @@ dbCellUsePrintFunc(cellUse, dolist)
     return 0;
 }
 
-
+/*
+ *  dbLockUseFunc()
+ */
+
+int
+dbLockUseFunc(selUse, use, transform, data)
+    CellUse *selUse;	/* Use from selection cell */
+    CellUse *use;	/* Use from layout corresponding to selection */
+    Transform *transform;
+    ClientData data;
+{
+    bool dolock = *((bool *)data);
+
+    if (!DBIsChild(use, EditCellUse))
+    {
+	TxError("Cell %s (%s) isn't a child of the edit cell.\n",
+		use->cu_id, use->cu_def->cd_name);
+	return 0;
+    }
+    if ((dolock && (use->cu_flags & CU_LOCKED)) ||
+		(!dolock && !(use->cu_flags & CU_LOCKED)))
+	return 0;	/* nothing to do */
+
+    if (UndoIsEnabled()) DBUndoCellUse(use, UNDO_CELL_LOCKDOWN);
+    if (dolock) use->cu_flags |= CU_LOCKED;
+    else use->cu_flags &= ~CU_LOCKED;
+    if (UndoIsEnabled()) DBUndoCellUse(use, UNDO_CELL_LOCKDOWN);
+
+    if (selUse != NULL)
+    {
+	if (dolock) selUse->cu_flags |= CU_LOCKED;
+	else selUse->cu_flags &= ~CU_LOCKED;
+    }
+
+    DBWAreaChanged(use->cu_parent, &use->cu_bbox,
+	(int) ~(use->cu_expandMask), &DBAllButSpaceBits);
+    DBWHLRedraw(EditRootDef, &selUse->cu_bbox, TRUE);
+    return 0;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBLockUse --
+ *
+ * 	This routine sets or clears a cell instance's lock flag
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	cu_flags changed for indicated cell use.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DBLockUse(UseName, bval)
+    char *UseName;
+    bool bval;
+{
+    int found;
+    HashSearch hs;
+    HashEntry *entry;
+    CellDef *celldef;
+    CellUse *celluse;
+
+    int dbUseLockFunc();
+    
+   /*
+    *
+    * Check to see if a cell name was specified. If not,
+    * search for selected cells.
+    *
+    */
+
+    if (UseName == NULL)
+    {
+	SelEnumCells(TRUE, (int *)NULL, (SearchContext *)NULL,
+		dbLockUseFunc, (ClientData)&bval);
+    }
+    else
+    {
+	SearchContext scx;
+
+	bzero(&scx, sizeof(SearchContext));
+	found = 0;
+
+	HashStartSearch(&hs);
+	while( (entry = HashNext(&dbCellDefTable, &hs)) != NULL)
+	{
+	    celldef = (CellDef *) HashGetValue(entry);
+	    if (celldef != (CellDef *) NULL)
+	    {
+		celluse = celldef->cd_parents;   /* only need one */
+		if (celluse != (CellUse *)NULL) {
+		    DBTreeFindUse(UseName, celluse, &scx);
+		    if (scx.scx_use != NULL) break;
+		}
+	    }
+	}
+
+	if (scx.scx_use == NULL)
+	    TxError("Cell %s is not currently loaded.\n", UseName);
+	else
+	    dbLockUseFunc(NULL, scx.scx_use, NULL, (ClientData)&bval);
+    }
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1218,6 +1326,7 @@ DBCellNewUse(cellDef, useName)
 
     cellUse = (CellUse *) mallocMagic((unsigned) (sizeof (CellUse)));
     cellUse->cu_id = StrDup((char **) NULL, useName);
+    cellUse->cu_flags = (unsigned char)0;
     cellUse->cu_expandMask = 0;
     cellUse->cu_transform = GeoIdentityTransform;
     cellUse->cu_def = cellDef;
