@@ -20,7 +20,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBconnect.c,v 1.2 2007/04/20 17:57:00 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBconnect.c,v 1.3 2008/01/29 02:41:57 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -565,7 +565,7 @@ dbcConnectLabelFunc(scx, lab, tpath, csa2)
     int dbcConnectFunc();		/* Forward declaration */
 
     GeoTransRect(&scx->scx_trans, &lab->lab_rect, &r);
-    pos = GeoTransPos(&scx->scx_trans, lab->lab_pos);
+    pos = GeoTransPos(&scx->scx_trans, lab->lab_just);
 
     DBEraseLabelsByContent(def, &r, pos, -1, lab->lab_text);
     DBPutLabel(def, &r, pos, lab->lab_text, lab->lab_type, lab->lab_flags);
