@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBio.c,v 1.7 2006/11/02 23:44:07 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBio.c,v 1.13 2008/02/03 21:23:56 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -54,6 +54,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 #include "utils/hash.h"
 #include "database/database.h"
 #include "database/databaseInt.h"
+#include "database/fonts.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
 #include "utils/tech.h"
@@ -225,9 +226,12 @@ file_is_not_writeable(name)
  *	7. If the cell contains labels, then the labels are preceded
  *	by the line "<< labels >>".  Each label is one line of the form
  *		rlabel <layer> <xbot> <ybot> <xtop> <ytop> <position> <text>
- *		or
+ *		 or
+ *		flabel <layer> <xbot> <ybot> <xtop> <ytop> <position> <fontname>
+ *			<size> <rotation> <offsetx> <offsety> <text>
+ *		 or
  *		label <layer> <x> <y> <position> <text>
- *	(the second form is obsolete and is for point labels only).
+ *	(the last form is obsolete and is for point labels only).
  *
  *	Ports are declared in the label line after the label they refer
  *	to.  The syntax is
@@ -1050,6 +1054,9 @@ dbReadOpen(cellDef, name, setFileName)
 		TxPrintf("Warning: cell <%s> from file %s is not writeable\n",
 			cellDef->cd_name, filename);
 	}
+	else
+	    cellDef->cd_flags &= ~CDNOEDIT;
+
 	if (is_locked == FALSE)
 	    cellDef->cd_fd = fileno(f);
 	cellDef->cd_flags &= ~CDNOTFOUND;
@@ -1105,6 +1112,7 @@ dbReadUse(cellDef, line, len, f, scalen, scaled)
     CellDef *subCellDef;
     Transform t;
     Rect r;
+    bool locked;
 
     if (strncmp(line, "use", 3) != 0)
     {
@@ -1119,6 +1127,8 @@ dbReadUse(cellDef, line, len, f, scalen, scaled)
 	return (FALSE);
     }
 
+    locked = (useid[0] == CULOCKCHAR) ? TRUE : FALSE;
+
     if (dbFgets(line, len, f) == NULL)
 	return (FALSE);
 
@@ -1257,8 +1267,11 @@ badTransform:
 	goto nextLine;
     }
 
-    subCellUse = DBCellNewUse(subCellDef, useid[0] ? useid : (char *) NULL);
+    
+    subCellUse = DBCellNewUse(subCellDef, (useid[0]) ?
+		((locked) ? useid + 1 : useid) : (char *) NULL);
 
+    if (locked) subCellUse->cu_flags |= CU_LOCKED;
     /*
      * Instead of calling DBLinkCell for each cell, DBGenerateUniqueIds()
      * gets called for the entire cell at the end.
@@ -1560,7 +1573,7 @@ nextelement:
  * up until the end of a label section.  On exit, 'line' contains
  * the line that terminated the label section, which will be either
  * a line of the form "<< something >>" or one beginning with a
- * character other than 'r', 'l', or 'p'.
+ * character other than 'r', 'l', 'f', or 'p'.
  *
  * Results:
  *	Returns TRUE normally, or FALSE on error or EOF.
@@ -1605,7 +1618,7 @@ dbReadLabels(cellDef, line, len, f, scalen, scaled)
 	 */
 	if (line[0] == 'r')
 	{
-	    if (sscanf(line, "rlabel %1023s %d %d %d %d %d %99[^\n]",
+	    if (sscanf(line, "rlabel %49s %d %d %d %d %d %99[^\n]",
 		    layername, &r.r_xbot, &r.r_ybot, &r.r_xtop, &r.r_ytop,
 			       &orient, text) != 7)
 	    {
@@ -1758,7 +1771,7 @@ dbReadLabels(cellDef, line, len, f, scalen, scaled)
 		if (TTMaskHasType(rmask, rtype))
 		    type = rtype;
 	}
-	(void) DBPutLabel(cellDef, &r, orient, text, type, 0);
+	DBPutLabel(cellDef, &r, orient, text, type, 0);
 
 nextlabel:
 	if (dbFgets(line, len, f) == NULL)
@@ -2076,12 +2089,12 @@ DBCellWriteFile(cellDef, f)
 	for (lab = cellDef->cd_labels; lab; lab = lab->lab_next)
 	{
 	    sprintf(lstring, "rlabel %s %d %d %d %d %d %s\n",
-		    DBTypeLongName(lab->lab_type),
-		    lab->lab_rect.r_xbot / reducer,
-		    lab->lab_rect.r_ybot / reducer,
-		    lab->lab_rect.r_xtop / reducer,
-		    lab->lab_rect.r_ytop / reducer,
-		    lab->lab_pos, lab->lab_text);
+			DBTypeLongName(lab->lab_type),
+			lab->lab_rect.r_xbot / reducer,
+			lab->lab_rect.r_ybot / reducer,
+			lab->lab_rect.r_xtop / reducer,
+			lab->lab_rect.r_ytop / reducer,
+			lab->lab_just, lab->lab_text);
 	    FPRINTF(f, lstring);
 	    if (lab->lab_flags & PORT_DIR_MASK)
 	    {
@@ -2333,7 +2346,6 @@ DBCellWrite(cellDef, fileName)
 
     if (exists)
     {
-
 	/*
 	 * Determine unique name for a temp file to write.
 	 */
@@ -2341,16 +2353,26 @@ DBCellWrite(cellDef, fileName)
 		+ strlen(template) + 1));
 	(void) sprintf(tmpname, "%s%s", expandname, template);
 	tmpres = mkstemp(tmpname);
-	if (tmpres != -1) close(tmpres);	/* We'll re-open it as a stream, below */
+	if (tmpres != -1)
+	{
+	    /* Assert the file permissions of the original file */
+	    /* This will prevent the overwriting of a file that	*/
+	    /* has write permissions blocked.			*/
+
+	    if (stat(expandname, &statb) == 0)
+		fchmod(tmpres, statb.st_mode & 0777);
+	    close(tmpres);	/* We'll re-open it as a stream, below */
+	}
 
 	/* Critical: disable interrupts while we do our work */
 	SigDisableInterrupts();
 
-	/* Don't allow a write if the file isn't writeable, or if mkstemp returned	*/
-	/* an error condition.							*/
+	/* Don't allow a write if the file isn't writeable, or if	*/
+	/* mkstemp() returned an error condition.			*/
 
 	if (file_is_not_writeable(expandname))
 	{
+	    if (tmpres != -1) unlink(tmpname);
 	    perror(expandname);
 	    goto cleanup;
 	}
@@ -2367,18 +2389,6 @@ DBCellWrite(cellDef, fileName)
      */
     if (tmpf = fopen(tmpname, "w"))
     {
-	/* mkstemp() generates a file with permissions 0600.   	*/
-	/* We want instead to respect the user's umask value	*/
-	/* so that the final and original .mag files have the	*/
-	/* same permissions.					*/
-
-	if (exists)
-	{
-	    mode_t userumask = umask((mode_t)0);
-	    umask(userumask);
-	    fchmod(fileno(tmpf), (mode_t)0666 & ~userumask);
-	}
-
 	result = DBCellWriteFile(cellDef, tmpf);
 	(void) fclose(tmpf);
 	tmpf = NULL;
@@ -2626,8 +2636,10 @@ dbWriteCellFunc(cellUse, cdarg)
 
     t = &(cellUse->cu_transform);
     b = &(cellUse->cu_def->cd_bbox);
-    sprintf(cstring, "use %s %s\n", cellUse->cu_def->cd_name,cellUse->cu_id);
-    FPRINTR(arg->wa_file,cstring);
+    sprintf(cstring, "use %s %c%s\n", cellUse->cu_def->cd_name,
+		(cellUse->cu_flags & CU_LOCKED) ? CULOCKCHAR : ' ',
+		cellUse->cu_id);
+    FPRINTR(arg->wa_file, cstring);
 
     if ((cellUse->cu_xlo != cellUse->cu_xhi)
 	    || (cellUse->cu_ylo != cellUse->cu_yhi))
