@@ -17,11 +17,12 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBlabel.c,v 1.3 2006/12/06 23:03:34 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBlabel.c,v 1.9 2008/02/03 21:23:56 tim Exp $";
 #endif  /* not lint */
 
 #include <sys/types.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include "utils/magic.h"
@@ -29,7 +30,9 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
+#include "utils/utils.h"
 #include "database/database.h"
+#include "database/fonts.h"
 #include "database/databaseInt.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
@@ -38,6 +41,11 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 
 static TileType DBPickLabelLayer(/* CellDef *def, Label *lab, int noreconnect */);
 
+/* Globally-accessible font information */
+
+MagicFont **DBFontList = NULL;
+int DBNumFonts = 0;
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -92,7 +100,6 @@ DBIsSubcircuit(cellDef)
  *
  * ----------------------------------------------------------------------------
  */
-
 int
 DBPutLabel(cellDef, rect, align, text, type, flags)
     CellDef *cellDef;	/* Cell in which label is placed */
@@ -153,7 +160,7 @@ DBPutLabel(cellDef, rect, align, text, type, flags)
 	}
     }
 
-    lab->lab_pos = align;
+    lab->lab_just = align;
     lab->lab_type = type;
     lab->lab_flags = flags;
     lab->lab_rect = *rect;
@@ -238,10 +245,10 @@ DBEraseLabel(cellDef, area, mask)
 	else labPrev->lab_next = lab->lab_next;
 	if (cellDef->cd_lastLabel == lab)
 	    cellDef->cd_lastLabel = labPrev;
-	DBUndoEraseLabel(cellDef, &lab->lab_rect, lab->lab_pos,
+	DBUndoEraseLabel(cellDef, &lab->lab_rect, lab->lab_just,
 	    lab->lab_text, lab->lab_type, lab->lab_flags);
 	DBWLabelChanged(cellDef, lab->lab_text, &lab->lab_rect,
-	    lab->lab_pos, DBW_ALLWINDOWS);
+	    lab->lab_just, DBW_ALLWINDOWS);
 	freeMagic((char *) lab);
 	lab = lab->lab_next;
 	erasedAny = TRUE;
@@ -305,12 +312,12 @@ DBEraseLabelsByContent(def, rect, pos, type, text)
 	nextCheck:
 	if ((rect != NULL) && !(RECTEQUAL(&lab->lab_rect, rect))) continue;
 	if ((type >= 0) && (type != lab->lab_type)) continue;
-	if ((pos >= 0) && (pos != lab->lab_pos)) continue;
+	if ((pos >= 0) && (pos != lab->lab_just)) continue;
 	if ((text != NULL) && (strcmp(text, lab->lab_text) != 0)) continue;
-	DBUndoEraseLabel(def, &lab->lab_rect, lab->lab_pos,
+	DBUndoEraseLabel(def, &lab->lab_rect, lab->lab_just,
 	    lab->lab_text, lab->lab_type, lab->lab_flags);
 	DBWLabelChanged(def, lab->lab_text, &lab->lab_rect,
-	    lab->lab_pos, DBW_ALLWINDOWS);
+	    lab->lab_just, DBW_ALLWINDOWS);
 	if (labPrev == NULL)
 	    def->cd_labels = lab->lab_next;
 	else labPrev->lab_next = lab->lab_next;
@@ -370,10 +377,10 @@ DBEraseLabelsByFunction(def, func)
     {
 	nextCheck:
 	if (!(*func)(lab)) continue;
-	DBUndoEraseLabel(def, &lab->lab_rect, lab->lab_pos,
+	DBUndoEraseLabel(def, &lab->lab_rect, lab->lab_just,
 	    lab->lab_text, lab->lab_type, lab->lab_flags);
 	DBWLabelChanged(def, lab->lab_text, &lab->lab_rect,
-	    lab->lab_pos, DBW_ALLWINDOWS);
+	    lab->lab_just, DBW_ALLWINDOWS);
 	if (labPrev == NULL)
 	    def->cd_labels = lab->lab_next;
 	else labPrev->lab_next = lab->lab_next;
@@ -424,15 +431,15 @@ DBReOrientLabel(cellDef, area, newPos)
     {
 	if (GEO_TOUCH(area, &lab->lab_rect))
 	{
-	    DBUndoEraseLabel(cellDef, &lab->lab_rect, lab->lab_pos,
+	    DBUndoEraseLabel(cellDef, &lab->lab_rect, lab->lab_just,
 		lab->lab_text, lab->lab_type, lab->lab_flags);
 	    DBWLabelChanged(cellDef, lab->lab_text, &lab->lab_rect,
-		lab->lab_pos, DBW_ALLWINDOWS);
-	    lab->lab_pos = newPos;
-	    DBUndoPutLabel(cellDef, &lab->lab_rect, lab->lab_pos,
+		lab->lab_just, DBW_ALLWINDOWS);
+	    lab->lab_just = newPos;
+	    DBUndoPutLabel(cellDef, &lab->lab_rect, lab->lab_just,
 		lab->lab_text, lab->lab_type, lab->lab_flags);
 	    DBWLabelChanged(cellDef, lab->lab_text, &lab->lab_rect,
-		lab->lab_pos, DBW_ALLWINDOWS);
+		lab->lab_just, DBW_ALLWINDOWS);
 	}
     }
 }
@@ -482,10 +489,10 @@ DBAdjustLabels(def, area)
 		    lab->lab_text, DBTypeLongName(lab->lab_type),
 		    DBTypeLongName(newType), def->cd_name);
 	};
-	DBUndoEraseLabel(def, &lab->lab_rect, lab->lab_pos,
+	DBUndoEraseLabel(def, &lab->lab_rect, lab->lab_just,
 		lab->lab_text, lab->lab_type, lab->lab_flags);
 	lab->lab_type = newType;
-	DBUndoPutLabel(def, &lab->lab_rect, lab->lab_pos,
+	DBUndoPutLabel(def, &lab->lab_rect, lab->lab_just,
 		lab->lab_text, lab->lab_type, lab->lab_flags);
 	modified = TRUE;
     }
@@ -536,10 +543,10 @@ DBAdjustLabelsNew(def, area, noreconnect)
 			    labPrev->lab_next = lab->lab_next;
 		    if (def->cd_lastLabel == lab)
 			    def->cd_lastLabel = labPrev;
-		    DBUndoEraseLabel(def, &lab->lab_rect, lab->lab_pos,
+		    DBUndoEraseLabel(def, &lab->lab_rect, lab->lab_just,
 				     lab->lab_text, lab->lab_type, lab->lab_flags);
 		    DBWLabelChanged(def, lab->lab_text, &lab->lab_rect,
-				    lab->lab_pos, DBW_ALLWINDOWS);
+				    lab->lab_just, DBW_ALLWINDOWS);
 		    freeMagic((char *) lab);
 		    lab = lab->lab_next;
 		    modified = TRUE;
@@ -550,10 +557,10 @@ DBAdjustLabelsNew(def, area, noreconnect)
 				     lab->lab_text, DBTypeLongName(lab->lab_type),
 				     DBTypeLongName(newType), def->cd_name);
 		    }
-		    DBUndoEraseLabel(def, &lab->lab_rect, lab->lab_pos,
+		    DBUndoEraseLabel(def, &lab->lab_rect, lab->lab_just,
 				     lab->lab_text, lab->lab_type, lab->lab_flags);
 		    lab->lab_type = newType;
-		    DBUndoPutLabel(def, &lab->lab_rect, lab->lab_pos,
+		    DBUndoPutLabel(def, &lab->lab_rect, lab->lab_just,
 				   lab->lab_text, lab->lab_type, lab->lab_flags);
 		    modified = TRUE;
 	    }
@@ -862,3 +869,4 @@ dbPickFunc2(tile, mask)
     TTMaskSetType(&mask[2], type);
     return 0;
 }
+
