@@ -23,7 +23,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBtcontact.c,v 1.3 2007/03/03 19:22:41 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBtcontact.c,v 1.8 2008/01/30 06:26:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -46,6 +46,7 @@ TileTypeBitMask DBLayerTypeMaskTbl[NT];
 
 /* Filled in after contact types have been generated */
 TileTypeBitMask DBPlaneTypes[PL_MAXTYPES];
+TileTypeBitMask DBHomePlaneTypes[PL_MAXTYPES];
 PlaneMask        DBTypePlaneMaskTbl[NT];
 
 /* --------------------- Data local to this file ---------------------- */
@@ -435,6 +436,7 @@ dbTechAddOneStackedContact(type1, type2)
  *
  * ----------------------------------------------------------------------------
  */
+
 TileType
 DBPlaneToResidue(type, plane)
     TileType type;
@@ -461,6 +463,28 @@ DBPlaneToResidue(type, plane)
     return TT_SPACE;	/* no residue on plane */
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBMaskAddStacking ---
+ *
+ * A general-purpose routine to add stacked types containing types that are
+ * already in the mask.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DBMaskAddStacking(mask)
+    TileTypeBitMask *mask;
+{
+    TileType ttype;
+
+    for (ttype = DBNumUserLayers; ttype < DBNumTypes; ttype++)
+	if (TTMaskIntersect(DBResidueMask(ttype), mask))
+	    TTMaskSetType(mask, ttype);
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -694,62 +718,6 @@ DBTechFindStacking(type1, type2)
     return -1;
 }
 
-/*
- * ----------------------------------------------------------------------------
- *
- * DBHomePlaneTypes ---
- *
- *	Generate a mask of all the tile types whose home plane is the
- *	indicated plane pNum.  First, all types that exist in the plane
- *	are counted with DBPlaneMask[].  Then, types whose home plane
- *	are not pNum are removed from the mask.  Stacking contact types
- *	are removed only if none of their residue contact types have
- *	a home plane of pNum.
- *
- * Results:
- *	A mask of tile types whose home plane is pNum.
- *
- * Side effects:
- *	None.
- *
- * ----------------------------------------------------------------------------
- */
-
-TileTypeBitMask
-DBHomePlaneTypes(pNum)
-    int pNum;
-{
-    TileTypeBitMask uMask, *rMask;
-    int m, n;
-    PlaneMask p;
-    LayerInfo *lc, *lr;
-
-    TTMaskZero(&uMask);
-    TTMaskSetMask(&uMask, &DBPlaneTypes[pNum]);
-
-    for (m = 0; m < dbNumContacts; m++)
-    {
-	lc = dbContactInfo[m];
-	if (lc->l_type >= DBNumUserLayers)
-	{
-	    rMask = DBResidueMask(lc->l_type);
-	    p = 0;
-	    for (n = 0; n < dbNumContacts; n++)
-	    {
-		lr = dbContactInfo[n];
-		if (TTMaskHasType(rMask, lr->l_type))
-		    p |= PlaneNumToMaskBit(DBPlane(lr->l_type));
-	    }
-	    if (!PlaneMaskHasPlane(p, pNum))
-		TTMaskClearType(&uMask, lc->l_type);
-	}
-	else if (pNum != DBPlane(lc->l_type))
-	    TTMaskClearType(&uMask, lc->l_type);
-    }
-    return uMask;
-}
-
-
 /*
  * ----------------------------------------------------------------------------
  *
@@ -821,15 +789,23 @@ DBTechFinalContact()
 	    }
 	}
     }
+
+    /* Create a mask for which each type only appears on one plane.     */
+    /* This is useful for non-redundant searches.                       */
+
+    for (pNum = 0;  pNum < PL_MAXTYPES;  pNum++)
+	TTMaskZero(&DBHomePlaneTypes[pNum]);
+
+    for (primaryType = TT_SPACE + 1; primaryType < DBNumTypes; primaryType++)
+	TTMaskSetType(&DBHomePlaneTypes[DBPlane(primaryType)], primaryType);
 }
 
 /*
  * ----------------------------------------------------------------------------
  * DBTechTypesOnPlane --
  *
- * This is a substitute for DBTechSubsetLayers() that properly handles
- * stacked contacts.  Given a tile type bitmask and a plane index, 
- * check if all types in the bitmask have an image on the indicated plane.
+ * Given a tile type bitmask and a plane index, check if all types in the
+ * bitmask have an image on the indicated plane.
  *
  * Results:
  *	TRUE if all types in "src" have at least one image on plane.
@@ -978,7 +954,6 @@ DBFullResidueMask(type, rmask)
     }
 }
 
-
 /*
  * ----------------------------------------------------------------------------
  *
