@@ -20,7 +20,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBtechname.c,v 1.5 2006/11/06 21:36:32 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBtechname.c,v 1.11 2008/01/10 14:51:42 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -399,8 +399,9 @@ DBTechTypesToPlanes(mask)
  */
 
 void
-DBTechPrintTypes(dolist)
-    bool dolist;	/* return as a list and don't print aliases */
+DBTechPrintTypes(mask, dolist)
+    TileTypeBitMask *mask;  /* Print layers defined by this mask.	*/
+    bool dolist;	    /* return as a list and don't print aliases	*/
 {
     TileType i;
     NameList *p;
@@ -413,6 +414,7 @@ DBTechPrintTypes(dolist)
     /* List technology-dependent types */
     for (i = TT_TECHDEPBASE; i < DBNumUserLayers; i++)
     {
+	if (!TTMaskHasType(mask, i)) continue;
 	first = TRUE;
 	for (p = dbTypeNameLists.sn_next; p != &dbTypeNameLists;
 		p = p->sn_next)
@@ -447,6 +449,7 @@ DBTechPrintTypes(dolist)
     /* List built-in types that are normally painted by name */
     for (dtp = dbTechDefaultTypes; dtp->dt_names; dtp++)
     {
+	if (!TTMaskHasType(mask, dtp->dt_type)) continue;
 	if (dtp->dt_print)
 	{
 	    first = TRUE;
@@ -683,11 +686,19 @@ DBTechNoisyNameMask(layers, mask)
 		    if (t >= 0)
 			m2 = DBLayerTypeMaskTbl[t];
 
-		    /* Include all types which have t as a residue */
+		    /* Include all types which have t as a residue		*/
+
 		    if (allResidues)
 			for (rtype = TT_TECHDEPBASE; rtype < DBNumUserLayers; rtype++)
 			    if (TTMaskHasType(DBResidueMask(rtype), t))
 				TTMaskSetType(&m2, rtype);
+
+		    /* Include all stacking types which have t as a residue	*/
+		    /* (this is done regardless of the presence of "*")		*/
+
+		    for (rtype = DBNumUserLayers; rtype < DBNumTypes; rtype++)
+			if (TTMaskIntersect(DBResidueMask(rtype), &m2))
+			    TTMaskSetType(&m2, rtype);
 		}
 		*p2 = save;
 	    }
@@ -722,7 +733,7 @@ DBTechNoisyNameMask(layers, mask)
 	    TileType t; 
 
 	    planemask = 0;
-	    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
+	    for (t = TT_TECHDEPBASE; t < DBNumUserLayers; t++)
 		if (TTMaskHasType(&m2, t))
 		    planemask |= DBTypePlaneMaskTbl[t];
 	}
@@ -730,133 +741,14 @@ DBTechNoisyNameMask(layers, mask)
 	TTMaskSetMask(mask, &m2);
 	while (*p == ',') p++;
     }
+
+    /* If there are no types, or if "space" is the only type, then	*/
+    /* return a full planemask						*/
+
+    if ((TTMaskIsZero(mask) || TTMaskEqual(mask, &DBSpaceBits)) &&
+		(planemask == (PlaneMask)0))
+	planemask = DBTypePlaneMaskTbl[TT_SPACE];
+
     return planemask;
 }
-
-/*
- * ----------------------------------------------------------------------------
- *
- * DBTechMinSetPlanes --
- *
- * Given a TileTypeBitMask, find the minimum set of planes that
- * contains one image for each of the tile types in the mask.
- * Also return a new tile type mask that contains the images
- * of the original tiles that fall in the result planes.
- *
- * WARNING: must be called AFTER all the database technology
- * initialization has completed.
- *
- * Results:
- *	A mask of planes.
- *
- * Side effects:
- *	The parameter newTypes is modified.
- *
- * ----------------------------------------------------------------------------
- */
 
-PlaneMask
-DBTechMinSetPlanes(typeMask, newTypes)
-    TileTypeBitMask typeMask;		/* Collection of tile types. */
-    TileTypeBitMask *newTypes;		/* Filled in with new set of types
-					 * equivalent to typeMask.
-					 */
-{
-    int i, j;
-    PlaneMask planes, okPlanes;
-    TileTypeBitMask tempTypes;
-
-    /*
-     * First go through and find all planes for non-contact types.
-     * These MUST be in the result.
-     */
-    planes = 0;
-    TTMaskZero(newTypes);
-    for (i = TT_SELECTBASE; i < DBNumTypes; i++)
-    {
-	if (!TTMaskHasType(&typeMask, i) || DBIsContact(i))
-	    continue;
-	TTMaskSetType(newTypes, i);
-	planes |= PlaneNumToMaskBit(DBPlane(i));
-    }
-
-    /*
-     * Now go through each contact type.  If one of the contact's planes
-     * is already necessary, use it.  Otherwise, add the home plane for
-     * the contact.
-     */
-    for (i = TT_SELECTBASE; i < DBNumTypes; i++)
-    {
-	if (!TTMaskHasType(&typeMask, i) || !DBIsContact(i))
-	    continue;
-	okPlanes = DBConnPlanes[i] & planes;
-	if (okPlanes != 0)
-	{
-	    for (j = PL_SELECTBASE; j < DBNumPlanes; j++)
-		if (PlaneMaskHasPlane(okPlanes, j))
-		{
-		    TTMaskAndMask3(&tempTypes,
-				&DBLayerTypeMaskTbl[i], &DBPlaneTypes[j]);
-		    TTMaskSetMask(newTypes, &tempTypes);
-		    break;
-		}
-	}
-	else
-	{
-	    TTMaskSetType(newTypes, i);
-	    planes |= PlaneNumToMaskBit(DBPlane(i));
-	}
-    }
-
-    return (planes);
-}
-
-/*
- * ----------------------------------------------------------------------------
- *
- * DBTechSubsetLayers --
- *
- * 	Eliminate all bits from one mask that aren't in another, and
- *	check to be sure that this elimination only occurs for contact
- *	types that will still have one image in the result.
- *
- * Results:
- *	TRUE is returned if the subsetting was successful.  Success
- *	means that for each layer in "src", the corresponding layer
- *	is in "mask" or else "src" contains another image of the bit
- *	that is in "mask".
- *
- * Side effects:
- *	The mask pointed to by "result" is modified to contain the
- *	subset of "src" that is in "mask".
- *
- * ----------------------------------------------------------------------------
- */
-
-bool
-DBTechSubsetLayers(src, mask, result)
-    TileTypeBitMask src;		/* Original set of tile types. */
-    TileTypeBitMask mask;		/* Only keep types in this mask. */
-    TileTypeBitMask *result;		/* Store subset here. */
-{
-    TileTypeBitMask tmp,tmp2,clrmask;
-    int i;
-
-    TTMaskZero(result);
-    TTMaskZero(&clrmask);
-    /* for (i = 0; i < DBNumUserLayers; i++) */	/* ? */
-    for (i = 0; i < DBNumTypes; i++)
-    {
-	TTMaskAndMask3(&tmp, &src, &DBLayerTypeMaskTbl[i]);
-	if (!TTMaskIsZero(&tmp))
-	{
-	     TTMaskAndMask3(&tmp2, &tmp,&mask);
-	     if (!TTMaskIsZero(&tmp2))
-	     {
-	     	  TTMaskSetMask(&clrmask,&tmp);
-	          TTMaskSetMask(result, &tmp2);
-	     }
-	}
-    }
-    if (TTMaskEqual(&clrmask,&src)) return TRUE; else return FALSE;
-}
