@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBtpaint2.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBtpaint2.c,v 1.5 2008/01/30 07:41:04 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -35,10 +35,12 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 #include "textio/textio.h"
 
 #define	SETPAINT(have,paint,plane,get) \
-	if (IsDefaultPaint((have), (paint))) \
+	if (IsDefaultPaint((have), (paint)) \
+		&& TTMaskHasType(&DBPlaneTypes[(plane)], (have))) \
 	    dbSetPaintEntry((have), (paint), (plane), (get))
 #define	SETERASE(have,erase,plane,get) \
-	if (IsDefaultErase((have), (erase))) \
+	if (IsDefaultErase((have), (erase)) \
+		&& TTMaskHasType(&DBPlaneTypes[(plane)], (have))) \
 	    dbSetEraseEntry((have), (erase), (plane), (get))
 
 LayerInfo *dbTechLpPaint;
@@ -87,6 +89,9 @@ extern void dbComposeDecompose();
 void
 DBTechFinalCompose()
 {
+    TileType i;
+    TileTypeBitMask testmask;
+
     /* Default rules for painting/erasing contacts */
     dbComposePaintAllImages();
     dbComposeResidues();
@@ -98,6 +103,21 @@ DBTechFinalCompose()
     /* Build up exported tables */
     dbTechPaintErasePlanes();
 
+    /* Adjust paint tables for any locked layers */
+    for (i = TT_TECHDEPBASE; i < DBNumUserLayers; i++)
+	if (!TTMaskHasType(&DBActiveLayerBits, i))
+	    if (DBIsContact(i))
+		DBLockContact(i);
+    for (i = DBNumUserLayers; i < DBNumTypes; i++)
+    {
+	TTMaskAndMask3(&testmask, &DBActiveLayerBits, DBResidueMask(i));
+	if (!TTMaskEqual(&testmask, DBResidueMask(i)))
+	{
+	    TTMaskClearType(&DBActiveLayerBits, i);
+	    DBLockContact(i);
+	}
+    }
+
     /* Diagnostic */
     /* dbTechPrintPaint("DBTechFinalCompose", TRUE, FALSE); */
     /* dbTechPrintPaint("DBTechFinalCompose", FALSE, FALSE); */
@@ -321,10 +341,12 @@ dbComposeContacts()
 
 
 /*
+ * ----------------------------------------------------------------------------
  * dbComposePaintContact --
  *
  * Construct the painting rules for painting type lpPaint over
  * the contact image lpImage.
+ * ----------------------------------------------------------------------------
  */
 
 void
@@ -429,20 +451,6 @@ dbComposePaintContact(lpImage, lpPaint)
 		     * types.
 		     */
 
-		    /* Temporary:  equals the condition below.  HOWEVER, */
-		    /* we should consider the case here where a type may */
-		    /* change due to incompatible residues, e.g., paint	 */
-		    /* pc over pd12c (pd12c becomes pm12c).  But maybe	 */
-		    /* this should be treated by the "compose" section	 */
-		    /* rules.						 */
-		
-/*
-		    SETPAINT(lpImage->l_type, lpPaint->l_type,
-				DBPlane(lpImage->l_type), lpImage->l_type);
-*/
-
-		    /* Do nothing?  These conditions don't seem to apply. */
-
 		    for (itype = TT_TECHDEPBASE; itype < DBNumUserLayers; itype++)
 		    {
 			if (TTMaskHasType(&lpImage->l_residues, itype))
@@ -452,13 +460,6 @@ dbComposePaintContact(lpImage, lpPaint)
 				SETPAINT(lpImage->l_type, lpPaint->l_type,
 						DBPlane(itype), lpPaint->l_type);
 			    }
-/*
-			    else
-			    {
-				SETPAINT(lpImage->l_type, lpPaint->l_type,
-						DBPlane(itype), itype);
-			    }
-*/
 			}
 		    }
 		}
@@ -613,10 +614,12 @@ dbComposeSubsetResidues(lpImage, lpErase, outMask)
 
 
 /*
+ * ----------------------------------------------------------------------------
  * dbComposeEraseContact --
  *
  * Construct the erasing rules for erasing type lpErase from
  * the contact image lpImage.
+ * ----------------------------------------------------------------------------
  */
 
 void
@@ -715,7 +718,77 @@ dbComposeEraseContact(lpImage, lpErase)
 */
 
 }
-
+
+/*
+ * ----------------------------------------------------------------------------
+ * DBLockContact --
+ *
+ * This procedure modifies the erase tables so that the specified contact
+ * type cannot be erased by erasing one of its residues, which is the default
+ * behavior.
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DBLockContact(ctype)
+   TileType ctype;
+{
+    LayerInfo *lpImage, *lpPaint;
+    TileType c, n, itype, eresult;
+    TileTypeBitMask *rmask;
+    int m, pNum;
+
+    /* Have type, Erase * --> Result is type */
+
+    lpPaint = &dbLayerInfo[ctype];
+    for (n = TT_TECHDEPBASE; n < DBNumTypes; n++)
+    {
+	if (n == ctype) continue;
+
+	/* Avoid the case, e.g., if ctype is pc+v, then pc+v - pc = v	*/
+	/* is still valid if pc is an active layer.			*/
+
+	if (ctype >= DBNumUserLayers)
+	    if (TTMaskHasType(DBResidueMask(ctype), n))
+		if (TTMaskHasType(&DBActiveLayerBits, n))
+		    continue;
+
+	for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
+	    if (PlaneMaskHasPlane(lpPaint->l_pmask, pNum))
+		SETERASE(ctype, n, pNum, ctype);
+    }
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ * DBUnlockContact --
+ *
+ * This procedure reverses the operation of DBLockContact, allowing a contact
+ * to be erased by erasing one of its residues.  This is the same code as
+ * dbComposeContacts(), for a single contact type.
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DBUnlockContact(ctype)
+   TileType ctype;
+{
+    LayerInfo *lpImage, *lpPaint;
+    TileType n, itype, eresult;
+    TileTypeBitMask *rmask;
+    int m, pNum;
+
+    lpImage = &dbLayerInfo[ctype];
+
+    for (n = TT_TECHDEPBASE; n < DBNumUserLayers; n++)
+    {
+	    lpPaint = &dbLayerInfo[n];
+	    dbComposeEraseContact(lpImage, lpPaint);
+    }
+
+    /* To be done (maybe):  revert rules for stacked contact types */
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -766,6 +839,7 @@ dbComposeSavedRules()
 }
 
 /*
+ * ----------------------------------------------------------------------------
  * dbComposeDecompose --
  *
  * Painting componentType over imageType is a no-op.
@@ -781,6 +855,7 @@ dbComposeSavedRules()
  *	Indicates that these modifications are not default
  *	rules by setting the corresponding bits in the tables
  *	dbNotDefaultPaintTbl[] and dbNotDefaultEraseTbl[].
+ * ----------------------------------------------------------------------------
  */
 
 void
@@ -808,6 +883,7 @@ dbComposeDecompose(imageType, componentType, remainingType)
 }
 
 /*
+ * ----------------------------------------------------------------------------
  * dbComposeCompose --
  *
  * Painting paintType over existingType gives imageType.
@@ -820,6 +896,7 @@ dbComposeDecompose(imageType, componentType, remainingType)
  *	Indicates that these modifications are not default
  *	rules by setting the corresponding bits in the tables
  *	dbNotDefaultPaintTbl[] and dbNotDefaultEraseTbl[].
+ * ----------------------------------------------------------------------------
  */
 
 void
