@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/dbwind/DBWdisplay.c,v 1.2 2006/06/16 16:16:11 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/dbwind/DBWdisplay.c,v 1.10 2008/02/03 21:23:56 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -167,7 +167,7 @@ DBWredisplay(w, rootArea, clipArea)
     int bitMask, lambdasPerPixel, pixelsPerLambda;
     DBWclientRec *crec;
     CellDef *cellDef;
-    TileTypeBitMask layers;
+    TileTypeBitMask layers, rmask;
 
     /*
     TxPrintf("Root area (%d, %d) (%d, %d) redisplay.\n", 
@@ -277,7 +277,7 @@ DBWredisplay(w, rootArea, clipArea)
 	crec->dbw_expandAmounts.r_ytop);
     **/
 
-    /* Set the clipping rectangle to only contain the area being displayed. */
+    /* Set the clipping rectangle to contain only the area being displayed. */
 
     dbwWindow = w;
     windClip = w->w_screenArea;
@@ -330,6 +330,29 @@ DBWredisplay(w, rootArea, clipArea)
 	TTMaskAndMask3(&layers, mask, &crec->dbw_visibleLayers);
 	if (!TTMaskIsZero(&layers))
 	{
+	    TileType t, s;
+
+	    /* For each contact type, if the contact is not visible,	*/  
+	    /* display any of its residue layers that are visible.	*/  
+
+	    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
+		if (DBIsContact(t))
+		    if (!TTMaskHasType(&layers, t))
+			if (TTMaskIntersect(DBResidueMask(t), &layers))
+			    TTMaskSetType(&layers, t);
+
+	    /* Treat any stacking type in the mask like the	*/
+	    /* residue type whose home plane is the same as	*/
+	    /* the home plane of the stacking type.		*/
+
+	    for (t = TT_TECHDEPBASE; t < DBNumUserLayers; t++)
+		if (TTMaskHasType(&layers, t))
+		    for (s = DBNumUserLayers; s < DBNumTypes; s++)
+			if (TTMaskHasType(&layers, s) &&
+				TTMaskHasType(DBResidueMask(s), t))
+			    if (DBPlane(s) != DBPlane(t))
+				TTMaskClearType(&layers, s);
+
 	    /*
 	     * To avoid holding a lock for too long, we don't set it
 	     * until we actually see something to display, and we
@@ -398,23 +421,25 @@ DBWredisplay(w, rootArea, clipArea)
      * large.
      */
 
-    GrLabelSize("BBB", GEO_CENTER, GR_TEXT_SMALL, &dbwMinBBox);
-    dbwMinBBox.r_xtop -= dbwMinBBox.r_xbot;
-    dbwMinBBox.r_ytop -= dbwMinBBox.r_ybot;
+    if (crec->dbw_flags & DBW_SEECELLS)
+    {
+	GrLabelSize("BBB", GEO_CENTER, GR_TEXT_SMALL, &dbwMinBBox);
+	dbwMinBBox.r_xtop -= dbwMinBBox.r_xbot;
+	dbwMinBBox.r_ytop -= dbwMinBBox.r_ybot;
 
-    /* Redisplay cell names as if the whole window were visible.
-     * This must be done since we slide the names around to fit
-     * into the clip area.  Set the style beforehand for speed.
-     */
-
-    GrClipTo(&GrScreenRect);	/* Will be cut down to window size by
-				 * the graphics module.
-				 */
-    scontext.scx_area = largerArea;
-    GrSetStuff(STYLE_BBOX);
-    (void) DBTreeSrCells(&scontext, bitMask, dbwBBoxFunc, (ClientData)NULL);
-    GrClipTo(&rootClip);
+	/* Redisplay cell names as if the whole window were visible.
+	 * This must be done since we slide the names around to fit
+	 * into the clip area.  Set the style beforehand for speed.
+	 */
 
+	GrClipTo(&GrScreenRect); /* Will be cut down to window size by
+				  * the graphics module.
+				  */
+	scontext.scx_area = largerArea;
+	GrSetStuff(STYLE_BBOX);
+	(void) DBTreeSrCells(&scontext, bitMask, dbwBBoxFunc, (ClientData)NULL);
+	GrClipTo(&rootClip);
+    }
 
     /* Now redisplay the grid.  This code is a bit tricky because
      * it makes sure that the origin ends up on a grid line.
@@ -636,8 +661,8 @@ dbwPaintFunc(tile, cxp)
  */
 
 void
-DBWDrawLabel(text, rect, pos, style, labelSize, sizeBox)
-    char *text;			/* Text to be displayed. */
+DBWDrawLabel(label, rect, pos, style, labelSize, sizeBox)
+    Label *label;		/* Text to be displayed. */
     Rect *rect;			/* Area of label's box/line/cross, in screen
 				 * coordinates.
 				 */
@@ -658,6 +683,8 @@ DBWDrawLabel(text, rect, pos, style, labelSize, sizeBox)
 {
     Point p;
     Rect location;
+    char *text = label->lab_text; 
+    int result;
 
     if (style < 0) GrFastBox(rect);
     else GrClipBox(rect, style);
@@ -701,8 +728,8 @@ DBWDrawLabel(text, rect, pos, style, labelSize, sizeBox)
 	    break;
     }
 
-    if (GrPutText(text, style, &p, pos, labelSize,
-	    FALSE, &GrScreenRect, &location) )
+    if (GrPutText(text, style, &p, pos, labelSize, FALSE,
+			&GrScreenRect, &location))
     {
 	sizeBox->r_xbot = MIN(sizeBox->r_xbot, location.r_xbot - p.p_x);
 	sizeBox->r_ybot = MIN(sizeBox->r_ybot, location.r_ybot - p.p_y);
@@ -741,14 +768,15 @@ dbwLabelFunc(scx, label, tpath)
     Rect labRect, tmp;
     int screenPos;
 
-    screenPos = GeoTransPos(&scx->scx_trans, label->lab_pos);
+    screenPos = GeoTransPos(&scx->scx_trans, label->lab_just);
     GeoTransRect(&scx->scx_trans, &label->lab_rect, &tmp);
     WindSurfaceToScreen(dbwWindow, &tmp, &labRect);
+
     if (!GEO_TOUCH(&labRect, &dbwWindow->w_screenArea)) return 0;
     if (label->lab_flags & PORT_DIR_MASK)
         GrSetStuff(STYLE_PORT);
 	
-    DBWDrawLabel(label->lab_text, &labRect, screenPos, -1,
+    DBWDrawLabel(label, &labRect, screenPos, -1,
 	dbwLabelSize, dbwExpandAmounts);
 
     if (label->lab_flags & PORT_DIR_MASK)
@@ -815,9 +843,11 @@ dbwBBoxFunc(scx)
     p.p_x = (r.r_xbot + r.r_xtop)/2;
     p.p_y = (r.r_ybot + 2*r.r_ytop)/3;
     GeoClip(&r, &windClip);
+
     GrPutText(cellUse->cu_def->cd_name, -1, &p,
-	GEO_CENTER, GR_TEXT_LARGE, TRUE, &r, (Rect *) NULL);
-    (void) DBPrintUseId(scx, idName, 100);
+		GEO_CENTER, GR_TEXT_LARGE, TRUE, &r, (Rect *) NULL);
+
+    (void) DBPrintUseId(scx, idName, 100, TRUE);
     p.p_y = (2*r.r_ybot + r.r_ytop)/3;
     GrPutText(idName, -1, &p, GEO_CENTER,
         GR_TEXT_LARGE, TRUE, &r, (Rect *)NULL);
@@ -1493,6 +1523,7 @@ DBWTechAddStyle(sectionName, argc, argv)
     TileType t, r;
     int i, sidx;
     static char styleType[50];
+    char *pathptr;
 
     if (argc < 2)
     {
@@ -1504,20 +1535,30 @@ DBWTechAddStyle(sectionName, argc, argv)
 	styleType[49] = 0;
 	DBWStyleType = styleType;
 
-	/* Learning the style type precipitates immediate reading of	*/
-	/* the color map and style files.				*/
+	/* Optional 3rd and higher arguments are pathnames */
 
-	if (!GrReadCMap(DBWStyleType, (char *)NULL, MainMonType, ".",
-		SysLibPath))
-	    return FALSE;
+	for (i = 2; i <= argc; i++)
+	{
+	    if (i == argc)
+		pathptr = SysLibPath;
+	    else
+		pathptr = argv[i];
 
-	if (GrLoadStyles(DBWStyleType, ".", SysLibPath) != 0)
+	    /* Learning the style type precipitates immediate reading	*/
+	    /* of the color map and style files.			*/
+
+	    if (GrReadCMap(DBWStyleType, (char *)NULL, MainMonType, ".", pathptr))
+		break;
+	}
+	if (i > argc) return FALSE;
+
+	if (GrLoadStyles(DBWStyleType, ".", pathptr) != 0)
 	    return FALSE;
 
 	DBWTechInitStyles();
 
-	if (!GrLoadCursors(".", SysLibPath))
-	   return FALSE;
+	if (!GrLoadCursors(".", pathptr))
+	    return FALSE;
 
 	GrSetCursor(0);
     }
