@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/dbwind/DBWprocs.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/dbwind/DBWprocs.c,v 1.3 2008/02/07 17:32:59 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -100,7 +100,7 @@ DBWcreate(window, argc, argv)
     dbwBitMask = newBitMask;
 
     crec = (DBWclientRec *) mallocMagic(sizeof(DBWclientRec));
-    crec->dbw_flags = DBW_SEELABELS;
+    crec->dbw_flags = DBW_SEELABELS | DBW_SEECELLS;
     crec->dbw_watchPlane = -1;
     crec->dbw_watchDef = (CellDef *) NULL;
     crec->dbw_bitmask = bitMask;
@@ -376,6 +376,8 @@ DBWloadWindow(window, name, ignoreTech, expand)
 	if (newEditUse->cu_def->cd_flags & CDNOEDIT)
 	{
 	    newEdit = FALSE;
+	    EditCellUse = NULL;
+	    EditRootDef = NULL;
 	}
 	else
 	{
