@@ -20,7 +20,7 @@
  */
 
 #ifndef	lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCbasic.c,v 1.5 2007/10/11 17:23:51 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCbasic.c,v 1.7 2007/12/24 22:56:43 tim Exp $";
 #endif	/* not lint */
 
 #include <sys/types.h>
@@ -43,7 +43,7 @@ int dbDRCDebug = 0;
 static DRCCookie drcOverlapCookie = {
     0, 0, 0, 0,
     { 0 }, { 0 },
-    0, 0,
+    0, 0, 0,
     "Can't overlap those layers",
     (DRCCookie *) NULL
 };
@@ -264,7 +264,6 @@ drcTile (tile, arg)
     Rect *rect = arg->dCD_rect;	/* Area being checked */
     Rect errRect;		/* Area checked for an individual rule */
     TileTypeBitMask tmpMask, *rMask;
-    unsigned char stackpending = (char)0;
     bool triggered, trigpending;	/* Hack for widespacing rule */
     int cdist, dist, ccdist, result;
 
@@ -384,41 +383,6 @@ drcTile (tile, arg)
 
 	    if (tt == to) continue;
 
-	    /*
-	     * Stacked contact types have no rules.  Any stacked contact
-	     * type must have DRC rules applied to each of its constituent
-	     * types.  But, if the residue mask for the stacking type
-	     * contains the other type, then this is a "false" edge, and
-	     * we will not process it.  Otherwise, start with the first
-	     * type encountered for each.
-	     */
-	    stackpending = (unsigned char)0x0;
-	    if (tt >= DBNumUserLayers)
-	    {
-		rMask = DBResidueMask(tt);
-		if (to >= DBNumUserLayers)
-		{
-		    TileTypeBitMask combined;
-		    TTMaskAndMask3(&combined, rMask, DBResidueMask(to));
-		    if (!TTMaskIsZero(&combined))
-			continue;
-		}
-		else if (TTMaskHasType(rMask, to)) continue;
-		stackpending |= (unsigned char)0x01;
-		for (tt = TT_TECHDEPBASE; tt < DBNumUserLayers; tt++)
-		    if (TTMaskHasType(rMask, tt))
-			break;
-	    }
-	    if (to >= DBNumUserLayers)
-	    {
-		rMask = DBResidueMask(to);
-		if (TTMaskHasType(rMask, tt)) continue;
-		stackpending |= (unsigned char)0x02;
-		for (to = TT_TECHDEPBASE; to < DBNumUserLayers; to++)
-		    if (TTMaskHasType(rMask, to))
-			break;
-	    }
-
 	    /*
 	     * Go through list of design rules triggered by the
 	     * left-to-right edge.
@@ -428,14 +392,10 @@ drcTile (tile, arg)
 	    if (edgeTop <= edgeBot)
 		continue;
 
-	    while(1)
-	    {
-		if (stackpending && (to == tt)) goto hnextstacktype;
-		
-		triggered = FALSE;
-		for (cptr = DRCCurStyle->DRCRulesTbl[to][tt]; cptr != (DRCCookie *) NULL;
+	    triggered = FALSE;
+	    for (cptr = DRCCurStyle->DRCRulesTbl[to][tt]; cptr != (DRCCookie *) NULL;
 			cptr = cptr->drcc_next)
-		{
+	    {
 		    if (cptr->drcc_flags & DRC_ANGLES) continue;
 
 		    /* Find the rule distances according to the scale factor */
@@ -443,15 +403,10 @@ drcTile (tile, arg)
 		    cdist = cptr->drcc_cdist;
 		    trigpending = (cptr->drcc_flags & DRC_TRIGGER) ? TRUE : FALSE;
 
-		    /* Contact images exist in multiple planes; only check	*/
-		    /* rules that are for the plane being checked.		*/
-		    /* NOTE:  Ideally, the plane of the edge should be recorded	*/
-		    /* in the rule; i.e., we should have cptr->drcc_edgeplane	*/
-		    /* and compare that to arg->dCD_plane.  However, the	*/
-		    /* following eliminates most of the redundant checks.	*/
-		
-		    if ((!(cptr->drcc_flags & DRC_XPLANE)) &&
-				(arg->dCD_plane != cptr->drcc_plane))
+		    /* drcc_edgeplane is used to avoid checks on edges	*/
+		    /* in more than one plane				*/
+
+		    if (arg->dCD_plane != cptr->drcc_edgeplane)
 		    {
 			if (trigpending) cptr = cptr->drcc_next;
 			continue;
@@ -600,7 +555,7 @@ drcTile (tile, arg)
 			if (LEFT(tpleft) <= errRect.r_xbot
 				&& BOTTOM(tpleft) <= errRect.r_ybot
 				&& TOP(tpleft) >= errRect.r_ytop
-				&& !(cptr->drcc_flags & DRC_XPLANE)
+				&& arg->dCD_plane == cptr->drcc_plane
 				&& TTMaskHasType(&cptr->drcc_mask,
 				TiGetRightType(tpleft)))
 			{
@@ -674,7 +629,7 @@ drcTile (tile, arg)
 			if (RIGHT(tile) >= errRect.r_xtop
 				&& BOTTOM(tile) <= errRect.r_ybot
 				&& TOP(tile) >= errRect.r_ytop
-				&& !(cptr->drcc_flags & DRC_XPLANE)
+				&& arg->dCD_plane == cptr->drcc_plane
 				&& TTMaskHasType(&cptr->drcc_mask,
 				TiGetLeftType(tile)))
 			{
@@ -709,48 +664,6 @@ drcTile (tile, arg)
 		    }
 		}
 		DRCstatEdges++;
-
-hnextstacktype:
-
-		/* Check if more processing is required for stacked	*/
-		/* contact types 					*/
-
-		if (!stackpending) break;
-		else if (stackpending == (unsigned char)0x01)
-		{
-		    rMask = DBResidueMask(TiGetLeftType(tile));
-		    for (++tt; tt < DBNumUserLayers; tt++)
-			if (TTMaskHasType(rMask, tt))
-			    break;
-		}
-		else if (stackpending == (unsigned char)0x02)
-		{
-		    rMask = DBResidueMask(TiGetRightType(tpleft));
-		    for (++to; to < DBNumUserLayers; to++)
-			if (TTMaskHasType(rMask, to))
-			    break;
-		}
-		else /* stackpending for both types */
-		{
-		    rMask = DBResidueMask(TiGetLeftType(tile));
-		    for (++tt; tt < DBNumUserLayers; tt++)
-			if (TTMaskHasType(rMask, tt))
-			    break;
-
-		    if (tt >= DBNumUserLayers)	
-		    {
-			for (tt = TT_TECHDEPBASE; tt < DBNumUserLayers; tt++)
-			    if (TTMaskHasType(rMask, tt))
-				break;
-
-			rMask = DBResidueMask(TiGetRightType(tpleft));
-			for (++to; to < DBNumUserLayers; to++)
-			    if (TTMaskHasType(rMask, to))
-				break;
-		    }
-		}
-		if ((to >= DBNumUserLayers) || (tt >= DBNumUserLayers)) break;
-	    }
         }
     }
 
@@ -818,39 +731,6 @@ checkbottom:
 
 	    if (tt == to) continue;
 
-	    /*
-	     * Stacked contact types have no rules.  Any stacked contact
-	     * type must have DRC rules applied to each of its constituent
-	     * types.  Start with the first type encountered for each.
-	     */
-
-	    stackpending = (unsigned char)0x0;
-	    if (tt >= DBNumUserLayers)
-	    {
-		rMask = DBResidueMask(tt);
-		if (to >= DBNumUserLayers)
-		{
-		    TileTypeBitMask combined;
-		    TTMaskAndMask3(&combined, rMask, DBResidueMask(to));
-		    if (!TTMaskIsZero(&combined))
-			continue;
-		}
-		else if (TTMaskHasType(rMask, to)) continue;
-		stackpending |= (unsigned char)0x01;
-		for (tt = TT_TECHDEPBASE; tt < DBNumUserLayers; tt++)
-		    if (TTMaskHasType(rMask, tt))
-			break;
-	    }
-	    if (to >= DBNumUserLayers)
-	    {
-		rMask = DBResidueMask(to);
-		if (TTMaskHasType(rMask, tt)) continue;
-		stackpending |= (unsigned char)0x02;
-		for (to = TT_TECHDEPBASE; to < DBNumUserLayers; to++)
-		    if (TTMaskHasType(rMask, to))
-			break;
-	    }
-
 	    /*
 	     * Check to insure that we are inside the clip area.
 	     * Go through list of design rules triggered by the
@@ -861,14 +741,10 @@ checkbottom:
 	    if (edgeLeft >= edgeRight)
 		continue;
 
-	    while(1)
-	    {
-		if (stackpending && (to == tt)) goto vnextstacktype;
-
-		triggered = FALSE;
-		for (cptr = DRCCurStyle->DRCRulesTbl[to][tt]; cptr != (DRCCookie *) NULL;
+	    triggered = FALSE;
+	    for (cptr = DRCCurStyle->DRCRulesTbl[to][tt]; cptr != (DRCCookie *) NULL;
 				cptr = cptr->drcc_next)
-		{
+	    {
 		    if (cptr->drcc_flags & DRC_ANGLES) continue;
 
 		    /* Find the rule distances according to the scale factor */
@@ -876,10 +752,10 @@ checkbottom:
 		    cdist = cptr->drcc_cdist;
 		    trigpending = (cptr->drcc_flags & DRC_TRIGGER) ? TRUE : FALSE;
 
-		    /* Contact images exist in multple planes; only check	*/
-		    /* rules that are for the plane being checked.		*/
-		    if ((!(cptr->drcc_flags & DRC_XPLANE)) &&
-				(arg->dCD_plane != cptr->drcc_plane))
+		    /* drcc_edgeplane is used to avoid checks on edges	*/
+		    /* in more than one plane				*/
+
+		    if (arg->dCD_plane != cptr->drcc_edgeplane)
 		    {
 			if (trigpending) cptr = cptr->drcc_next;
 			continue;
@@ -989,7 +865,7 @@ checkbottom:
 			if (BOTTOM(tpbot) <= errRect.r_ybot
 				&& LEFT(tpbot) <= errRect.r_xbot
 				&& RIGHT(tpbot) >= errRect.r_xtop
-				&& !(cptr->drcc_flags & DRC_XPLANE)
+				&& arg->dCD_plane == cptr->drcc_plane
 				&& TTMaskHasType(&cptr->drcc_mask,
 				TiGetTopType(tpbot)))
 			{
@@ -1043,7 +919,7 @@ checkbottom:
 			if (TOP(tile) >= errRect.r_ytop
 				&& LEFT(tile) <= errRect.r_xbot
 				&& RIGHT(tile) >= errRect.r_xtop
-				&& !(cptr->drcc_flags & DRC_XPLANE)
+				&& arg->dCD_plane == cptr->drcc_plane
 				&& TTMaskHasType(&cptr->drcc_mask,
 				TiGetBottomType(tile)))
 			{
@@ -1078,47 +954,6 @@ checkbottom:
 		}
 		DRCstatEdges++;
 
-vnextstacktype:
-		/* Check if more processing is required for stacked	*/
-		/* contact types 					*/
-		
-		if (!stackpending) break;
-		else if (stackpending == (unsigned char)0x01)
-		{
-		    rMask = DBResidueMask(TiGetBottomType(tile));
-		    for (++tt; tt < DBNumUserLayers; tt++)
-			if (TTMaskHasType(rMask, tt))
-			    break;
-		}
-		else if (stackpending == (unsigned char)0x02)
-		{
-		    rMask = DBResidueMask(TiGetTopType(tpbot));
-		    for (++to; to < DBNumUserLayers; to++)
-			if (TTMaskHasType(rMask, to))
-			    break;
-		}
-		else /* stackpending for both types */
-		{
-		    rMask = DBResidueMask(TiGetBottomType(tile));
-		    for (++tt; tt < DBNumUserLayers; tt++)
-			if (TTMaskHasType(rMask, tt))
-			    break;
-
-		    if (tt >= DBNumUserLayers)	
-		    {
-			for (tt = TT_TECHDEPBASE; tt < DBNumUserLayers; tt++)
-			    if (TTMaskHasType(rMask, tt))
-				break;
-
-			rMask = DBResidueMask(TiGetTopType(tpbot));
-			for (++to; to < DBNumUserLayers; to++)
-			    if (TTMaskHasType(rMask, to))
-				break;
-
-		    }
-		}
-		if ((to >= DBNumUserLayers) || (tt >= DBNumUserLayers)) break;
-	    }
         }
     }
     return (0);
