@@ -24,7 +24,7 @@
  */
 
 #ifndef	lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCcif.c,v 1.1 2007/10/12 22:14:36 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCcif.c,v 1.4 2008/01/09 00:51:55 tim Exp $";
 #endif	/* not lint */
 
 #include <stdio.h>
@@ -69,6 +69,7 @@ TileTypeBitMask drcCifGenLayers;
 DRCCookie *drcCifRules[MAXCIFLAYERS][2];
 DRCCookie *drcCifCur=NULL;
 int	drcCifValid = FALSE;
+int	beenWarned;
 
 #define DRC_CIF_SPACE		0
 #define DRC_CIF_SOLID		1
@@ -116,9 +117,11 @@ drcCifSetStyle(argc, argv)
 		drcCifStyle = CIFCurStyle;
 	    else
 	    {
-		TechError("DRC cif extensions are not enabled.\n");
-		TechError("Use \"cif ostyle %s\" to enable them.\n", new->cs_name);
+		TechError("DRC cif extensions are not enabled.\n\t"
+			"Use \"cif ostyle %s\" to enable them.\n",
+			new->cs_name);
 		drcCifStyle = NULL;
+		beenWarned = TRUE;	/* post no more error messages */
 	    }
 	    return 0;
 	}
@@ -127,6 +130,23 @@ drcCifSetStyle(argc, argv)
     return (0);
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ * ----------------------------------------------------------------------------
+ */
+
+int
+drcCifWarning()
+{
+    if (!beenWarned)
+    {
+	TechError("Missing cif style for drc\n\t"
+		"This message will not be repeated.\n");
+	beenWarned = TRUE;
+    }
+    return 0;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -157,10 +177,8 @@ drcCifWidth(argc, argv)
     TileType i;
 
     if (drcCifStyle == NULL)
-    {
-    	 TechError("Missing cif style for drc\n");
-	 return 0;
-    } 
+	return drcCifWarning();
+
     for (i = 0; i < drcCifStyle->cs_nLayers;i++)
     {
     	 CIFLayer	*layer = drcCifStyle->cs_layers[i];
@@ -178,10 +196,11 @@ drcCifWidth(argc, argv)
     }
 
     scalefactor = drcCifStyle->cs_scaleFactor;
+    centidistance *= drcCifStyle->cs_expander;		// BSI
     dpnext = drcCifRules[thislayer][DRC_CIF_SPACE];
     dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
     drcAssign(dpnew,centidistance,dpnext,CIFSolidBits,
-    		CIFSolidBits,why,centidistance,DRC_FORWARD,thislayer);
+    		CIFSolidBits,why,centidistance,DRC_FORWARD,thislayer,0);
     drcCifRules[thislayer][DRC_CIF_SPACE] = dpnew;
 
     return ((centidistance+scalefactor-1)/scalefactor);
@@ -223,10 +242,8 @@ drcCifSpacing(argc, argv)
     layers[1] = argv[2];
     
     if (drcCifStyle == NULL)
-    {
-    	 TechError("Missing cif style for drc\n");
-	 return 0;
-    } 
+	return drcCifWarning();
+
     for (k=0; k!= 2;k++)
     {
          for (i = 0; i < drcCifStyle->cs_nLayers;i++)
@@ -270,20 +287,21 @@ drcCifSpacing(argc, argv)
     }
     
     scalefactor = drcCifStyle->cs_scaleFactor;
+    centidistance *= drcCifStyle->cs_expander;		// BSI
     dpnext = drcCifRules[layer[0]][DRC_CIF_SOLID];
     dpnew = (DRCCookie *) mallocMagic((unsigned) sizeof (DRCCookie));
     drcAssign(dpnew,centidistance,dpnext,DBSpaceBits,
-    		cmask,why,centidistance,DRC_FORWARD,layer[1]);
+    		cmask,why,centidistance,DRC_FORWARD,layer[1],0);
     drcCifRules[layer[0]][DRC_CIF_SOLID] = dpnew;
     
     if (needReverse)
     {
-         dpnew->drcc_flags |= DRC_BOTHCORNERS|DRC_XPLANE;
+         dpnew->drcc_flags |= DRC_BOTHCORNERS;
          dpnext = drcCifRules[layer[1]][DRC_CIF_SOLID];
          dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
          drcAssign(dpnew,centidistance,dpnext,DBSpaceBits,
     		cmask,why,centidistance,
-		DRC_XPLANE|DRC_FORWARD|DRC_BOTHCORNERS,layer[0]);
+		DRC_FORWARD|DRC_BOTHCORNERS,layer[0],0);
          drcCifRules[layer[1]][DRC_CIF_SOLID] = dpnew;
 	 
 	 if (layer[0] == layer[1])
@@ -292,14 +310,14 @@ drcCifSpacing(argc, argv)
               dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
               drcAssign(dpnew,centidistance,dpnext,DBSpaceBits,
     		     cmask,why,centidistance,
-		     DRC_XPLANE|DRC_REVERSE|DRC_BOTHCORNERS,layer[0]);
+		     DRC_REVERSE|DRC_BOTHCORNERS,layer[0],0);
                  drcCifRules[layer[1]][DRC_CIF_SPACE] = dpnew;
 	 
               dpnext = drcCifRules[layer[0]][DRC_CIF_SPACE];
               dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
               drcAssign(dpnew,centidistance,dpnext,DBSpaceBits,
     		     cmask,why,centidistance,
-		     DRC_XPLANE|DRC_REVERSE|DRC_BOTHCORNERS,layer[1]);
+		     DRC_REVERSE|DRC_BOTHCORNERS,layer[1],0);
                  drcCifRules[layer[0]][DRC_CIF_SPACE] = dpnew;
 	 }
     }
@@ -310,14 +328,14 @@ drcCifSpacing(argc, argv)
          dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
          drcAssign(dpnew,scalefactor,dpnext,DBSpaceBits,
     		DBZeroTypeBits,why,scalefactor,
-		DRC_FORWARD,layer[0]);
+		DRC_FORWARD,layer[0],0);
          drcCifRules[layer[1]][DRC_CIF_SPACE] = dpnew;
 
          dpnext = drcCifRules[layer[0]][DRC_CIF_SPACE];
          dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
          drcAssign(dpnew,scalefactor,dpnext,DBSpaceBits,
     		DBZeroTypeBits,why,scalefactor,
-		DRC_FORWARD,layer[1]);
+		DRC_FORWARD,layer[1],0);
          drcCifRules[layer[0]][DRC_CIF_SPACE] = dpnew;
     }
     
@@ -378,6 +396,7 @@ drcCifInit()
     }
     drcCifValid = FALSE;
     TTMaskZero(&drcCifGenLayers);
+    beenWarned = FALSE;
 }
 
 /*
@@ -623,7 +642,7 @@ drcCifTile (tile, arg)
 		    if (LEFT(tpleft) <= errRect.r_xbot
 			&& BOTTOM(tpleft) <= errRect.r_ybot
 			&& TOP(tpleft) >= errRect.r_ytop
-			&& !(cptr->drcc_flags & DRC_XPLANE)
+			&& arg->dCD_plane == cptr->drcc_plane
 			&& TTMaskHasType(&cptr->drcc_mask, TiGetType(tpleft)))
 			    continue;
 		    errRect.r_xtop = edgeX;
@@ -669,7 +688,7 @@ drcCifTile (tile, arg)
 		    if (RIGHT(tile) >= errRect.r_xtop
 			&& BOTTOM(tile) <= errRect.r_ybot
 			&& TOP(tile) >= errRect.r_ytop
-			&& !(cptr->drcc_flags & DRC_XPLANE)
+			&& arg->dCD_plane == cptr->drcc_plane
 			&& TTMaskHasType(&cptr->drcc_mask, TiGetLeftType(tile)))
 			    continue;
 		    errRect.r_xbot = edgeX;
@@ -800,7 +819,7 @@ drcCifTile (tile, arg)
 		    if (BOTTOM(tpbot) <= errRect.r_ybot
 			&& LEFT(tpbot) <= errRect.r_xbot
 			&& RIGHT(tpbot) >= errRect.r_xtop
-			&& !(cptr->drcc_flags & DRC_XPLANE)
+			&& arg->dCD_plane == cptr->drcc_plane
 			&& TTMaskHasType(&cptr->drcc_mask, TiGetTopType(tpbot)))
 			    continue;
 		    errRect.r_ytop = edgeY;
@@ -847,7 +866,7 @@ drcCifTile (tile, arg)
 		    if (TOP(tile) >= errRect.r_ytop
 			&& LEFT(tile) <= errRect.r_xbot
 			&& RIGHT(tile) >= errRect.r_xtop
-			&& !(cptr->drcc_flags & DRC_XPLANE)
+			&& arg->dCD_plane == cptr->drcc_plane
 			&& TTMaskHasType(&cptr->drcc_mask, TiGetType(tile)))
 			    continue;
 		    errRect.r_ybot = edgeY;
@@ -965,7 +984,7 @@ drcCifArea(argc, argv)
     char *argv[];
 {
     char *layers = argv[1];
-    int centidistance = atoi(argv[2]);
+    int centiarea = atoi(argv[2]);
     int	centihorizon = atoi(argv[3]);
     char *why = drcWhyDup(argv[4]);
     TileTypeBitMask set, setC, tmp1;
@@ -976,10 +995,8 @@ drcCifArea(argc, argv)
     int scalefactor;
 
     if (drcCifStyle == NULL)
-    {
-    	 TechError("Missing cif style for drc\n");
-	 return 0;
-    } 
+	return drcCifWarning();
+
     for (i = 0; i < drcCifStyle->cs_nLayers;i++)
     {
     	 CIFLayer	*layer = drcCifStyle->cs_layers[i];
@@ -997,10 +1014,11 @@ drcCifArea(argc, argv)
     }
 
     scalefactor = drcCifStyle->cs_scaleFactor;
+    centiarea *= (drcCifStyle->cs_expander * drcCifStyle->cs_expander);
     dpnext = drcCifRules[thislayer][DRC_CIF_SPACE];
     dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
     drcAssign(dpnew,centihorizon,dpnext,CIFSolidBits,
-    		CIFSolidBits,why,centidistance,DRC_AREA|DRC_FORWARD,thislayer);
+    		CIFSolidBits,why,centiarea,DRC_AREA|DRC_FORWARD,thislayer,0);
     drcCifRules[thislayer][DRC_CIF_SPACE] = dpnew;
 
 
@@ -1039,10 +1057,8 @@ drcCifMaxwidth(argc, argv)
     int scalefactor;
 
     if (drcCifStyle == NULL)
-    {
-    	 TechError("Missing cif style for drc\n");
-	 return 0;
-    } 
+	return drcCifWarning();
+
     for (i = 0; i < drcCifStyle->cs_nLayers;i++)
     {
     	 CIFLayer	*layer = drcCifStyle->cs_layers[i];
@@ -1068,10 +1084,11 @@ drcCifMaxwidth(argc, argv)
     }
     
     scalefactor = drcCifStyle->cs_scaleFactor;
+    centidistance *= drcCifStyle->cs_expander;		// BSI
     dpnext = drcCifRules[thislayer][DRC_CIF_SPACE];
     dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
     drcAssign(dpnew,centidistance,dpnext,CIFSolidBits,
-    		CIFSolidBits,why,centidistance,DRC_MAXWIDTH|bend,thislayer);
+    		CIFSolidBits,why,centidistance,DRC_MAXWIDTH|bend,thislayer,0);
     drcCifRules[thislayer][DRC_CIF_SPACE] = dpnew;
 
 
