@@ -19,13 +19,14 @@
  */
 
 #ifndef	lint
-static char rcsid[] = "$Header: /ufs/repository/magic/drc/DRCmain.c,v 1.2 2001/01/12 22:12:29 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCmain.c,v 1.2 2006/04/15 01:04:50 anonymous Exp $";
 #endif	/* not lint */
 
 #include <sys/types.h>
 #include <stdio.h>
 
 #include "utils/magic.h"
+#include "utils/malloc.h"
 #include "textio/textio.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
@@ -49,6 +50,14 @@ extern void TxError(char *, ...);
  * variables.
  */
 
+/* The following points to a list of all the DRC styles currently
+ * understood:
+ */
+
+DRCKeep *DRCStyleList = NULL;
+
+DRCStyle *DRCCurStyle = NULL;
+
 /* Used by both routines: */
 
 int DRCErrorCount;		/* Incremented by each call to either routine.
@@ -65,12 +74,6 @@ HashTable DRCErrorTable;	/* Hash table used to eliminate duplicate
 				 * error strings.
 				 */
 
-/* Values shared by DRCFind and drcFindFunc: */
-
-Rect *drcFindRect;		/* Rectangle to fill in with found error. */
-int drcFindIndex = 0;		/* Index of error to return. */
-int drcCurrentIndex;
-
 /* Global variables used by all DRC modules to record statistics.
  * For each statistic we keep two values, the count since stats
  * were last printed (in DRCstatXXX), and the total count (in
@@ -193,11 +196,11 @@ drcPrintError (celldef, rect, cptr, area)
     if ((area != NULL) && (!GEO_OVERLAP(area, rect))) return;
     DRCErrorCount += 1;
     h = HashFind(&DRCErrorTable, cptr->drcc_why);
-    i = (int) HashGetValue(h);
+    i = (spointertype) HashGetValue(h);
     if (i == 0)
 	TxPrintf("%s\n", cptr->drcc_why);
     i += 1;
-    HashSetValue(h, i);
+    HashSetValue(h, (spointertype)i);
 }
 
 /*
@@ -311,7 +314,7 @@ DRCWhy(use, area)
 
     /* Create a hash table to used for eliminating duplicate messages. */
 
-    HashInit(&DRCErrorTable, 16, 0);
+    HashInit(&DRCErrorTable, 16, HT_STRINGKEYS);
     DRCErrorCount = 0;
     box = DRCdef->cd_bbox;
 
@@ -373,7 +376,7 @@ drcWhyFunc(scx, cdarg)
 
     /* Check paint and interactions in this subcell. */
     
-    GEO_EXPAND(&scx->scx_area, TechHalo, &haloArea);
+    GEO_EXPAND(&scx->scx_area, DRCTechHalo, &haloArea);
     (void) DRCBasicCheck(def, &haloArea, &scx->scx_area,
 	drcPrintError, (ClientData) &scx->scx_area);
     (void) DRCInteractionCheck(def, &scx->scx_area,
@@ -445,7 +448,7 @@ drcCheckFunc(scx, cdarg)
     cellArea = scx->scx_area;
     def = scx->scx_use->cu_def;
     GeoClip(&cellArea, &def->cd_bbox);
-    GEO_EXPAND(&cellArea, TechHalo, &cellArea);
+    GEO_EXPAND(&cellArea, DRCTechHalo, &cellArea);
 
     DBPaintPlane(def->cd_planes[PL_DRC_CHECK], &cellArea,
 		DBStdPaintTbl(TT_CHECKPAINT, PL_DRC_CHECK),
@@ -476,30 +479,32 @@ drcCheckFunc(scx, cdarg)
  *	that cell and outputs the counts.
  *
  * Results:
- *	None.
+ *	Return linked list of cell definitions and their error counts.
  *
  * Side effects:
- *	None, except for the text output.
+ *	None.
  *
  * ----------------------------------------------------------------------------
  */
 
-void
+DRCCountList *
 DRCCount(use, area)
     CellUse *use;		/* Top-level use of hierarchy. */
     Rect *area;			/* Area in which violations are counted. */
 {
-    HashTable dupTable;
+    DRCCountList  *dcl, *newdcl;
+    HashTable	  dupTable;
+    HashEntry	  *he;
+    HashSearch	  hs;
+    int		  count;
     SearchContext scx;
-    extern int drcCountFunc();	/* Forward reference. */
+    extern int    drcCountFunc();	/* Forward reference. */
 
     /* Use a hash table to make sure that we don't output information
      * for any cell more than once.
      */
 
-    HashInit(&dupTable, 16, 1);
-
-    DBCellReadArea(use, area);
+    HashInit(&dupTable, 16, HT_WORDKEYS);
 
     scx.scx_use = use;
     scx.scx_x = use->cu_xlo;
@@ -508,15 +513,33 @@ DRCCount(use, area)
     scx.scx_trans = GeoIdentityTransform;
     (void) drcCountFunc(&scx, &dupTable);
 
-    /* Clean up the hash table to avoid core leak. */
+    /* Create the list from the hash table */
 
+    dcl = NULL;
+    if (dupTable.ht_table != (HashEntry **) NULL)
+    {
+	HashStartSearch(&hs);
+	while ((he = HashNext(&dupTable, &hs)) != (HashEntry *)NULL)
+	{
+            count = (spointertype)HashGetValue(he);
+	    if (count > 1)
+	    {
+		newdcl = (DRCCountList *)mallocMagic(sizeof(DRCCountList));
+		newdcl->dcl_count = count - 1;
+		newdcl->dcl_def = (CellDef *)he->h_key.h_ptr;
+		newdcl->dcl_next = dcl;
+		dcl = newdcl;
+	    }
+	}
+    }
     HashKill(&dupTable);
+    return dcl;
 }
 
 int
 drcCountFunc(scx, dupTable)
     SearchContext *scx;
-    HashTable *dupTable;		/* Passsed as client data, used to
+    HashTable *dupTable;		/* Passed as client data, used to
 					 * avoid searching any cell twice.
 					 */
 {
@@ -530,7 +553,7 @@ drcCountFunc(scx, dupTable)
      */
 
     def = scx->scx_use->cu_def;
-    h = HashFind(dupTable, (char *) def);
+    h = HashFind(dupTable, def);
     if (HashGetValue(h) != 0) goto done;
     HashSetValue(h, 1);
 
@@ -539,10 +562,15 @@ drcCountFunc(scx, dupTable)
     count = 0;
     (void) DBSrPaintArea((Tile *) NULL, def->cd_planes[PL_DRC_ERROR],
 	&def->cd_bbox, &DBAllButSpaceBits, drcCountFunc2, (ClientData) &count);
-    if (count > 1)
-        TxPrintf("Cell %s has %d error tiles.\n", def->cd_name, count);
-    else if (count == 1)
-	TxPrintf("Cell %s has just one error tile.\n", def->cd_name);
+    HashSetValue(h, (spointertype)count + 1);
+
+    /* Ignore children that have not been loaded---we will only report	*/
+    /* errors that can be seen.  This avoids immediately loading and	*/
+    /* drc processing large layouts simply because we asked for an	*/
+    /* error count.  When the cell is loaded, drc will be checked	*/
+    /* anyway, and the count can be updated in response to that check.	*/
+
+    if ((scx->scx_use->cu_def->cd_flags & CDAVAILABLE) == 0) return 0;
 
     /* Scan children recursively. */
 
@@ -598,15 +626,16 @@ DRCCatchUp()
  *
  * DRCFind --
  *
- * 	Locates the next violation tile in the cell pointed to by def.
+ * 	Locates the next violation tile in the cell pointed to by
+ *	"use" and its children.
  *	Successive calls will located successive violations, in
  *	circular order.
  *
  * Results:
  *	If an error tile was found in def, returns the indx of
  *	the error tile (> 0).  Returns 0 if there were no error
- *	tiles in def, or if a specific indx was given and there
- *	weren't that many error tiles in def.
+ *	tiles in def.  Returns -1 if indx was out-of-range
+ *	(not that many errors in def).
  *
  * Side effects:
  *	Rect is filled with the location of the tile, if one is found.
@@ -614,52 +643,98 @@ DRCCatchUp()
  * ----------------------------------------------------------------------------
  */
 
+typedef struct {
+    int current;	/* count of current error */
+    int target;		/* count of target error */
+    Rect *rect;		/* Return rectangle */
+    Transform trans;	/* Return transform */
+    HashTable *deft;	/* Table of cell definitions to avoid duplicates */
+} Sindx;
+
 int
-DRCFind(def, rect, indx)
-    CellDef *def;		/* Cell definition to check. */
+DRCFind(use, area, rect, indx)
+    CellUse *use;		/* Cell use to check. */
+    Rect *area;			/* Area of search */
     Rect *rect;			/* Rectangle to fill in with tile location. */
-    int indx;			/* If > 1, go to this error.  If < 1, just go
-				 * to the next error.
-				 */
+    int indx;			/* Go to this error. */
 {
+    SearchContext scx;
+    Sindx finddata;
+    Rect trect;	
+    int result;
     int drcFindFunc();
+    HashTable defTable;
 
-    drcFindRect = rect;
-    if (indx > 0) drcFindIndex = indx;
-    else drcFindIndex += 1;
+    scx.scx_use = use;
+    scx.scx_x = use->cu_xlo;
+    scx.scx_y = use->cu_ylo;
+    scx.scx_area = *area;
+    scx.scx_trans = GeoIdentityTransform;
 
-    /* We may have to make two searches (the second is necessary if
-     * we've seen the last error in the cell;  it goes back to the
-     * first again).
-     */
+    HashInit(&defTable, 16, HT_WORDKEYS);
 
-    (void) DBCellRead(def, (char *) NULL, TRUE);
-    while (TRUE)
+    finddata.current = 0;
+    finddata.target = indx;
+    finddata.rect = &trect;
+    finddata.trans = scx.scx_trans;
+    finddata.deft = &defTable;
+
+    result = drcFindFunc(&scx, &finddata);
+
+    HashKill(&defTable);
+
+    if (result == 0)
     {
-	drcCurrentIndex = 1;
-	if (DBSrPaintArea((Tile *) NULL, def->cd_planes[PL_DRC_ERROR],
-	    &def->cd_bbox, &DBAllButSpaceBits, drcFindFunc,
-	    (ClientData) NULL) != 0)
-	{
-	    return drcFindIndex;
-	}
-	if ((drcFindIndex == 1) || (indx > 0)) return FALSE;
-	drcFindIndex = 1;
+	if (finddata.current == 0)
+	    return 0;
+	else
+	    return -1;
     }
+
+    /* Translate rectangle back into coordinate system of "use" */
+    GeoTransRect(&finddata.trans, &trect, rect);
+    return indx;
 }
 
 int
-drcFindFunc(tile, dummy)
+drcFindFunc(scx, finddata)
+    SearchContext *scx;
+    Sindx *finddata;
+{
+    CellDef *def;
+    HashEntry *h;
+    int drcFindFunc2();
+
+    def = scx->scx_use->cu_def;
+    h = HashFind(finddata->deft, def);
+    if (HashGetValue(h) != 0) return 0;
+    HashSetValue(h, 1);
+
+    (void) DBCellRead(def, (char *) NULL, TRUE);
+
+    if (DBSrPaintArea((Tile *) NULL, def->cd_planes[PL_DRC_ERROR],
+	    &def->cd_bbox, &DBAllButSpaceBits, drcFindFunc2,
+	    (ClientData)finddata) != 0)
+    {
+	finddata->trans = scx->scx_trans;
+	return 1;
+    }
+    
+    /* Recursively search children */
+    return DBCellSrArea(scx, drcFindFunc, (ClientData)finddata);
+}
+
+int
+drcFindFunc2(tile, finddata)
     Tile *tile;			/* Tile in error plane. */
-    ClientData dummy;		/* Not used. */
+    Sindx *finddata;		/* Information about error to find */
 
 {
     if (TiGetType(tile) == (TileType) TT_SPACE) return 0;
-    if (drcCurrentIndex == drcFindIndex)
+    if (++finddata->current == finddata->target)
     {
-	TiToRect(tile, drcFindRect);
+	TiToRect(tile, finddata->rect);
 	return 1;
     }
-    drcCurrentIndex += 1;
     return 0;
 }
