@@ -19,7 +19,7 @@
  */
 
 #ifndef	lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCsubcell.c,v 1.2 2007/10/11 17:23:51 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCsubcell.c,v 1.3 2007/12/24 22:56:44 tim Exp $";
 #endif	/* not lint */
 
 #include <stdio.h>
@@ -58,7 +58,7 @@ static ClientData drcSubClientData;
 static DRCCookie drcSubcellCookie = {
     0, 0, 0, 0,
     { 0 }, { 0 },
-    0, 0,
+    0, 0, 0,
     "This layer can't abut or partially overlap between subcells",
     (DRCCookie *) NULL
 };
