@@ -17,7 +17,7 @@
  *
  * Needs to include: magic.h database.h
  *
- * rcsid $Header: /usr/cvsroot/magic-7.5/drc/drc.h,v 1.7 2007/10/12 22:14:36 tim Exp $
+ * rcsid $Header: /usr/cvsroot/magic-7.5/drc/drc.h,v 1.10 2008/01/05 04:17:10 tim Exp $
  */
 
 #ifndef _DRC_H
@@ -29,50 +29,52 @@
 
 typedef struct drccookie
 {
-    short    	      drcc_dist;	/* Extent of rule from edge. */
+    int    	      drcc_dist;	/* Extent of rule from edge. */
     unsigned char     drcc_mod;		/* Fractional part of drcc_dist */
-    short	      drcc_cdist;	/* Size of corner extension. */
+    int	      	      drcc_cdist;	/* Size of corner extension.* */
     unsigned char     drcc_cmod;	/* Fractional part of drcc_cdist */
     TileTypeBitMask   drcc_mask;	/* Legal types on RHS */
     TileTypeBitMask   drcc_corner;	/* Types that trigger corner check */
-    int		      drcc_flags;	/* Miscellaneous flags, see below. */
+    unsigned char     drcc_flags;	/* Miscellaneous flags, see below. */
+    int		      drcc_edgeplane;	/* Plane of edge */
     int		      drcc_plane;	/* Index of plane on which to check
 					 * legal types. */
     char              *drcc_why;	/* Explanation of error found */
     struct drccookie  *drcc_next;
 } DRCCookie;
 
+/* *This is size "int" because it holds an area for DRC_AREA rules,	  */
+/* and therefore may have twice the bit length of a normal rule distance. */
+
 /* DRCCookie flags:
  * DRC_FORWARD:		Rule applies from left to right (or bottom to top).
  * DRC_REVERSE:		Rule applies from right to left (or top to bottom).
  * DRC_BOTHCORNERS:	Must make corner extensions in both directions.
- * DRC_XPLANE:		Means check areas for rule are on different plane
- *			than edges from which rule triggers.
  */
 
 #define		DRC_FORWARD		0x0
 #define		DRC_REVERSE		0x1
 #define		DRC_BOTHCORNERS		0x2
-#define		DRC_XPLANE		0x4
-#define		DRC_CHECKCONNECT	0x10
-#define		DRC_ZEROSPACERULE	0x20
-#define		DRC_LEFT		0x100
-#define		DRC_RIGHT		0x200
-#define		DRC_TOP			0x400
-#define		DRC_BOTTOM		0x800
-#define		DRC_TRIGGER		0x1000
-
-#define		DRC_AREA		0x8000
-#define		DRC_MAXWIDTH		0x10000
-#define		DRC_BENDS		0x20000
-#define		DRC_RECTSIZE		0x40000
-#define		DRC_ANGLES		0x80000
-#define 	DRC_WEIRDONES		(DRC_AREA|DRC_MAXWIDTH|DRC_RECTSIZE|DRC_ANGLES)
+#define		DRC_TRIGGER		0x4
+#define		DRC_AREA		0x8
+#define		DRC_MAXWIDTH		0x10
+#define		DRC_BENDS		0x20
+#define		DRC_RECTSIZE		0x40
+#define		DRC_ANGLES		0x80
+#define 	DRC_WEIRD_ONES		(DRC_AREA|DRC_MAXWIDTH|DRC_RECTSIZE|DRC_ANGLES)
 
 #define	DRC_PENDING			0
 #define DRC_UNPROCESSED 		CLIENTDEFAULT
 #define DRC_PROCESSED 			1
 
+/*
+ * Background DRC (DRC Idle proc) for Tcl-based Magic
+ */
+#ifdef MAGIC_WRAPPER
+enum { DRC_NOT_RUNNING, DRC_IN_PROGRESS, DRC_BREAK_PENDING };
+extern unsigned char DRCStatus;
+#endif
+
 /* This is client data passed down through the various DRC checking
  * routines, and contains information about the area and rule being
  * checked.
