@@ -13,7 +13,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/ext2spice/ext2spice.c,v 1.4 2006/09/01 18:37:05 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/ext2spice/ext2spice.c,v 1.5 2008/01/03 07:48:15 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -2242,6 +2242,9 @@ spcnodeVisit(node, res, cap)
     if (!isConnected && esDevNodesOnly) 
 	return 0;
 
+    /* Don't mark known ports as "FLOATING" nodes */
+    if (!isConnected && node->efnode_flags & EF_PORT) isConnected = TRUE;
+
     hierName = (HierName *) node->efnode_name->efnn_hier;
     nsn = nodeSpiceName(hierName);
 
