@@ -16,7 +16,7 @@
  *     * of the United States of America may require an export license.    * 
  *     *********************************************************************
  *
- * rcsid $Header: /ufs/repository/magic/extflat/EFint.h,v 1.2 2001/01/12 22:12:35 jsolomon Exp $
+ * rcsid $Header: /usr/cvsroot/magic-7.5/extflat/EFint.h,v 1.2 2006/05/12 06:21:47 tim Exp $
  */
 
 #ifndef _EFINT_H
@@ -151,7 +151,7 @@ typedef struct conn
 typedef struct def
 {
     char	*def_name;	/* Name of this def */
-    int		 def_scale;	/* Multiply all dimensions by this */
+    float	 def_scale;	/* Multiply all dimensions by this */
     int		 def_flags;	/* Flags -- see below */
     HashTable	 def_nodes;	/* Map names into EFNodeNames */
     HashTable	 def_dists;	/* Map pairs of names into Distances */
@@ -298,7 +298,7 @@ extern bool efHNUseCompare();
 extern int efHNUseHash();
 
 extern EFCapValue CapHashGetValue();
-extern int CapHashSetValue();
+extern void CapHashSetValue();
 
    /* efBuild procedures */
 /* Warning: The capacitance argument to these should be always double
