@@ -16,7 +16,7 @@
  *     * of the United States of America may require an export license.    * 
  *     *********************************************************************
  *
- * rcsid $Header: /ufs/repository/magic/extflat/extflat.h,v 1.3 2001/01/12 22:12:36 jsolomon Exp $
+ * rcsid $Header: /usr/cvsroot/magic-7.5/extflat/extflat.h,v 1.2 2006/05/12 06:21:47 tim Exp $
  */
 
 #ifndef _EXTFLAT_H
@@ -33,6 +33,7 @@ typedef unsigned char U_char;
 #define	EF_FLATCAPS		0x02	/* Flatten capacitors */
 #define	EF_FLATRESISTS		0x04	/* Flatten resistors */
 #define	EF_FLATDISTS		0x08	/* Flatten distances */
+#define	EF_NOFLATSUBCKT		0x10	/* Don't flatten standard cells */
 
 /* Flags to control output of node names.  Stored in EFTrimFlags */
 #define	EF_TRIMGLOB	0x01	/* Delete trailing '!' from names */
@@ -271,7 +272,7 @@ typedef struct dev
 #define	FNSIZE		1024
 
 
-extern int EFScale;		/* Scale factor to multiply all coords by */
+extern float EFScale;		/* Scale factor to multiply all coords by */
 extern char *EFTech;		/* Technology of extracted circuit */
 extern char *EFStyle;           /* Extraction style of extracted circuit */
 extern char *EFSearchPath;	/* Path to search for .ext files */
