@@ -13,6 +13,7 @@
 
 #include <GL/gl.h>
 #include <GL/glx.h>
+#include <GL/glu.h>
 
 #include "tcltk/tclmagic.h"
 #include "utils/magic.h"
@@ -26,6 +27,7 @@
 #include "utils/hash.h"
 #include "graphics/grTOGLInt.h"
 #include "graphics/grTkCommon.h"
+#include "database/fonts.h"
 
 extern Display *grXdpy;
 
