@@ -9,17 +9,16 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/lef/defRead.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";            
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/lef/defRead.c,v 1.8 2006/11/02 19:16:34 tim Exp $";            
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
-#ifdef SYSV
 #include <string.h>
-#endif
 #include <errno.h>
 #include <stdarg.h>
+#include <math.h>		/* for roundf() function, if std=c99 */
 
 #include "tcltk/tclmagic.h"
 #include "utils/magic.h"
@@ -121,7 +120,7 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
 		    continue;
 		}
 		if (w != 0)
-		    paintWidth = (int)(w / oscale);
+		    paintWidth = (int)roundf(w / oscale);
 		else
 		    paintWidth = (lefl) ? lefl->info.route.width : DEFAULT_WIDTH;
 		saveWidth = paintWidth;
@@ -266,7 +265,7 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
 	    }
 	    else if (sscanf(token, "%f", &x) == 1)
 	    {
-		refp.p_x = (int)((2 * x) / oscale);
+		refp.p_x = (int)roundf((2 * x) / oscale);
 	    }
 	    else
 	    {
@@ -286,7 +285,7 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
 	    }
 	    else if (sscanf(token, "%f", &y) == 1)
 	    {
-		refp.p_y = (int)((2 * y) / oscale);
+		refp.p_y = (int)roundf((2 * y) / oscale);
 	    }
 	    else
 	    {
@@ -306,6 +305,12 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
 		/* non-default route extension */
 		if (sscanf(token, "%f", &z) != 1)
 		    LefError("Can't parse route extension value.\n");
+
+		/* all values will be divided by 2, so we need	*/
+		/* to multiply up by 2 now.			*/
+
+		else
+		    z *= 2;
 	    }
 
 	    /* Indicate that we have a valid reference point */
@@ -624,7 +629,7 @@ DefReadLocation(use, f, oscale)
     }
     GeoTransRect(&t, r, &tr);
     GeoTranslateTrans(&t, -tr.r_xbot, -tr.r_ybot, &t2);
-    GeoTranslateTrans(&t2, (int)(x / oscale), (int)(y / oscale), &t);
+    GeoTranslateTrans(&t2, (int)roundf(x / oscale), (int)roundf(y / oscale), &t);
     DBSetTrans(use, &t);
     return;
 
@@ -667,7 +672,7 @@ DefReadVias(f, sname, oscale, total)
     int keyword, subkey, values;
     int processed = 0;
     TileType curlayer;
-    Rect viaArea, *currect;
+    Rect *currect;
     lefLayer *lefl;
     HashEntry *he;
 
@@ -701,7 +706,6 @@ DefReadVias(f, sname, oscale, total)
 		/* processed and spit out a message for every 5% done.	*/
  
 		LefEstimate(processed++, total, "vias");
-		viaArea = GeoNullRect;
 
 		/* Get via name */
 		token = LefNextToken(f, TRUE);
@@ -725,6 +729,11 @@ DefReadVias(f, sname, oscale, total)
 		    HashSetValue(he, lefl);
 		    lefl->canonName = (char *)he->h_key.h_name;
 		}
+		else
+		{
+		    LefError("Warning:  Composite via \"%s\" redefined.\n", vianame);
+		    lefl = LefRedefined(lefl, vianame);
+		}
 
 		/* Now do a search through the line for "+" entries	*/
 		/* And process each.					*/
@@ -746,18 +755,7 @@ DefReadVias(f, sname, oscale, total)
 		    {
 			case DEF_VIAS_PROP_RECT:
 			    curlayer = LefReadLayer(f, FALSE);
-
-			    /* Divide oscale by 2 so that the rectangle	*/
-			    /* read is in units of 1/2 lambda.		*/
-
-			    currect = LefReadRect(f, curlayer, (oscale / 2));
-			    if (currect)
-			    {
-				GeoInclude(currect, &viaArea);
-				if ((lefl->type < 0) || DBIsContact(curlayer))
-				    lefl->type = curlayer;
-				lefl->info.via.area = viaArea;
-			    }
+			    LefAddViaGeometry(f, lefl, curlayer, oscale);
 			    break;
 		    }
 		}
