@@ -20,7 +20,7 @@
  *     *********************************************************************
  *
  *
- * rcsid $Header: /usr/cvsroot/magic-7.5/mzrouter/mzInternal.h,v 1.8 2006/12/02 00:36:58 tim Exp $
+ * rcsid $Header: /usr/cvsroot/magic-7.5/mzrouter/mzInternal.h,v 1.10 2007/04/20 17:57:00 tim Exp $
  */
 
 #ifndef _MZINTERNAL_H
@@ -88,7 +88,7 @@ extern ClientData mzDebugID;
 #define DEF_EXPAND_ENDPOINTS 1
 #define MZ_EXPAND_START	     1	/* ClientData type for start tiles */
 #define MZ_EXPAND_DEST	     0  /* ClientData type for dest tiles */
-#define MZ_EXPAND_NONE	     MINFINITY  /* Normal ClientData type */
+#define MZ_EXPAND_NONE	     CLIENTDEFAULT  /* Normal ClientData type */
 
 /* If set only hints in toplevel cell are recognized - speeds up processing
  * of hint planes when there are lots of expanded subcells.
@@ -143,17 +143,19 @@ extern CellUse *mzResultUse;
     /* Block due to start or destination terminal */
 #define TT_SAMENODE_BLOCK (2 + TT_OFF)
     /* Can reach dest via contact */
-#define TT_ABOVE_WALK	  (3 + TT_OFF)
-#define TT_BELOW_WALK	  (4 + TT_OFF)
+#define TT_ABOVE_UD_WALK  (3 + TT_OFF)
+#define TT_BELOW_UD_WALK  (4 + TT_OFF)
+#define TT_ABOVE_LR_WALK  (5 + TT_OFF)
+#define TT_BELOW_LR_WALK  (6 + TT_OFF)
     /* Approach to dest area */
-#define TT_LEFT_WALK	  (5 + TT_OFF)  
-#define TT_RIGHT_WALK     (6 + TT_OFF)
-#define TT_TOP_WALK       (7 + TT_OFF)
-#define TT_BOTTOM_WALK    (8 + TT_OFF)
+#define TT_LEFT_WALK	  (7 + TT_OFF)  
+#define TT_RIGHT_WALK     (8 + TT_OFF)
+#define TT_TOP_WALK       (9 + TT_OFF)
+#define TT_BOTTOM_WALK    (10 + TT_OFF)
     /* route destination area */
-#define TT_DEST_AREA      (9 + TT_OFF)  
+#define TT_DEST_AREA      (11 + TT_OFF)  
     /* Can't route in this space */
-#define	TT_BLOCKED	  (10 + TT_OFF)	
+#define	TT_BLOCKED	  (12 + TT_OFF)	
 
 #define TT_MAXROUTETYPES (1 + TT_BLOCKED)
 
@@ -474,7 +476,8 @@ extern List *mzMarkedCellsList;
 #define RC_ROTINSIDE	64
 #define RC_BOUNDS	128
 #define RC_WALK		256
-#define RC_WALKC	512
-#define RC_DONE		1024
+#define RC_WALKUDC	512
+#define RC_WALKLRC	1024
+#define RC_DONE		2048
 
 #endif /* _MZINTERNAL_H */
