@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/mzrouter/mzSubrs.c,v 1.7 2006/10/15 03:33:27 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/mzrouter/mzSubrs.c,v 1.9 2007/04/20 18:04:33 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -524,13 +524,12 @@ mzConnectedSubcellFunc(scx, cdarg)
     CellUse *cu = scx->scx_use;
 
     /* If not already marked, mark celluse and add to marked list */
-    if((int)(cu->cu_client) == MZ_EXPAND_NONE)
+    if (cu->cu_client == (ClientData)MZ_EXPAND_NONE)
     {
-	cu->cu_client = (ClientData) MZ_EXPAND_DEST;
+	cu->cu_client = (ClientData)MZ_EXPAND_DEST;
 	LIST_ADD(cu, mzMarkedCellsList);
     }
 
-
     /* continue search */
     return (0);
 }
@@ -606,15 +605,29 @@ mzPaintContact(path, prev)
 
     /* compute bounds of contact tile */
     r.r_ll = path->rp_entry;
-    r.r_xtop = r.r_xbot + cWidth;
-    r.r_ytop = r.r_ybot + cWidth;
+
+    if (path->rp_orient == 'X')
+    {
+	r.r_xtop = r.r_xbot + cWidth;
+	r.r_ytop = r.r_ybot + rC->rc_routeType.rt_length;
+    }
+    else if (path->rp_orient == 'O')
+    {
+	r.r_xtop = r.r_xbot + rC->rc_routeType.rt_length;
+	r.r_ytop = r.r_ybot + cWidth;
+    }
+    else	/* Type "C", residues only */
+    {
+	r.r_xtop = r.r_xbot + cWidth;
+	r.r_ytop = r.r_ybot + cWidth;
+    }
 
     /* Paint the contact on all connected mask planes */
     /* (should just let DBPaint() do this. . . ) */
 
     if (DBIsContact(cType))
     {
-	if (path->rp_orient != 'X')
+	if (path->rp_orient != 'C')
 	{
 	    for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
 		if (PlaneMaskHasPlane(DBConnPlanes[cType], pNum))
