@@ -16,7 +16,7 @@
  *     * of the United States of America may require an export license.    * 
  *     *********************************************************************
  *
- * rcsid $Header: /usr/cvsroot/magic-7.5/plot/plotInt.h,v 1.1.1.1 2006/04/10 22:03:13 tim Exp $
+ * rcsid $Header: /usr/cvsroot/magic-7.5/plot/plotInt.h,v 1.4 2008/01/15 23:51:35 tim Exp $
  */
 
 #ifndef _PLOTINT_H
@@ -25,6 +25,8 @@
 #include "utils/magic.h"
 #include "utils/geometry.h"
 
+#define VERSATEC		/* Add this for HP plotter support */
+
 /* system V machines lack vfont.h, so include the defs below. */
 #if  !defined(SYSV) && !defined(__FreeBSD__) && !defined(__NetBSD__) && !defined(CYGWIN) && !defined(macosx)
 #include <vfont.h>
@@ -139,7 +141,7 @@ typedef struct font
     struct font *fo_next;		/* Next in list, or NULL for
 					 * end of list.
 					 */
-} Font;
+} RasterFont;
 
 /* Technology-file reading procedures: */
 
@@ -169,9 +171,10 @@ extern Raster *PlotNewRaster();
 extern void PlotFreeRaster();
 extern void PlotClearRaster();
 extern void PlotFillRaster();
+extern void PlotPolyRaster();
 extern int PlotDumpRaster();
 extern Stipple PlotBlackStipple;
-extern Font *PlotLoadFont();
+extern RasterFont *PlotLoadFont();
 extern void PlotTextSize();
 extern void PlotRasterText();
 extern int PlotSwapBytes();
@@ -210,9 +213,11 @@ extern char *PlotTempDirectory;
 extern char *PlotVersIdFont;
 extern char *PlotVersNameFont;
 extern char *PlotVersLabelFont;
-extern bool PlotVersColor;
+extern unsigned char PlotVersPlotType;
 #endif
 
+enum { VERSATEC_COLOR = 0, VERSATEC_BW, HPRTL, HPGL2 };
+
 /* for plotPixels.c: */
 #ifdef LLNL
 extern int PlotPixHeight;
@@ -225,6 +230,9 @@ extern bool PlotShowCellNames;
 /* for plotPnm.c */
 extern int PlotPNMmaxmem;
 extern int PlotPNMdownsample;
-extern int PlotPNMBG;
+extern unsigned char PlotPNMBG;
+#ifdef VERSATEC
+extern bool PlotPNMRTL;
+#endif
 
 #endif /* _PLOTINT_H */
