@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/plot/plotMain.c,v 1.1.1.1 2006/04/10 22:03:13 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/plot/plotMain.c,v 1.3 2008/01/15 23:51:35 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -63,6 +63,16 @@ static char *plotStyles[] =		/* Names of tech subsections. */
     NULL
 };
 
+/* These names need to match the plot types enumerated in plotInt.h */
+static char *plotTypeNames[] =
+{
+    "versatec_color",
+    "versatec_bw",
+    "hprtl",
+    "hpgl2",
+    NULL
+};
+
 
 static void (*plotInitProcs[])() =	/* Initialization procedures for
 					 * each style.
@@ -271,7 +281,6 @@ PlotPrintParams()
 {
     TxPrintf("General plotting parameters are:\n");
     TxPrintf("    showCellNames: %s\n", PlotShowCellNames ? "true" : "false");
-
     TxPrintf("");
     TxPrintf("Postscript plotting parameters are:\n");
     TxPrintf("    PS_cellIdFont:  \"%s\"\n", PlotPSIdFont);
@@ -295,8 +304,9 @@ PlotPrintParams()
     TxPrintf("    pnmbackground: %d\n", PlotPNMBG);
 
 #ifdef VERSATEC
+    TxPrintf("    pnmplotRTL: %s\n", PlotPNMRTL ? "true" : "false");
     TxPrintf("");
-    TxPrintf("Versatec plotting parameters are:\n");
+    TxPrintf("HP/Versatec plotting parameters are:\n");
     TxPrintf("    cellIdFont:    \"%s\"\n", PlotVersIdFont);
     TxPrintf("    cellNameFont:  \"%s\"\n", PlotVersNameFont);
     TxPrintf("    directory:     \"%s\"\n", PlotTempDirectory);
@@ -306,7 +316,7 @@ PlotPrintParams()
     TxPrintf("    spoolCommand:  \"%s\"\n", PlotVersCommand);
     TxPrintf("    swathHeight:   %d\n", PlotVersSwathHeight);
     TxPrintf("    width:         %d\n", PlotVersWidth);
-    TxPrintf("    color:         %s\n", PlotVersColor ? "true" : "false");
+    TxPrintf("    plotType:      %s\n", plotTypeNames[PlotVersPlotType]);
 #endif
 #ifdef LLNL
     TxPrintf("");
@@ -356,7 +366,8 @@ typedef enum {
 	SPOOLCOMMAND,
 	SWATHHEIGHT,
 	WIDTH,
-	COLOR,
+	PLOTTYPE,
+	PNMRTL,
 #endif
 #ifdef LLNL
 	PIXWIDTH,
@@ -397,7 +408,8 @@ PlotSetParam(name, value)
 	"spoolcommand",
 	"swathheight",
 	"width",
-	"color",
+	"plottype",
+	"pnmplotRTL",	  /* PNM output piped through an HP plotter */
 #endif
 #ifdef LLNL
 	"pixwidth",
@@ -512,7 +524,7 @@ PlotSetParam(name, value)
 	case DOTSPERINCH:
 	    if (!StrIsInt(value) || (i <= 0))
 	    {
-		TxError("DotsPerInch must be an integer greater than zero.]n");
+		TxError("DotsPerInch must be an integer greater than zero.\n");
 		return;
 	    }
 	    else PlotVersDotsPerInch = i;
@@ -520,16 +532,39 @@ PlotSetParam(name, value)
 	case PRINTER:
 	    StrDup(&PlotVersPrinter, value);
 	    break;
-	case COLOR:
-	    i = Lookup(value, tfNames);
+	case PLOTTYPE:
+	    i = Lookup(value, plotTypeNames);
 	    if (i < 0)
 	    {
-		TxError("Color can only be \"true\" or \"false\".\n");
+		int j;
+
+		TxError("%s is not a supported plot type.  Plot types are:\n");
+		for (j = 0 ; plotTypeNames[j] != NULL; j++)
+		{
+		    TxError("\t%s\n", plotTypeNames[j]);
+		}
 		return;
 	    }
-	    PlotVersColor = i;
-	    /* color plotting means our scanlines can only be 6848 (856 bytes) long */
-	    PlotVersWidth = i?6848:7040;
+	    PlotVersPlotType = i;
+	    switch(PlotVersPlotType)
+	    {
+		case VERSATEC_COLOR:
+		    PlotVersDotsPerInch = 215;
+		    PlotVersWidth = 7904;
+		    break;
+		case VERSATEC_BW:
+		    PlotVersDotsPerInch = 215;
+		    PlotVersWidth = 7904;
+		    break;
+		case HPRTL:
+		    PlotVersDotsPerInch = 316;
+		    PlotVersWidth = 2400;
+		    break;
+		case HPGL2:
+		    PlotVersDotsPerInch = 300;
+		    PlotVersWidth = 10650;
+		    break;
+	    }
 	    break;
 	case SPOOLCOMMAND:
 	    StrDup(&PlotVersCommand, value);
@@ -551,6 +586,16 @@ PlotSetParam(name, value)
 	    else PlotVersWidth = i;
 	    break;
 
+	case PNMRTL:
+	    i = Lookup(value, tfNames);
+	    if (i < 0)
+	    {
+		TxError("pnmplotRTL can only be \"true\" or \"false\".\n");
+		return;
+	    }
+	    PlotPNMRTL = i;
+	    break;
+
 #endif /* VERSATEC */
 
 #ifdef LLNL
