@@ -21,7 +21,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/plot/plotRutils.c,v 1.1.1.1 2006/04/10 22:03:13 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/plot/plotRutils.c,v 1.4 2008/01/11 20:11:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -30,6 +30,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 
 #include "utils/magic.h"
 #include "utils/geometry.h"
+#include "utils/geofast.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
@@ -99,7 +100,7 @@ static unsigned int singleBit[32] =
     0x08000000, 0x04000000, 0x02000000, 0x01000000
 };
 
-Font *PlotFontList;		/* Linked list of all fonts that have
+RasterFont *PlotFontList;	/* Linked list of all fonts that have
 				 * been read in so far.
 				 */
 
@@ -281,7 +282,128 @@ PlotClearRaster(raster, area)
 	right += raster->ras_intsPerLine;
     }
 }
-
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ *  PlotPolyRaster --
+ *
+ *	Fill a polygonal raster area, given the area of a triangular
+ *	tile (in pixel coordinates), information about what side and
+ *	direction are filled, and the area to clip to (also in raster
+ *	coordinates).  This is *not* a general-purpose polygon-filling
+ *	routine.  It can only handle clipped right triangles.
+ *
+ *  Results:
+ *  	None.
+ *
+ * Side effects:
+ *	None.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+PlotPolyRaster(raster, tileArea, clipArea, dinfo, stipple)
+    Raster *raster;	/* Pointer to raster whose bits are
+			 * to be filled in.
+			 */
+    Rect *tileArea;	/* Area of split tile, in pixel coordinates */
+    Rect *clipArea;	/* Area to clip, in pixel coordinates */
+    TileType dinfo;	/* Split tile side and direction information */
+    Stipple stipple;	/* Stipple pattern to be used to fill
+			 * in the raster area.
+			 */
+{
+    int *cur, *rasleft, *tbase, *right, *left;
+    int leftMask, rightMask, curStipple;
+    int line, width, height, locleft, locright;
+    Rect area;
+
+    /* Compute the address of the leftmost word in the topmost line
+     * to be filled, and the rightmost word in the topmost line to
+     * be filled.
+     */
+
+    area = *tileArea;
+    GEOCLIP(&area, clipArea);
+
+    /* Ensure that we have not been clipped out of existence */
+    if (area.r_xbot > area.r_xtop) return;
+    if (area.r_ybot >= area.r_ytop) return;
+    
+    rasleft = raster->ras_bits +
+	((raster->ras_height-1) - area.r_ytop)*raster->ras_intsPerLine;
+    width = tileArea->r_xtop - tileArea->r_xbot;
+    height = tileArea->r_ytop - tileArea->r_ybot;
+
+    /* Process the stippled area one raster line at a time, top to bottom. */
+
+    if (dinfo & TT_SIDE)
+    {
+	locright = area.r_xtop;
+	tbase = rasleft + area.r_xtop / 32;	/* base is on right */
+    }
+    else
+    {
+	locleft = area.r_xbot;
+	tbase = rasleft + area.r_xbot / 32;	/* base is on left */
+    }
+
+    for (line = area.r_ytop; line >= area.r_ybot; line -= 1)
+    {
+	if (dinfo & TT_SIDE)
+	{
+	    if (dinfo & TT_DIRECTION)
+		locleft = tileArea->r_xbot + (((tileArea->r_ytop - line) * width)
+			/ height);
+	    else
+		locleft = tileArea->r_xbot + (((line - tileArea->r_ybot) * width)
+			/ height);
+	    right = tbase;
+	    left = rasleft + locleft / 32;
+	}
+	else
+	{
+	    if (dinfo & TT_DIRECTION)
+		locright = tileArea->r_xbot + (((tileArea->r_ytop - line) * width)
+			/ height);  
+	    else
+		locright = tileArea->r_xbot + (((line - tileArea->r_ybot) * width)
+			/ height);
+	    right = rasleft + locright / 32;
+	    left = tbase;
+	}
+	if (left > right) continue;
+
+	leftMask = rightBits[locleft & 037];
+	rightMask = leftBits[locright & 037];
+	if (left == right)
+	    leftMask &= rightMask;
+    
+	curStipple = stipple[(-line)&017];
+
+	/* Fill the leftmost word on this line. */
+
+	*left |= curStipple & leftMask;
+
+	if (left != right)
+	{
+	    /* Fill the center of the line. */
+
+	    for (cur = left+1; cur < right; cur += 1)
+		*cur |= curStipple;
+	    
+	    /* Fill the rightmost word on this line. */
+
+	    *cur |= curStipple & rightMask;
+	}
+
+	rasleft += raster->ras_intsPerLine;
+	tbase += raster->ras_intsPerLine;
+    }
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -429,12 +551,12 @@ PlotDumpRaster(raster, file)
  * ----------------------------------------------------------------------------
  */
 
-Font *
+RasterFont *
 PlotLoadFont(name)
     char *name;			/* Name of font file. */
 {
     FILE *f;
-    Font *new;
+    RasterFont *new;
     struct dispatch *d;
 
     /* See if we've already got the font. */
@@ -452,7 +574,7 @@ PlotLoadFont(name)
 	return NULL;
     }
 
-    new = (Font *) mallocMagic(sizeof(Font));
+    new = (RasterFont *) mallocMagic(sizeof(RasterFont));
     new->fo_name = NULL;
     StrDup(&new->fo_name, name);
 
@@ -572,7 +694,7 @@ PlotLoadFont(name)
 
 void
 PlotTextSize(font, string, area)
-    Font *font;			/* Font to use in computing text size. */
+    RasterFont *font;		/* Font to use in computing text size. */
     char *string;		/* String to compute size of. */
     Rect *area;	/* Place to store bounding box. */
 {
@@ -626,7 +748,7 @@ PlotRasterText(raster, clip, font, string, point)
     Rect *clip;			/* Area to which to clip the text.  Must be
 				 * entirely within the area of the raster.
 				 */
-    Font *font;			/* Font to use for rasterizing string.  Must
+    RasterFont *font;		/* Font to use for rasterizing string.  Must
 				 * have been obtained by calling PlotLoadFont.
 				 */
     char *string;		/* String of text to rasterize. */
