@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/plot/plotVers.c,v 1.2 2006/04/15 03:36:47 anonymous Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/plot/plotVers.c,v 1.13 2008/01/29 02:41:57 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -39,6 +39,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 #include "utils/tech.h"
 #include "utils/signals.h"
 #include "dbwind/dbwind.h"
+#include "cif/cif.h"		/* for CIFGetOutputScale() */
 
 #ifdef VERSATEC
 
@@ -51,7 +52,6 @@ extern int rasFileByteCount;
  * stipple pattern.
  */
 
-
 typedef struct versatecstyle
 {
     TileTypeBitMask vs_layers;		/* Layers to plot in this style. */
@@ -96,10 +96,14 @@ char *plotVersatecColorNames[] = {
  * ----------------------------------------------------------------------------
  */
 
-int PlotVersWidth = 7040;	/* Number of dots across Versatec page.
+/* Supported format.  Default is "hprtl". */
+
+unsigned char PlotVersPlotType = HPRTL;
+
+int PlotVersWidth = 2400;	/* Number of dots across Versatec page.
 				 * Should be a multiple of 32.
 				 */
-int PlotVersDotsPerInch = 200;	/* Dots per inch. */
+int PlotVersDotsPerInch = 300;	/* Dots per inch. */
 int PlotVersSwathHeight = 64;	/* Width of swath to generate at one time,
 				 * in dots.
 				 */
@@ -109,22 +113,16 @@ int PlotVersSwathHeight = 64;	/* Width of swath to generate at one time,
 static char *defaultPrinter = "versatec";
 char *PlotVersPrinter = NULL;
 
-/* Should we produce Versatec plots in color or B&W?  "true" means color,
- * "false" means B&W
- */
-
-bool PlotVersColor = 0;
-
 /* Command to use to actually print rasterized file.  Contains two %s'es,
  * which are supplied with the printer name and the name of the raster file.
  */
 
-static char *defaultCommand = "/usr/ucb/lpr -P%s -c -s -r -JMagic %s";
+static char *defaultCommand = "lp -d %s %s";
 char *PlotVersCommand = NULL;
 
 /* Directory in which to create temporary file to hold raster: */
 
-static char *defaultDirectory = "/usr/tmp";
+static char *defaultDirectory = "/tmp";
 char *PlotTempDirectory = NULL;
 
 /* Name of fonts to use: */
@@ -155,7 +153,6 @@ char *PlotVersLabelFont = NULL;
  * ----------------------------------------------------------------------------
  */
 
-static Raster *raster = NULL;	/* Raster used to hold current swath. */
 Point plotLL;		/* Point in root Magic coords that corresponds
 				 * to (0,0) in raster coordinates.
 				 */
@@ -184,9 +181,9 @@ static TileTypeBitMask curMask; /* Mask of layers currently being stippled.
 static int crossSize;		/* Length of each arm of the crosses used
 				 * to draw labels, in pixel units.
 				 */
-static Font *labelFont;		/* Font to use when rendering labels. */
-static Font *cellNameFont;	/* Font to use when rendering cell names. */
-static Font *cellIdFont;	/* Font to use when rendering cell ids. */
+static RasterFont *labelFont;	 /* Font to use when rendering labels. */
+static RasterFont *cellNameFont; /* Font to use when rendering cell names. */
+static RasterFont *cellIdFont;	 /* Font to use when rendering cell ids. */
 
 #endif /* VERSATEC */
 
@@ -324,18 +321,6 @@ PlotVersTechLine(sectionName, argc, argv)
 
     DBTechNoisyNameMask(argv[0], &new->vs_layers);
 
-    /* I'm not sure why this transformation is done, and it doesn't work.
-     * --Bob
-     */
-#ifdef	NOT_DEF
-    /* Replace non-primary contact images with primary images. */
-    for (i = TT_TECHDEPBASE; i < DBNumTypes; i++)
-    {
-	if TTMaskHasType(&new->vs_layers, i)
-	    TTMaskSetMask(&new->vs_layers, &DBLayerTypeMaskTbl[i]);
-    }
-    TTMaskAndMask(&new->vs_layers, &DBUserLayerBits);
-#endif
     if (argc == 2)
     {
         if (strcmp(argv[1], "X") == 0)
@@ -417,27 +402,22 @@ PlotColorVersTechLine(sectionName, argc, argv)
 {
     VersatecStyle *new;
     static struct { char *l_str; int l_color; } colors[] = {
-	"black",	BLACK,
-	"cyan",		CYAN,
-	"magenta",	MAGENTA,
-	"yellow",	YELLOW,
+	"black",   BLACK,
+	"cyan",	   CYAN,
+	"magenta", MAGENTA,
+	"yellow",  YELLOW,
+	"K",	   BLACK,
+	"C",	   CYAN,
+	"M",	   MAGENTA,
+	"Y",	   YELLOW,
 	0
     };
     int i;
 
-    new = (VersatecStyle *) mallocMagic(sizeof(VersatecStyle));
+    new = (VersatecStyle *)mallocMagic(sizeof(VersatecStyle));
 
     DBTechNoisyNameMask(argv[0], &new->vs_layers);
 
-    /* Replace non-primary contact images with primary images. */
-    /****  I'm not sure why this is done, and it doesn't work.  --Bob
-    for (i = TT_TECHDEPBASE; i < DBNumTypes; i++)
-    {
-	if TTMaskHasType(&new->vs_layers, i)
-	    TTMaskSetMask(&new->vs_layers, &DBLayerTypeMaskTbl[i]);
-    }
-    TTMaskAndMask(&new->vs_layers, &DBUserLayerBits);
-    ****/
     if (argc == 2)
     {
 	new->vs_color = BLACK;
@@ -454,12 +434,12 @@ PlotColorVersTechLine(sectionName, argc, argv)
     }
     else
     {
-	int i, value;
+	int i, j, value;
 
-	if (argc != 18)
+	if (argc != 3 && argc != 4 && argc != 6 && argc != 10 && argc != 18)
 	{
-	    TechError(
-		"\"colorversatec\" lines must have either 2 or 18 fields.\n");
+	    TechError("\"colorversatec\" lines must have 2 fields + 1, 2, 4, 8,"
+		" or 16 stipple word values.\n");
 	    freeMagic((char *)new);
 	    return TRUE;
 	}
@@ -472,13 +452,16 @@ PlotColorVersTechLine(sectionName, argc, argv)
 	}
 	new->vs_color = colors[i].l_color;
 	new->vs_flags = 0;
-	for (i = 0; i < 16; i++)
+	for (j = 0; j < 16; j += (argc - 2))
 	{
-	    (void) sscanf(argv[i+2], "%x", &value);
-	    new->vs_stipple[i] = (value<<16) | (value & 0xffff);
+	    for (i = 0; i < (argc - 2); i++)
+	    {
+		sscanf(argv[i + 2], "%x", &value);
+		new->vs_stipple[j + i] = (value << 16) | (value & 0xffff);
 #ifndef WORDS_BIGENDIAN
-	    new->vs_stipple[i] = PlotSwapBytes(new->vs_stipple[i]);
+		new->vs_stipple[j + i] = PlotSwapBytes(new->vs_stipple[i]);
 #endif  /* WORDS_BIGENDIAN */
+	    }
 	}
     }
 
@@ -551,7 +534,7 @@ plotTransToSwath(src, dst)
  */
 
 void
-plotVersLine(area, widen)
+plotVersLine(area, widen, raster)
     Rect *area;			/* The "corner" points of this rectangle
 				 * give the endpoints of the line, in
 				 * Magic root coordinates.
@@ -560,6 +543,7 @@ plotVersLine(area, widen)
 				 * line is drawn one pixel wide, 1 means 3
 				 * pixels wide, etc.
 				 */
+    Raster *raster;		/* Raster to write to */
 {
     Rect swathArea;
 
@@ -599,7 +583,7 @@ plotVersLine(area, widen)
  */
 
 void
-plotVersRect(area, widen)
+plotVersRect(area, widen, raster)
     Rect *area;	/* Rectangular area to draw, in root
 				 * coordinates.
 				 */
@@ -608,6 +592,7 @@ plotVersRect(area, widen)
 				 * is widened by this many units on each
 				 * side.
 				 */
+    Raster *raster;		/* Raster to plot to */
 {
     Rect side;
 
@@ -617,7 +602,7 @@ plotVersRect(area, widen)
     {
 	side = *area;
 	side.r_ytop = side.r_ybot;
-	plotVersLine(&side, widen);
+	plotVersLine(&side, widen, raster);
 
 	/* Now the top side, if it doesn't coincide with the bottom. */
 
@@ -625,7 +610,7 @@ plotVersRect(area, widen)
 	{
 	    side = *area;
 	    side.r_ybot = side.r_ytop;
-	    plotVersLine(&side, widen);
+	    plotVersLine(&side, widen, raster);
 	}
     }
 
@@ -635,7 +620,7 @@ plotVersRect(area, widen)
     {
 	side = *area;
 	side.r_xtop = side.r_xbot;
-	plotVersLine(&side, widen);
+	plotVersLine(&side, widen, raster);
 
 	/* Now the right side, if it doesn't coincide with the left. */
 
@@ -643,7 +628,7 @@ plotVersRect(area, widen)
 	{
 	    side = *area;
 	    side.r_xbot = side.r_xtop;
-	    plotVersLine(&side, widen);
+	    plotVersLine(&side, widen, raster);
 	}
     }
 }
@@ -671,8 +656,10 @@ plotVersTile(tile, cxp)
     TreeContext *cxp;		/* Describes search in progress. */
 {
     Rect tileArea, rootArea, swathArea, edge;
+    TileType ntype;
     Tile *neighbor;
     Transform *trans = &cxp->tc_scx->scx_trans;
+    Raster *raster = (Raster *)cxp->tc_filter->tf_arg;
 
     /* Transform tile coords to root coords and then to swath coords. */
     
@@ -680,32 +667,64 @@ plotVersTile(tile, cxp)
     GEOTRANSRECT(trans, &tileArea, &rootArea);
     plotTransToSwath(&rootArea, &swathArea);
 
-    /* Handle bordered and X'ed things specially. */
+    /* Handle X'ed things specially. */
     
-    if (curStyle->vs_flags & (VS_CROSS|VS_BORDER))
+    if (curStyle->vs_flags & VS_CROSS)
     {
-	plotVersRect(&rootArea, 0);
-	if ((curStyle->vs_flags & VS_CROSS)
-	    && ((swathArea.r_xtop - swathArea.r_xbot) > 6)
-	    && ((swathArea.r_ytop - swathArea.r_ybot) > 6))
-	{
-	    Rect r2;
-	    plotVersLine(&rootArea, 0);
-	    r2.r_xtop = rootArea.r_xbot;
-	    r2.r_ybot = rootArea.r_ybot;
-	    r2.r_xbot = rootArea.r_xtop;
-	    r2.r_ytop = rootArea.r_ytop;
-	    plotVersLine(&r2, 0);
-	}
-	return 0;
+#ifdef NONMANHATTAN
+	if (!IsSplit(tile))
+#endif
+	    if (((swathArea.r_xtop - swathArea.r_xbot) > 6)
+			&& ((swathArea.r_ytop - swathArea.r_ybot) > 6))
+	    {
+		Rect r2;
+		plotVersLine(&rootArea, 0, raster);
+		r2.r_xtop = rootArea.r_xbot;
+		r2.r_ybot = rootArea.r_ybot;
+		r2.r_xbot = rootArea.r_xtop;
+		r2.r_ytop = rootArea.r_ytop;
+		plotVersLine(&r2, 0, raster);
+	    }
     }
 
+#ifdef NONMANHATTAN
+    if (IsSplit(tile))
+    {
+	int i, j;
+	TileType dinfo;
+	Rect r;
+
+	dinfo = DBTransformDiagonal(TiGetTypeExact(tile), &cxp->tc_scx->scx_trans);
+	if (!(curStyle->vs_flags & VS_BORDER) && !(curStyle->vs_flags & VS_CROSS))
+	    PlotPolyRaster(raster, &swathArea, &swathClip, dinfo,
+				curStyle->vs_stipple);
+
+	/* Diagonal is always drawn (clipping handled in plotVersLine) */
+
+	r = rootArea;
+	if (dinfo & TT_DIRECTION)
+	{
+	    /* swap X to make diagonal go the other way */
+	    r.r_xbot = r.r_xtop;
+	    r.r_xtop = rootArea.r_xbot;
+	}
+	plotVersLine(&r, 0, raster);
+    }
+    else
+    {
+#endif
+
     /* Clip and then stipple. */
 
     GEOCLIP(&swathArea, &swathClip);
     if (swathArea.r_xbot > swathArea.r_xtop) return 0;
     if (swathArea.r_ybot > swathArea.r_ytop) return 0;
-    PlotFillRaster(raster, &swathArea, curStyle->vs_stipple);
+    if (!(curStyle->vs_flags & VS_BORDER) && !(curStyle->vs_flags & VS_CROSS))
+	PlotFillRaster(raster, &swathArea, curStyle->vs_stipple);
+
+#ifdef NONMANHATTAN
+    }
+#endif
 
     /* Now output lines for any edges between material of the type
      * currently being drawn and material of other types.  This is
@@ -714,22 +733,34 @@ plotVersTile(tile, cxp)
      * (unless it is at infinity).
      */
     
+#ifdef NONMANHATTAN
+    if (IsSplit(tile) && (!(SplitSide(tile) ^ SplitDirection(tile))))
+	goto searchleft;	/* nothing on bottom of split */
+#endif
+
     if (tileArea.r_ybot > TiPlaneRect.r_ybot)
     {
 	edge.r_ybot = edge.r_ytop = tileArea.r_ybot;
 	for (neighbor = LB(tile); LEFT(neighbor) < tileArea.r_xtop;
 		neighbor = TR(neighbor))
 	{
-	    if (TTMaskHasType(&curMask, TiGetType(neighbor))) continue;
+	    ntype = TiGetTopType(neighbor);
+	    if (TTMaskHasType(&curMask, ntype)) continue;
 	    edge.r_xbot = LEFT(neighbor);
 	    edge.r_xtop = RIGHT(neighbor);
 	    if (edge.r_xbot < tileArea.r_xbot) edge.r_xbot = tileArea.r_xbot;
 	    if (edge.r_xtop > tileArea.r_xtop) edge.r_xtop = tileArea.r_xtop;
 	    GEOTRANSRECT(trans, &edge, &rootArea);
-	    plotVersLine(&rootArea, 0);
+	    plotVersLine(&rootArea, 0, raster);
 	}
     }
 
+searchleft:
+#ifdef NONMANHATTAN
+    if (IsSplit(tile) && (SplitSide(tile)))
+	goto searchtop;		/* Nothing on left side of split */
+#endif
+
     /* Now go along the tile's left border, doing the same thing.   Ignore
      * edges that are at infinity.
      */
@@ -740,49 +771,64 @@ plotVersTile(tile, cxp)
 	for (neighbor = BL(tile); BOTTOM(neighbor) < tileArea.r_ytop;
 		neighbor = RT(neighbor))
 	{
-	    if (TTMaskHasType(&curMask, TiGetType(neighbor))) continue;
+	    ntype = TiGetRightType(neighbor);
+	    if (TTMaskHasType(&curMask, ntype)) continue;
 	    edge.r_ybot = BOTTOM(neighbor);
 	    edge.r_ytop = TOP(neighbor);
 	    if (edge.r_ybot < tileArea.r_ybot) edge.r_ybot = tileArea.r_ybot;
-	    if (edge.r_ytop < tileArea.r_ytop) edge.r_ytop = tileArea.r_ytop;
+	    if (edge.r_ytop > tileArea.r_ytop) edge.r_ytop = tileArea.r_ytop;
 	    GEOTRANSRECT(trans, &edge, &rootArea);
-	    plotVersLine(&rootArea, 0);
+	    plotVersLine(&rootArea, 0, raster);
 	}
     }
 
     /* Same thing for the tile's top border. */
 
+searchtop:
+#ifdef NONMANHATTAN
+    if (IsSplit(tile) && (SplitSide(tile) ^ SplitDirection(tile)))
+	goto searchright;		/* Nothing on top side of tile */
+#endif
+
     if (tileArea.r_ytop < TiPlaneRect.r_ytop)
     {
 	edge.r_ybot = edge.r_ytop = tileArea.r_ytop;
 	for (neighbor = RT(tile); RIGHT(neighbor) > tileArea.r_xbot;
 		neighbor = BL(neighbor))
 	{
-	    if (TTMaskHasType(&curMask, TiGetType(neighbor))) continue;
+	    ntype = TiGetBottomType(neighbor);
+	    if (TTMaskHasType(&curMask, ntype)) continue;
 	    edge.r_xbot = LEFT(neighbor);
 	    edge.r_xtop = RIGHT(neighbor);
 	    if (edge.r_xbot < tileArea.r_xbot) edge.r_xbot = tileArea.r_xbot;
 	    if (edge.r_xtop > tileArea.r_xtop) edge.r_xtop = tileArea.r_xtop;
 	    GEOTRANSRECT(trans, &edge, &rootArea);
-	    plotVersLine(&rootArea, 0);
+	    plotVersLine(&rootArea, 0, raster);
 	}
     }
 
     /* Finally, the right border. */
 
+searchright:
+#ifdef NONMANHATTAN
+    if (IsSplit(tile) && !(SplitSide(tile)))
+	return 0;		/* Nothing on right side of tile */
+#endif
+
     if (tileArea.r_xtop < TiPlaneRect.r_xtop)
     {
 	edge.r_xbot = edge.r_xtop = tileArea.r_xtop;
 	for (neighbor = TR(tile); TOP(neighbor) > tileArea.r_ybot;
 		neighbor = LB(neighbor))
 	{
-	    if (TTMaskHasType(&curMask, TiGetType(neighbor))) continue;
+	    ntype = TiGetLeftType(neighbor);
+	    if (TTMaskHasType(&curMask, ntype)) continue;
 	    edge.r_ybot = BOTTOM(neighbor);
 	    edge.r_ytop = TOP(neighbor);
 	    if (edge.r_ybot < tileArea.r_ybot) edge.r_ybot = tileArea.r_ybot;
-	    if (edge.r_ytop < tileArea.r_ytop) edge.r_ytop = tileArea.r_ytop;
+	    if (edge.r_ytop > tileArea.r_ytop) edge.r_ytop = tileArea.r_ytop;
 	    GEOTRANSRECT(trans, &edge, &rootArea);
-	    plotVersLine(&rootArea, 0);
+	    plotVersLine(&rootArea, 0, raster);
 	}
     }
     return 0;
@@ -806,11 +852,13 @@ plotVersTile(tile, cxp)
  */
 
 int
-plotVersLabel(scx, label)
+plotVersLabel(scx, label, tpath, raster)
     SearchContext *scx;		/* Describes state of search when label
 				 * was found.
 				 */
     Label *label;		/* Label that was found. */
+    TerminalPath *tpath;	/* Ignored. */
+    Raster *raster;		/* Raster to write to */
 {
     Rect rootArea, swathArea, labelSize;
     Point point;
@@ -823,7 +871,7 @@ plotVersLabel(scx, label)
 
     GeoTransRect(&scx->scx_trans, &label->lab_rect, &rootArea);
     plotTransToSwath(&rootArea, &swathArea);
-    pos = GeoTransPos(&scx->scx_trans, label->lab_pos);
+    pos = GeoTransPos(&scx->scx_trans, label->lab_just);
     PlotTextSize(labelFont, label->lab_text, &labelSize);
 
     switch (pos)
@@ -902,7 +950,7 @@ plotVersLabel(scx, label)
     {
 	/* Line or rectangle.  Draw outline. */
 
-	plotVersRect(&rootArea, 1);
+	plotVersRect(&rootArea, 1, raster);
     }
 
     /* Output the text for the label.  Before outputting the label,
@@ -939,10 +987,11 @@ plotVersLabel(scx, label)
  */
 
 int
-plotVersCell(scx)
+plotVersCell(scx, raster)
     SearchContext *scx;		/* Describes cell whose bbox is to
 				 * be plotted.
 				 */	
+    Raster *raster;		/* Raster to write to */
 {
     char idName[100];
     Rect rootArea, swathArea, textSize;
@@ -955,28 +1004,33 @@ plotVersCell(scx)
 
     def = scx->scx_use->cu_def;
     GeoTransRect(&scx->scx_trans, &def->cd_bbox, &rootArea);
-    plotVersRect(&rootArea, 2);
+    plotVersRect(&rootArea, 2, raster);
 
     if (!PlotShowCellNames)
 	return (0);
 
     /* Output the cell's name and id text. */
-    plotTransToSwath(&rootArea, &swathArea);
-    PlotTextSize(cellNameFont, def->cd_name, &textSize);
-    point.p_x = (swathArea.r_xtop + swathArea.r_xbot)/2;
-    point.p_x -= (textSize.r_xtop + textSize.r_xbot)/2;
-    point.p_y = (2*swathArea.r_ytop + swathArea.r_ybot)/3;
-    point.p_y -= (textSize.r_ytop + textSize.r_ybot)/2;
-    PlotRasterText(raster, &swathClip, cellNameFont, def->cd_name, &point);
-
-    (void) DBPrintUseId(scx, idName, 100);
-    PlotTextSize(cellIdFont, idName, &textSize);
-    point.p_x = (swathArea.r_xtop + swathArea.r_xbot)/2;
-    point.p_x -= (textSize.r_xtop + textSize.r_xbot)/2;
-    point.p_y = (swathArea.r_ytop + 2*swathArea.r_ybot)/3;
-    point.p_y -= (textSize.r_ytop + textSize.r_ybot)/2;
-    PlotRasterText(raster, &swathClip, cellIdFont, idName, &point);
+    if (cellNameFont != NULL)
+    {
+	plotTransToSwath(&rootArea, &swathArea);
+	PlotTextSize(cellNameFont, def->cd_name, &textSize);
+	point.p_x = (swathArea.r_xtop + swathArea.r_xbot)/2;
+	point.p_x -= (textSize.r_xtop + textSize.r_xbot)/2;
+	point.p_y = (2*swathArea.r_ytop + swathArea.r_ybot)/3;
+	point.p_y -= (textSize.r_ytop + textSize.r_ybot)/2;
+	PlotRasterText(raster, &swathClip, cellNameFont, def->cd_name, &point);
+    }
 
+    if (cellIdFont != NULL)
+    {
+	DBPrintUseId(scx, idName, 100, TRUE);
+	PlotTextSize(cellIdFont, idName, &textSize);
+	point.p_x = (swathArea.r_xtop + swathArea.r_xbot)/2;
+	point.p_x -= (textSize.r_xtop + textSize.r_xbot)/2;
+	point.p_y = (swathArea.r_ytop + 2*swathArea.r_ybot)/3;
+	point.p_y -= (textSize.r_ytop + textSize.r_ybot)/2;
+	PlotRasterText(raster, &swathClip, cellIdFont, idName, &point);
+    }
     return 0;
 }
 
@@ -987,9 +1041,13 @@ plotVersCell(scx)
  *
  * 	This procedure generates a raster file suitable for driving
  *	printers like the Versatec black-and-white family, and runs
- *	a spooling program to print the file.  If PlotVersColor is true,
- *      it will generate a versatec color plot file in straight color raster
- *      format.
+ *	a spooling program to print the file.
+ *
+ *	If PlotVersPlotType is VERSATEC_COLOR, it will generate a
+ *	versatec color plot file in straight color raster format.
+ *
+ *	If PlotVersPlotType is HPGL2 or HPRTL, it will generate
+ *	an HPRTL file for the supported HP plotters. 
  *
  * Results:
  *	None.
@@ -1001,7 +1059,7 @@ plotVersCell(scx)
  */
 
 void
-PlotVersatec(scx, layers, xMask, width)
+PlotVersatec(scx, layers, xMask, user_scale)
     SearchContext *scx;			/* The use and area and transformation
 					 * in this describe what to plot.
 					 */
@@ -1022,14 +1080,12 @@ PlotVersatec(scx, layers, xMask, width)
 					 * visibility.  Zero means treat
 					 * everything as expanded.
 					 */
-    float width;			/* How many inches across the plot
-					 * should be.  If greater than the
-					 * printer width or <= 0, then use
-					 * the printer width.
-					 */
+    int user_scale;			/* Scalefactor of output */
 {
     static char *yesNo[] = {"no", "yes", NULL};
     int dotsAcross, dotsDown, swathsDown, scaleDown;
+    int mag_width;			/* lambda */
+    float width;			/* inches */	
     char fileName[200], command[300], answer[32];
     float length, mBytes;
     Transform tinv;
@@ -1037,13 +1093,38 @@ PlotVersatec(scx, layers, xMask, width)
     FILE *file;
     VersatecColor color;
     bool haveColorMessage;
+    int usedScale, maxScale;
+    float oscale;
+    Raster *raster = NULL;
+
+    /* CMYK color separated raster buffers.	*/
+    Raster *kRaster, *cRaster, *mRaster, *yRaster;
 
     haveColorMessage = FALSE;
     GeoTransRect(&scx->scx_trans, &scx->scx_area, &rootClip);
     GEO_EXPAND(&rootClip, 1, &rootClip);
-    dotsAcross = width * PlotVersDotsPerInch;
+
+    /* Get conversion factor (internal units to inches) */
+    oscale = CIFGetOutputScale(1000);	/* convert to microns */
+    oscale *= 3.937e-5;			/* convert to inches */
+   
+    /* Compute plot width from scalefactor */
+    mag_width = rootClip.r_xtop - rootClip.r_xbot;
+    maxScale = ((float)PlotVersWidth / (float)PlotVersDotsPerInch)
+		/ (oscale * (float)mag_width);
+    width = (float)user_scale * oscale * (float)mag_width;
+
+    dotsAcross = (int)(width * (float)PlotVersDotsPerInch);
     if (dotsAcross <= 0 || dotsAcross > PlotVersWidth)
+    {
 	dotsAcross = PlotVersWidth;
+	usedScale = maxScale;
+    }
+    else
+	usedScale = user_scale;
+
+    /* Recompute width based on the actual scale used */
+    width = (float)usedScale * oscale * (float)mag_width;
 
     /*
      * Compute the number of pixels per magic unit.
@@ -1055,7 +1136,7 @@ PlotVersatec(scx, layers, xMask, width)
     for (scale = 0, scaleShift = 4; ; scaleShift++)
     {
 	scaleDown = 1 << scaleShift;
-	scale = (scaleDown * dotsAcross) / (rootClip.r_xtop - rootClip.r_xbot);
+	scale = (scaleDown * dotsAcross) / mag_width;
 	if (scaleShift >= 8 * sizeof (int))
 	{
 	    TxError("The area selected is too many lambda wide to plot.\n");
@@ -1064,7 +1145,7 @@ PlotVersatec(scx, layers, xMask, width)
 	    TxError("a wider plot.\n");
 	    return;
 	}
-	if (scale >= 4)
+	if (scale >= 8)
 	    break;
     }
 
@@ -1078,9 +1159,13 @@ PlotVersatec(scx, layers, xMask, width)
     mBytes = ((PlotVersWidth/8)*dotsDown)/1000000.0;
     length = dotsDown;
     length /= PlotVersDotsPerInch;
-    TxPrintf("Plot will be %.1f inches long, and will take", length);
-    TxPrintf(" %.2f Megabytes in \"%s\".\n", 
-	     PlotVersColor ? 4.0*mBytes : mBytes, PlotTempDirectory);
+    TxPrintf("Plot will be %.1f inches wide by %.1f inches long.\n", width, length);
+    TxPrintf("It will take %.2f Megabytes in \"%s\".\n", 
+		(PlotVersPlotType == HPRTL || PlotVersPlotType == HPGL2)
+		? 4.0 * mBytes : mBytes, PlotTempDirectory);
+    TxPrintf("Lambda: %.3f (um)	  Requested scale: %dX    Actual scale: %dX   "
+		"[Full scale: %dX].\n", CIFGetOutputScale(1000),
+		user_scale, usedScale, maxScale);
     do
     {
 	TxPrintf("Do you still want the plot? [yes] ");
@@ -1097,16 +1182,29 @@ PlotVersatec(scx, layers, xMask, width)
      * recycle the raster for a new one.
      */
     
-    if (raster == NULL)
-	raster = PlotNewRaster(PlotVersSwathHeight, PlotVersWidth);
-    else
+    if ((raster != NULL) && ((raster->ras_width != PlotVersWidth)
+		|| (raster->ras_height != PlotVersSwathHeight)))
     {
-	if ((raster->ras_width != PlotVersWidth)
-		|| (raster->ras_height != PlotVersSwathHeight))
+	if (PlotVersPlotType == HPGL2 || PlotVersPlotType == HPRTL)
 	{
-	    PlotFreeRaster(raster);
-	    raster = PlotNewRaster(PlotVersSwathHeight, PlotVersWidth);
+	    PlotFreeRaster(cRaster);
+	    PlotFreeRaster(mRaster);
+	    PlotFreeRaster(yRaster);
 	}
+	PlotFreeRaster(kRaster);
+	raster = NULL;
+    }
+
+    if (raster == NULL)
+    {
+	if (PlotVersPlotType == HPGL2 || PlotVersPlotType == HPRTL)
+	{
+	    cRaster = PlotNewRaster(PlotVersSwathHeight, PlotVersWidth);
+	    mRaster = PlotNewRaster(PlotVersSwathHeight, PlotVersWidth);
+	    yRaster = PlotNewRaster(PlotVersSwathHeight, PlotVersWidth);
+	}
+	kRaster = PlotNewRaster(PlotVersSwathHeight, PlotVersWidth);
+	raster = kRaster;
     }
 
     /* Load font information for the plot, if it isn't already
@@ -1116,8 +1214,6 @@ PlotVersatec(scx, layers, xMask, width)
     labelFont = PlotLoadFont(PlotVersLabelFont);
     cellNameFont = PlotLoadFont(PlotVersNameFont);
     cellIdFont = PlotLoadFont(PlotVersIdFont);
-    if ((labelFont == NULL) || (cellNameFont == NULL) || (cellIdFont == NULL))
-	return;
 
     /* Compute the name of the file to use for output, and open it. */
 
@@ -1167,17 +1263,24 @@ PlotVersatec(scx, layers, xMask, width)
 	rasFileByteCount = 0;
 
 	/* issue preamble for this color's raster. if we are doing colors */
-	if (PlotVersColor) 
+	switch (PlotVersPlotType)
 	{
-	    if (PlotDumpColorPreamble(color, file, dotsDown, PlotVersWidth) != 0)
-	      goto error;
-	    if (SigInterruptPending)
-	      goto error;
-	}
-	if (PlotVersColor)
-	{
-	    TxPrintf("\nDumping %s Raster:", plotVersatecColorNames[color]);
-	    TxFlush();
+	    case HPRTL:
+		PlotHPRTLHeader(PlotVersWidth, dotsDown, PlotVersDotsPerInch, file);
+		break;
+	    case HPGL2:
+		PlotHPGL2Header(PlotVersWidth, dotsDown, PlotVersDotsPerInch,
+			usedScale, file);
+		break; 
+	    case VERSATEC_COLOR:
+	        if (PlotDumpColorPreamble(color, file, dotsDown, PlotVersWidth) != 0)
+		    goto error;
+		if (SigInterruptPending)
+		    goto error;
+
+		TxPrintf("\nDumping %s Raster:", plotVersatecColorNames[color]);
+		TxFlush();
+		break;
 	}
 	for (swathsDownThisColor -= 1; 
 	     swathsDownThisColor >= 0; 
@@ -1188,7 +1291,13 @@ PlotVersatec(scx, layers, xMask, width)
 	    int labelHeight;
 
 	    swathY = swathsDownThisColor * PlotVersSwathHeight;
-	    PlotClearRaster(raster, (Rect *) NULL);
+	    if (PlotVersPlotType == HPGL2 || PlotVersPlotType == HPRTL)
+	    {
+		PlotClearRaster(cRaster, (Rect *) NULL);
+		PlotClearRaster(mRaster, (Rect *) NULL);
+		PlotClearRaster(yRaster, (Rect *) NULL);
+	    }
+	    PlotClearRaster(kRaster, (Rect *) NULL);
 
 	    /* Compute the area of the swath that overlaps the portion of
 	     * the layout we're plotting.
@@ -1223,84 +1332,142 @@ PlotVersatec(scx, layers, xMask, width)
 
 	    labelArea.r_xbot = root.r_xbot;
 	    labelArea.r_xtop = root.r_xtop;
-	    labelHeight =
-	      (labelFont->fo_bbox.r_ytop - labelFont->fo_bbox.r_ybot) + 2;
-	    labelArea.r_ybot =
-	      (scaleDown*(swathY-crossSize-labelHeight))/scale
-		+ plotLL.p_y;
-	    labelArea.r_ytop =
-	      (scaleDown*(swathY+swathClip.r_ytop+crossSize+labelHeight))/scale
-		+ plotLL.p_y;
-	    GEO_EXPAND(&labelArea, 1, &labelArea);
+
+	    if (labelFont != NULL)
+	    {
+		labelHeight = (labelFont->fo_bbox.r_ytop
+			- labelFont->fo_bbox.r_ybot) + 2;
+		labelArea.r_ybot = (scaleDown * (swathY - crossSize
+			- labelHeight)) / scale + plotLL.p_y;
+		labelArea.r_ytop = (scaleDown * (swathY + swathClip.r_ytop
+			+ crossSize + labelHeight)) / scale + plotLL.p_y;
+		GEO_EXPAND(&labelArea, 1, &labelArea);
+	    }
 
 	    /* For each Versatec style, output stippled areas for all
 	     * the tiles requested by the style. 
 	     */
 
-	    curStyle = PlotVersColor?plotColorVersStyles:plotVersStyles;
-	    if (PlotVersColor && (plotColorVersStyles == NULL))
+	    switch (PlotVersPlotType)
 	    {
-		if (!haveColorMessage) 
-		{
-		    TxError("\nWarning:  No color versatec styles are defined in the technology file!\nPlot will be monochrome.\n");
-		}
-		haveColorMessage = TRUE;
-		curStyle = plotVersStyles;
+	        case VERSATEC_BW:
+		    curStyle = plotVersStyles;
+		    break;
+	        case HPGL2: case HPRTL:
+		    curStyle = plotColorVersStyles;	
+		    if (curStyle == NULL)
+		    {
+		        TxError("Warning:  No color versatec styles are defined"
+				" in the technology file!\nPlotting aborted.\n");
+			return;
+		    }
+		    break;
+		default:
+		    curStyle = plotColorVersStyles;	
+		    if (!haveColorMessage)
+		    {
+			TxError("Warning:  No color versatec styles are defined"
+				    " in the technology file!\nPlot will be"
+				    " monochrome.\n");
+			haveColorMessage = TRUE; 
+			curStyle = plotVersStyles;
+		    }
 	    }
 	    if (curStyle == NULL)
 	    {
-		TxError("Warning:  No monochrome versatec styles are defined in the technology file!\nPlotting aborted.\n");
+		TxError("Warning:  No monochrome versatec styles are"
+			    " defined in the technology file!\nPlotting"
+			    " aborted.\n");
 		return;
 	    }
 
-	    for ( ;
-		 curStyle != NULL;
-		 curStyle = curStyle->vs_next)
+	    for ( ; curStyle != NULL; curStyle = curStyle->vs_next)
 	    {
 		/* if we are plotting in B&W, then visit all the tiles in this
 		 * swath, otherwise only visit them if they should be
 		 * plotted in the current style's color.
 		 */
+		if (PlotVersPlotType == HPGL2 || PlotVersPlotType == HPRTL)
+		{
+		    switch (curStyle->vs_color)
+		    {
+			case CYAN:
+			    raster = cRaster;
+			    break;
+			case MAGENTA:
+			    raster = mRaster;
+			    break;
+			case YELLOW:
+			    raster = yRaster;
+			    break;
+			default:
+			    raster = kRaster;
+			    break;
+		    }
+		}
 
-		if ((PlotVersColor == FALSE) || (curStyle->vs_color == color))
+		if ((PlotVersPlotType != VERSATEC_COLOR) || (curStyle->vs_color == color))
 		{
 		    TTMaskAndMask3(&curMask, layers, &curStyle->vs_layers);
 		    (void) DBTreeSrTiles(&scx2, &curMask, xMask, plotVersTile,
-				     (ClientData) NULL);
+				     (ClientData) raster);
 		}
 	    }
 
+	    raster = kRaster;
+
 	    /* Output labels, if they are wanted. */
 
-	    if (TTMaskHasType(layers, L_LABEL) && color == BLACK)
+	    if (TTMaskHasType(layers, L_LABEL) && (color == BLACK) &&
+			(labelFont != NULL))
 	    {
 		curMask = *layers;
 		TTMaskSetType(&curMask, TT_SPACE);
 		GeoTransRect(&tinv, &labelArea, &scx2.scx_area);
 		(void) DBTreeSrLabels(&scx2, &curMask, xMask, (TerminalPath *) NULL,
-				      plotVersLabel, (ClientData) NULL);
+				      plotVersLabel, (ClientData) raster);
 	    }
 
 	    /* Output subcell bounding boxes, if they are wanted. */
 
 	    if (TTMaskHasType(layers, L_CELL) && color == BLACK)
 	    {
-		(void) DBTreeSrCells(&scx2, xMask, plotVersCell, (ClientData) NULL);
+		(void) DBTreeSrCells(&scx2, xMask, plotVersCell, (ClientData) raster);
 	    }
 
 	    TxPrintf("#");
 	    TxFlush();
 
-	    if (PlotDumpRaster(raster, file) != 0)
-	      goto error;
-	    if (SigInterruptPending)
-	      goto error;
+	    switch (PlotVersPlotType)
+	    {
+		case HPGL2: case HPRTL:
+		    PlotDumpHPRTL(file, kRaster, cRaster, mRaster, yRaster);
+		    break;
+		case VERSATEC_COLOR: case VERSATEC_BW:
+		    if (PlotDumpRaster(raster, file) != 0)
+			goto error;
+		    if (SigInterruptPending)
+			goto error;
+		    break;
+	    }
 	}
-	/* stop after first color (black) if this is not a color plot */
-	if (PlotVersColor == FALSE) break;
+	/* Only the VERSATEC_COLOR type runs colors separately */
+	if (PlotVersPlotType != VERSATEC_COLOR) break;
 	TxPrintf ("\nWrote %d bytes of data.\n", rasFileByteCount);
     }
 
+    /* Write trailers */
+
+    switch (PlotVersPlotType)
+    {
+	case HPGL2:
+	    PlotHPGL2Trailer(file);
+	    break;
+	case HPRTL:
+	    PlotHPRTLTrailer(file);
+	    break;
+    }
+
     /* Close the file and issue the command to plot it. */
 
     TxPrintf("\n");
