@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/resis/ResMain.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/resis/ResMain.c,v 1.2 2007/04/20 17:57:00 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -234,7 +234,7 @@ ResAddBreakpointFunc(tile, node)
 {
     tileJunk *junk;
 
-    if (tile->ti_client == (ClientData) MINFINITY)
+    if (tile->ti_client == (ClientData) CLIENTDEFAULT)
 	return 0;
 
     NEWPORT(node, tile);
@@ -647,7 +647,7 @@ ResExtractNet(startlist,goodies,cellname)
 	 ResFracture(plane,rect);
 	 (void) DBSrPaintClient((Tile *) NULL,plane,rect,
 	 		&DBAllButSpaceAndDRCBits,
-			(ClientData) MINFINITY, ResAddPlumbing,
+			(ClientData) CLIENTDEFAULT, ResAddPlumbing,
 			(ClientData) &ResTransList);
     }
 
@@ -716,7 +716,7 @@ ResCleanUpEverything()
     {
     	 (void) DBSrPaintClient((Tile *) NULL,ResUse->cu_def->cd_planes[pNum],
 	 		&(ResUse->cu_def->cd_bbox),&DBAllButSpaceAndDRCBits,
-			(ClientData) MINFINITY,ResRemovePlumbing,
+			(ClientData) CLIENTDEFAULT,ResRemovePlumbing,
 			(ClientData) NULL);
 			
     }
