@@ -1,6 +1,6 @@
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/resis/ResRex.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/resis/ResRex.c,v 1.2 2008/01/01 18:44:08 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -45,9 +45,9 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 
 
 /* time constants are produced by multipliying attofarads by milliohms, */
-/* giving milliattoseconds (or piconanoseconds, depending on how you    */
-/* look at it.)  This constant is used to convert them to nanoseconds.  */
-#define MA_TO_N		1e12
+/* giving zeptoseconds (yes, really.  Look it up).  This constant 	*/
+/* converts zeptoseconts to nanoseconds.				*/
+#define Z_TO_N		1e12
 
 /* ResSimNode is a node read in from a sim file */
 
@@ -1462,7 +1462,7 @@ ResWriteExtFile(celldef, node, tol, rctol, nidx, eidx)
 	    if ((ResOptionsFlags & (ResOpt_RunSilent|ResOpt_Tdi)) == ResOpt_Tdi)
 	    {
 		TxError("Adding  %s; Tnew = %.2fns,Told = %.2fns\n",
-		     	    node->name,gparams.rg_Tdi/MA_TO_N, RCtran/MA_TO_N);
+		     	    node->name,gparams.rg_Tdi/Z_TO_N, RCtran/Z_TO_N);
 	    }
         }
         for (ptr = node->firstTran; ptr != NULL; ptr=ptr->nextTran)
