@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/select/selCreate.c,v 1.2 2006/04/15 03:36:49 anonymous Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/select/selCreate.c,v 1.11 2008/02/02 01:18:51 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -116,6 +116,7 @@ SelectInit()
     SelectUse = DBCellNewUse(SelectDef, (char *) NULL);
     DBSetTrans(SelectUse, &GeoIdentityTransform);
     SelectUse->cu_expandMask = CU_DESCEND_SPECIAL;	/* This is always expanded. */
+    SelectUse->cu_flags = 0;	/* never locked down */
 
     Select2Def = DBCellLookDef("__SELECT2__");
     if (Select2Def == (CellDef *) NULL)
@@ -128,6 +129,7 @@ SelectInit()
     Select2Use = DBCellNewUse(Select2Def, (char *) NULL);
     DBSetTrans(Select2Use, &GeoIdentityTransform);
     Select2Use->cu_expandMask = CU_DESCEND_SPECIAL;	/* This is always expanded. */
+    Select2Use->cu_flags = 0;	/* never locked down */
     UndoEnable();
 
     SelUndoInit();
@@ -183,9 +185,11 @@ SelectClear()
 	SelRememberForUndo(TRUE, (CellDef *) NULL, (Rect *) NULL);
 
 	/* Erase all the paint from the select cell. */
-
 	DBEraseMask(SelectDef, &TiPlaneRect, &DBAllButSpaceBits);
 
+	/* For the line below to work, paint tables need to be set up correctly! */
+	/* DBErase(SelectDef, &TiPlaneRect, TT_SPACE); */
+
 	/* Erase all of the labels from the select cell. */
 
 	(void) DBEraseLabel(SelectDef, &TiPlaneRect, &DBAllTypeBits);
@@ -538,6 +542,7 @@ SelectChunk(scx, type, xMask, pArea, less)
 #define INITIALSIZE 10
     SearchContext newscx;
     TileTypeBitMask wrongTypes, typeMask;
+    TileType ttype;
     Rect bestChunk;
     int bestMin, bestMax, width, height;
 
@@ -565,8 +570,11 @@ SelectChunk(scx, type, xMask, pArea, less)
      */
 
     UndoDisable();
-    TTMaskZero(&typeMask);
-    TTMaskSetType(&typeMask, type);
+    TTMaskSetOnlyType(&typeMask, type);
+
+    /* Stacked types containing "type" need to be added to the mask */
+    if (DBIsContact(type)) DBMaskAddStacking(&typeMask);
+
     TTMaskCom2(&wrongTypes, &typeMask);
 
     newscx = *scx;
@@ -656,6 +664,11 @@ chunkdone:
     else
       {
 	newscx.scx_area = bestChunk;
+
+	/* Remove any stacked contact types for SelectArea */
+	if (DBIsContact(type))
+	    TTMaskSetOnlyType(&typeMask, type);
+
 	SelectArea(&newscx, &typeMask, xMask);
       }
 
@@ -955,6 +968,7 @@ SelectCell(use, rootDef, trans, replace)
     DBSetArray(use, newUse);
     DBSetTrans(newUse, trans);
     newUse->cu_expandMask = use->cu_expandMask;
+    newUse->cu_flags = use->cu_flags;
 
     /* If this cell is already selected, there's nothing more to do.
      * Since we didn't change the selection here, be sure NOT to remember
@@ -1060,6 +1074,7 @@ SelectAndCopy2(newSourceDef)
     SearchContext scx;
     Rect editArea, labelArea;
     int plane;
+    void (*savedPaintPlane)();
     extern int selACPaintFunc();	/* Forward reference. */
     extern int selACCellFunc();
 
@@ -1070,8 +1085,10 @@ SelectAndCopy2(newSourceDef)
     scx.scx_use = Select2Use;
     scx.scx_area = Select2Use->cu_bbox;
     scx.scx_trans = RootToEditTransform;
+    savedPaintPlane = DBNewPaintPlane(DBPaintPlaneActive);
     (void) DBCellCopyAllPaint(&scx, &DBAllButSpaceAndDRCBits, CU_DESCEND_SPECIAL,
 		EditCellUse);
+    DBNewPaintPlane(savedPaintPlane);
     (void) DBCellCopyAllLabels(&scx, &DBAllTypeBits, CU_DESCEND_SPECIAL, EditCellUse,
 		(Rect *) NULL);
     (void) DBCellCopyAllCells(&scx, CU_DESCEND_SPECIAL, EditCellUse, (Rect *) NULL);
@@ -1138,7 +1155,7 @@ SelectAndCopy2(newSourceDef)
  */
 typedef struct {
     Rect *editClip;
-    TileType dinfo;
+    TileType ttype;
 } acparg;
 
 /* Utility function: for each tile, copy information over its area from
@@ -1161,14 +1178,14 @@ selACPaintFunc(tile, plane)
     GeoTransRect(&RootToEditTransform, &area, &editArea);
 
 #ifndef NONMANHATTAN
-    selACarg.dinfo = 0;
+    selACarg.ttype = TiGetType(tile);
 #else
-    selACarg.dinfo = TiGetTypeExact(tile);
+    selACarg.ttype = TiGetTypeExact(tile);
 
     if (IsSplit(tile))
     {
 	DBSrPaintNMArea((Tile *) NULL, EditCellUse->cu_def->cd_planes[plane],
-		selACarg.dinfo, &editArea, &DBAllButSpaceAndDRCBits,
+		selACarg.ttype, &editArea, &DBAllButSpaceAndDRCBits,
 		selACPaintFunc2, (ClientData) &selACarg);
     }
     else
@@ -1198,9 +1215,11 @@ selACPaintFunc2(tile, selACarg)
     Rect *editClip = selACarg->editClip;
     Rect area, selArea;
     TileType type = TiGetTypeExact(tile);
+    TileTypeBitMask tmask;
+    TileType ttype, rtype;
 
 #ifdef NONMANHATTAN
-    TileType dinfo = selACarg->dinfo & (TT_DIAGONAL | TT_DIRECTION | TT_SIDE);
+    TileType dinfo = selACarg->ttype & (TT_DIAGONAL | TT_DIRECTION | TT_SIDE);
 #endif
 
     TiToRect(tile, &area);
@@ -1216,7 +1235,9 @@ selACPaintFunc2(tile, selACarg)
 	Point points[5];
 	Rect rrect, orect;
 	int np, i, j;
-	TileType rtype;
+
+	ttype = (selACarg->ttype & TT_SIDE) ? ((ttype & TT_RIGHTMASK) >> 14) :
+		ttype & TT_LEFTMASK;
 
 	if (type & TT_DIAGONAL)
 	    rtype = (type & TT_SIDE) ? SplitRightType(tile) :
@@ -1224,6 +1245,10 @@ selACPaintFunc2(tile, selACarg)
 	else
 	    rtype = type;
 
+	if (rtype >= DBNumUserLayers && TTMaskHasType(DBResidueMask(rtype), ttype))
+	    rtype = ttype;
+	TTMaskSetOnlyType(&tmask, rtype);
+
 	type = (dinfo & TT_SIDE) ? (rtype << 14) : rtype;
 	type |= dinfo;
 
@@ -1283,7 +1308,7 @@ selACPaintFunc2(tile, selACarg)
 		else
 		    goto topbottom;
 
-		DBPaint(SelectDef, &rrect, rtype);
+		DBPaintValid(SelectDef, &rrect, &tmask, 0);
 topbottom:
 		/* Rectangle to top or bottom */
 		rrect.r_xbot = selArea.r_xbot;
@@ -1301,16 +1326,36 @@ topbottom:
 		else
 		    goto splitdone;
 
-		DBPaint(SelectDef, &rrect, rtype);
+		DBPaintValid(SelectDef, &rrect, &tmask, 0);
 	    }
 	}
     }
     else
+    {
+	ttype = selACarg->ttype;
+
+	if (type & TT_DIAGONAL)
+	    rtype = (type & TT_SIDE) ? SplitRightType(tile) : SplitLeftType(tile);
+	else
+	    rtype = type;
+
+	if (rtype >= DBNumUserLayers && TTMaskHasType(DBResidueMask(rtype), ttype))
+	    rtype = ttype;
+	TTMaskSetOnlyType(&tmask, rtype);
+    }
+#else	/* !NONMANHATTAN */
+
+    ttype = selACarg->ttype;
+    if (type >= DBNumUserLayers && TTMaskHasType(DBResidueMask(type), ttype))
+	    type = ttype;
+    TTMaskSetOnlyType(&tmask, type);
+
 #endif
+
     GeoClip(&selArea, editClip);	/* editClip already in root coords */
 
 splitdone:
-    DBPaint(SelectDef, &selArea, type);
+    DBPaintValid(SelectDef, &selArea, &tmask, type);
     return 0;
 }
 
@@ -1338,6 +1383,7 @@ selACCellFunc(selUse, realUse)
 	(void) DBLinkCell(newUse, SelectDef);
     }
     newUse->cu_expandMask = realUse->cu_expandMask;
+    newUse->cu_flags = realUse->cu_flags;
     DBSetArray(selUse, newUse);
     DBSetTrans(newUse, &selUse->cu_transform);
     DBPlaceCell(newUse, SelectDef);
