@@ -20,7 +20,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/select/selEnum.c,v 1.2 2006/09/24 23:35:58 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/select/selEnum.c,v 1.12 2008/02/07 06:22:44 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -38,6 +38,15 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 #include "utils/malloc.h"
 #include "textio/textio.h"
 
+/* Linked rectangle structure including tiletype information */
+
+typedef struct extRectList
+{
+    TileType r_type;
+    Rect r_r;
+    struct extRectList *r_next;
+} ExtRectList;
+
 /* Structure passed from top-level enumeration procedures to lower-level
  * ones:
  */
@@ -49,8 +58,9 @@ struct searg
     bool sea_editOnly;		/* Only consider stuff that's in edit cell. */
     bool *sea_nonEdit;		/* Word to set if non-edit stuff is found. */
     int sea_plane;		/* Index of plane currently being searched. */
+    Rect sea_rect;		/* Area of tile being searched */
     TileType sea_type;		/* Type of current piece of selected paint. */
-    LinkedRect *sea_rectList;	/* List of rectangles found in edit cell. */
+    ExtRectList *sea_rectList;	/* List of rectangles found in edit cell. */
     CellUse *sea_use;		/* Use that we're looking for an identical
 				 * copy of in the layout.
 				 */
@@ -92,6 +102,13 @@ struct searg
  *	0.  If it returns a non-zero return value, then the search
  *	will be aborted.
  *
+ *	Note that the "type" value passed to func() always contains a
+ *	tiletype in the TT_LEFTMASK field regardless of the upper bits
+ *	describing the diagonal of a non-Manhattan tile.  The function
+ *	is responsible for shifting the type field up into TT_RIGHTMASK
+ *	if the TT_SIDE bit is set.  This method is obscure but usually
+ *	simplifies processing of non-Manhattan tiles in func().
+ *
  * Results:
  *	Returns 0 if the search finished normally.  Returns 1 if the
  *	search was aborted.
@@ -128,7 +145,8 @@ SelEnumPaint(layers, editOnly, foundNonEdit, func, clientData)
     if (foundNonEdit != NULL) *foundNonEdit = FALSE;
 
     /* First, find all the paint in the selection that has the right
-     * layers.
+     * layers.  Use the same procedure as "dbCellUniqueTileSrFunc()"
+     * so that contacts are not double-counted.
      */
     
     for (plane = PL_SELECTBASE; plane < DBNumPlanes; plane++)
@@ -155,67 +173,32 @@ selEnumPFunc1(tile, arg)
     Tile *tile;			/* Tile of matching type. */
     struct searg *arg;		/* Describes the current search. */
 {
-    Rect rect, editRect, rootRect;
-    extern int selEnumPFunc2();	/* Forward declaration. */
+    Rect editRect, rootRect;
+    TileType loctype;
+    TileTypeBitMask uMask;
+    extern int selEnumPFunc2();
 
-    TiToRect(tile, &rect);
+    TiToRect(tile, &arg->sea_rect);
 
 #ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
+	arg->sea_type = TiGetTypeExact(tile) & (TT_DIAGONAL | TT_SIDE | TT_DIRECTION);
 	if (SplitSide(tile))
-	    arg->sea_type = SplitRightType(tile);
+	    loctype = SplitRightType(tile);
 	else
-	    arg->sea_type = SplitLeftType(tile);
+	    loctype = SplitLeftType(tile);
     }
     else 
 #endif
-    arg->sea_type = TiGetType(tile);
-
-    /* Only process contacts once:  only process the home plane */
-    /* image of any contact type.  Stacked contacts are treated	*/
-    /* as whichever residue (contact) is in its home plane.	*/
-
-    if (DBIsContact(arg->sea_type))
-    {
-	if (arg->sea_type < DBNumUserLayers)
-	{
-	    if (arg->sea_plane != DBPlane(arg->sea_type)) return 0;
-#ifdef NONMANHATTAN
-	    arg->sea_type = TiGetTypeExact(tile);
-#endif
-	}
-	else	/* stacked contact type */
-	{
-	    TileTypeBitMask *rmask = DBResidueMask(arg->sea_type);
-	    TileType rtype;
-	    for (rtype = TT_TECHDEPBASE; rtype < DBNumUserLayers; rtype++)
-		if (TTMaskHasType(rmask, rtype) &&
-				arg->sea_plane == DBPlane(rtype))
-		    break;
-	    if (rtype == DBNumUserLayers) return 0;
-
-	    /* Continue, with the tile type recast as rtype */
-
-#ifdef NONMANHATTAN
-	    if (IsSplit(tile))
-	    {
-		if (SplitSide(tile))
-		    arg->sea_type = (rtype << 14) | SplitLeftType(tile);
-		else
-		    arg->sea_type = rtype | (SplitRightType(tile) << 14);
-		arg->sea_type |= TT_DIAGONAL | TT_DIRECTION | TT_SIDE;
-	    }
-	    else
-#endif
-	    arg->sea_type = rtype;
-	}
-    }
+    loctype = TiGetType(tile);
 
 #ifdef NONMANHATTAN
+    if (IsSplit(tile))
+	arg->sea_type |= loctype;
     else
-	arg->sea_type = TiGetTypeExact(tile);
 #endif
+	arg->sea_type = loctype;
 
     /* If the paint doesn't have to be in the edit cell, life's pretty
      * simple:  just call the client and quit.
@@ -223,30 +206,47 @@ selEnumPFunc1(tile, arg)
     
     if (!arg->sea_editOnly)
     {
-	if ((*arg->sea_func)(&rect, arg->sea_type, arg->sea_cdarg) != 0)
+	if ((*arg->sea_func)(&arg->sea_rect, arg->sea_type, arg->sea_cdarg) != 0)
 	    return 1;
 	return 0;
     }
-    
-    /* Find the stuff that's in the edit cell. */
 
-    GeoTransRect(&RootToEditTransform, &rect, &editRect);
+    /* From here to the end of the routine gets tricky.  To apply the	*/
+    /* function only to what's in the edit cell, we use the selection	*/
+    /* paint to compare against the edit cell, and apply the function	*/
+    /* to matching paint, ignoring paint from non-edit cells.  Because	*/
+    /* the function may alter the edit cell, we must collect a list of	*/
+    /* areas to paint, and apply them one by one without reference to	*/
+    /* any actual tiles.						*/
+
+    GeoTransRect(&RootToEditTransform, &arg->sea_rect, &editRect);
     arg->sea_rectList = NULL;
-    (void) DBSrPaintArea((Tile *) NULL,
-	    EditCellUse->cu_def->cd_planes[arg->sea_plane],
-	    &editRect, &DBAllTypeBits, selEnumPFunc2,
-	    (ClientData) arg);
-    
-    /* Call the client for each rectangle found. */
+
+#ifdef NONMANHATTAN
+    if (IsSplit(tile))
+	DBSrPaintNMArea((Tile *)NULL,
+		EditCellUse->cu_def->cd_planes[arg->sea_plane],
+		arg->sea_type, &editRect, &DBAllTypeBits, selEnumPFunc2,
+		(ClientData)arg);
+    else
+#endif
+	DBSrPaintArea((Tile *)NULL,
+		EditCellUse->cu_def->cd_planes[arg->sea_plane],
+		&editRect, &DBAllTypeBits, selEnumPFunc2,
+		(ClientData)arg);
+
+    /* Each rectangle found represents paint that is both in	*/
+    /* the selection and also in the edit cell.  Call the	*/
+    /* client for each such area.				*/
 
     while (arg->sea_rectList != NULL)
     {
 	GeoTransRect(&EditToRootTransform, &arg->sea_rectList->r_r, &rootRect);
-	GeoClip(&rootRect, &rect);
-	
-	if ((*arg->sea_func)(&rootRect, arg->sea_type, arg->sea_cdarg) != 0)
+	GeoClip(&rootRect, &arg->sea_rect);
+
+	if ((*arg->sea_func)(&rootRect, arg->sea_rectList->r_type, arg->sea_cdarg) != 0)
 	    return 1;
-	freeMagic((char *) arg->sea_rectList);
+	freeMagic((char *)arg->sea_rectList);
 	arg->sea_rectList = arg->sea_rectList->r_next;
     }
     return 0;
@@ -259,38 +259,27 @@ selEnumPFunc1(tile, arg)
 
 int
 selEnumPFunc2(tile, arg)
-    Tile *tile;			/* Tile found in the edit cell. */
-    struct searg *arg;		/* Describes our search. */
+    Tile *tile;			/* Tile found in the edit cell */ 
+    struct searg *arg;		/* Describes our search */
 {
-    LinkedRect *lr;
+    ExtRectList *lr;
     int ttype;
-    TileType loctype;
+    TileType seltype;
 
-#ifdef NONMANHATTAN
-    if (arg->sea_type & TT_DIAGONAL)
-	loctype = (arg->sea_type & TT_SIDE) ?
-		(arg->sea_type & TT_RIGHTMASK) >> 14 :
-		arg->sea_type & TT_LEFTMASK;
-    else
-#endif
-    loctype = arg->sea_type;
-     
 #ifdef NONMANHATTAN
     if (IsSplit(tile))
-    {
-	ttype = SplitLeftType(tile);
-	if (loctype != ttype)
-	ttype = SplitRightType(tile);
-    }
+	ttype = SplitSide(tile) ? SplitRightType(tile) : SplitLeftType(tile);
     else
-	ttype = TiGetType(tile);
-        
+	ttype = TiGetTypeExact(tile);
+    seltype = arg->sea_type & TT_LEFTMASK;
+
 #else
-    ttype = TiGetTypeExact(tile);
+    ttype = TiGetType(tile);
+    seltype = arg->sea_type;
 #endif
 
     /* Handle stacked contact types */
-    if (ttype >= DBNumUserLayers)
+    if ((ttype != seltype) && (ttype >= DBNumUserLayers))
     {
 	TileTypeBitMask *rmask = DBResidueMask(ttype);
 	TileType rtype;
@@ -303,18 +292,166 @@ selEnumPFunc2(tile, arg)
 		}
     }
 
-    if (loctype != ttype)
+    /* Check for compatible types.  That is:  paint X (in use) over Y	*/
+    /* (in select) yields Y means that it is okay to add X to the list.	*/
+    /* Also okay if the X over Y yields a stacking type whose residues	*/
+    /* are X and Y.							*/
+
+    if ((ttype != seltype) && (((ttype == TT_SPACE) &&
+		TTMaskHasType(&DBHomePlaneTypes[arg->sea_plane], seltype))) ||
+		(DBPaintResultTbl[arg->sea_plane][ttype][seltype] != seltype))
     {
-	if (arg->sea_nonEdit != NULL) *(arg->sea_nonEdit) = TRUE;
+	TileType chktype = DBPaintResultTbl[arg->sea_plane][ttype][seltype];
+	if (chktype < DBNumUserLayers || !TTMaskHasType(DBResidueMask(chktype),
+		ttype) || !TTMaskHasType(DBResidueMask(chktype), seltype))
+	{
+	    if (arg->sea_nonEdit != NULL) *(arg->sea_nonEdit) = TRUE;
+	}
 	return 0;
     }
 
-    lr = (LinkedRect *) mallocMagic(sizeof(LinkedRect));
+    /* Only process contacts once:  only process the home plane */
+    /* image of any contact type.  Stacked contacts are treated	*/
+    /* as whichever residue (contact) is in its home plane.	*/
+
+    if (DBIsContact(ttype))
+    {
+	if (ttype < DBNumUserLayers)
+	{
+	    if (arg->sea_plane != DBPlane(ttype)) return 0;
+	}
+	else	/* stacked contact type */
+	{
+	    TileTypeBitMask *rmask = DBResidueMask(ttype);
+	    TileType rtype;
+	    for (rtype = TT_TECHDEPBASE; rtype < DBNumUserLayers; rtype++)
+		if (TTMaskHasType(rmask, rtype) &&
+				arg->sea_plane == DBPlane(rtype))
+		    break;
+	    if (rtype == DBNumUserLayers) return 0;
+
+	    /* Continue, with the tile type recast as rtype */
+
+	    ttype = rtype;
+	}
+    }
+
+    lr = (ExtRectList *)mallocMagic(sizeof(ExtRectList));
     TiToRect(tile, &lr->r_r);
+
+#ifdef NONMANHATTAN
+    if (arg->sea_type & TT_DIAGONAL)
+    {
+	Point points[5];
+	int np, i, j;
+	Rect area, orect;
+
+	lr->r_type = ttype | (arg->sea_type & (TT_DIAGONAL | TT_SIDE | TT_DIRECTION));
+	if (!IsSplit(tile))
+	{
+	    /* Things get messy here.  Clip against the select triangle	*/
+	    /* and decompose the result into rectangles and triangles.	*/ 
+
+	    GrClipTriangle(&lr->r_r, &arg->sea_rect, TRUE, lr->r_type, points, &np);
+
+	    if (np < 3)
+	    {
+		freeMagic((char *)lr);
+		return 0;	/* Clipped out of existence */
+	    }
+	    else
+	    {
+		for (i = 0; i < np; i++)
+		{
+		    j = (i + 1) % np;
+		    if (points[i].p_x != points[j].p_x && points[i].p_y !=
+				points[j].p_y)
+		    {
+			/* Break out the triangle */
+			lr->r_r.r_xbot = points[i].p_x;
+			lr->r_r.r_xtop = points[j].p_x;
+			lr->r_r.r_ybot = points[i].p_y;
+			lr->r_r.r_ytop = points[j].p_y;
+			GeoCanonicalRect(&lr->r_r, &area);
+			break;
+		    }
+		}
+		if (i == np)	/* Exactly one Manhattan rectangle */
+		{
+		    lr->r_r.r_xbot = points[0].p_x;
+		    lr->r_r.r_xtop = points[2].p_x;
+		    lr->r_r.r_ybot = points[0].p_y;
+		    lr->r_r.r_ytop = points[2].p_y;
+		    GeoCanonicalRect(&lr->r_r, &area);
+		    lr->r_type = ttype;
+		}
+		else if (np >= 4) /* Process extra rectangles */
+		{
+		    orect.r_xtop = orect.r_xbot = points[0].p_x;
+		    orect.r_ytop = orect.r_ybot = points[0].p_y;
+		    for (i = 0; i < np; i++)
+			GeoIncludePoint(&points[i], &orect);
+
+		    /* Rectangle to right or left */
+		    lr->r_r.r_ybot = orect.r_ybot;
+		    lr->r_r.r_ytop = orect.r_ytop;
+		    if (area.r_xbot > orect.r_xbot)
+		    {
+			lr->r_r.r_xbot = orect.r_xbot;
+			lr->r_r.r_xtop = area.r_xbot;
+		    }
+		    else if (area.r_xtop < orect.r_xtop)
+		    {
+			lr->r_r.r_xtop = orect.r_xtop;
+			lr->r_r.r_xbot = area.r_xtop;
+		    }
+		    else
+			goto topbottom;
+
+		    lr->r_type = ttype;
+		    lr->r_next = arg->sea_rectList;
+		    arg->sea_rectList = lr;
+		    lr = (ExtRectList *)mallocMagic(sizeof(ExtRectList));
+		    lr->r_r = arg->sea_rectList->r_r;
+topbottom:
+		    /* Rectangle to top or bottom */
+		    lr->r_r.r_xbot = area.r_xbot;
+		    lr->r_r.r_xtop = area.r_xtop;
+		    if (area.r_ybot > orect.r_ybot)
+		    {
+			lr->r_r.r_ybot = orect.r_ybot;
+			lr->r_r.r_ytop = area.r_ybot;
+		    }
+		    else if (area.r_ytop < orect.r_ytop)
+		    {
+			lr->r_r.r_ytop = orect.r_ytop;
+			lr->r_r.r_ybot = area.r_ytop;
+		    }
+		    else
+			goto splitdone;
+
+		    lr->r_type = ttype;
+		    lr->r_next = arg->sea_rectList;
+		    arg->sea_rectList = lr;
+		    lr = (ExtRectList *)mallocMagic(sizeof(ExtRectList));
+		    lr->r_r = area;
+		    lr->r_type = arg->sea_type;
+		}
+	    }
+	}
+    }
+    else
+	lr->r_type = ttype;
+#else
+    lr->r_type = ttype;
+#endif
+
+splitdone:
     lr->r_next = arg->sea_rectList;
     arg->sea_rectList = lr;
     return 0;
 }
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -467,6 +604,7 @@ selEnumCFunc1(scx, arg)
     
     scx2.scx_use = &dummy;
     dummy.cu_def = SelectRootDef;
+    dummy.cu_id = NULL;
     GeoTransRect(&scx->scx_use->cu_transform, &scx->scx_use->cu_def->cd_bbox,
 	    &scx2.scx_area);
     scx2.scx_area.r_xtop = scx2.scx_area.r_xbot + 1;
@@ -619,6 +757,7 @@ SelEnumLabels(layers, editOnly, foundNonEdit, func, clientData)
 	
 	scx.scx_use = &dummy;
 	dummy.cu_def = SelectRootDef;
+	dummy.cu_id = NULL;
 	GEO_EXPAND(&selLabel->lab_rect, 1, &scx.scx_area);
 	scx.scx_trans = GeoIdentityTransform;
 	arg.sea_label = selLabel;
@@ -672,8 +811,8 @@ selEnumLFunc(scx, label, tpath, arg)
     if (want->r_ybot != got.r_ybot) return 0;
     if (want->r_xtop != got.r_xtop) return 0;
     if (want->r_ytop != got.r_ytop) return 0;
-    if (arg->sea_label->lab_pos
-	!= GeoTransPos(&scx->scx_trans, label->lab_pos)) return 0;
+    if (arg->sea_label->lab_just
+	!= GeoTransPos(&scx->scx_trans, label->lab_just)) return 0;
     if (strcmp(label->lab_text, arg->sea_label->lab_text) != 0) return 0;
 
     arg->sea_foundLabel = label;
