@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/select/selOps.c,v 1.3 2006/09/24 23:35:59 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/select/selOps.c,v 1.11 2008/01/29 02:41:58 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -134,27 +134,28 @@ selDelPaintFunc(rect, type)
     TileType type;		/* Type of paint to delete. */
 {
     Rect editRect;
+    TileTypeBitMask tmask;
+    TileType dinfo;
 
 #ifdef NONMANHATTAN
-    TileType newtype;
 
-    /* Change diagonal direction according to the transform */
+    /* Change diagonal side & direction according to the transform */
 
     if (type & TT_DIAGONAL)
     {
-	TileType dinfo = DBTransformDiagonal(type, &RootToEditTransform);
-
-	if ((type & TT_SIDE) != (dinfo & TT_SIDE))
-	    newtype = dinfo | ((type & TT_LEFTMASK) << 14) |
-                        ((type & TT_RIGHTMASK) >> 14);
-	else
-	    newtype = dinfo | (type & (TT_LEFTMASK | TT_RIGHTMASK));
-	type = newtype;
+	dinfo = DBTransformDiagonal(type, &RootToEditTransform);
+	TTMaskSetOnlyType(&tmask, type & TT_LEFTMASK);   
     }
+    else
 #endif
+    {
+	dinfo = 0;
+	TTMaskSetOnlyType(&tmask, type);   
+    }
 
     GeoTransRect(&RootToEditTransform, rect, &editRect);
-    DBErase(EditCellUse->cu_def, &editRect, type);
+
+    DBEraseValid(EditCellUse->cu_def, &editRect, &tmask, dinfo);
     return 0;
 }
 
@@ -166,6 +167,8 @@ selDelCellFunc(selUse, use)
     CellUse *selUse;		/* Not used. */
     CellUse *use;		/* What to delete. */
 {
+    if (use->cu_flags & CU_LOCKED) return 0;
+
     DBUnLinkCell(use, use->cu_parent);
     DBDeleteCell(use);
     (void) DBCellDeleteUse(use);
@@ -182,7 +185,7 @@ selDelLabelFunc(label)
     Label *label;		/* Label to delete. */
 {
     DBEraseLabelsByContent(EditCellUse->cu_def, &label->lab_rect,
-	    label->lab_pos, -1, label->lab_text);
+	    label->lab_just, -1, label->lab_text);
     return 0;
 }
 
@@ -223,11 +226,11 @@ SelectCopy(transform)
     scx.scx_use = SelectUse;
     scx.scx_area = SelectUse->cu_bbox;
     GeoTransTrans(transform, &SelectUse->cu_transform, &scx.scx_trans);
-    (void) DBCellCopyAllPaint(&scx, &DBAllButSpaceAndDRCBits, CU_DESCEND_SPECIAL,
+    (void) DBCellCopyAllPaint(&scx, &DBAllButSpaceAndDRCBits, CU_DESCEND_NO_LOCK,
 		Select2Use);
-    (void) DBCellCopyAllLabels(&scx, &DBAllTypeBits, CU_DESCEND_SPECIAL, Select2Use,
+    (void) DBCellCopyAllLabels(&scx, &DBAllTypeBits, CU_DESCEND_NO_LOCK, Select2Use,
 		(Rect *) NULL);
-    (void) DBCellCopyAllCells(&scx, CU_DESCEND_SPECIAL, Select2Use, (Rect *) NULL);
+    (void) DBCellCopyAllCells(&scx, CU_DESCEND_NO_LOCK, Select2Use, (Rect *) NULL);
     DBReComputeBbox(Select2Def);
     UndoEnable();
 
@@ -285,27 +288,23 @@ selTransPaintFunc(rect, type, transform)
     Transform *transform;	/* How to change coords before painting. */
 {
     Rect newarea;
+    TileType loctype;
 
 #ifdef NONMANHATTAN
-    TileType newtype;
 
     /* Change diagonal direction according to the transform */
-
     if (type & TT_DIAGONAL)
     {
-	TileType dinfo = DBTransformDiagonal(type, transform);
-
-	if ((type & TT_SIDE) != (dinfo & TT_SIDE))
-	    newtype = dinfo | ((type & TT_LEFTMASK) << 14) |
-                        ((type & TT_RIGHTMASK) >> 14);
-	else
-	    newtype = dinfo | (type & (TT_LEFTMASK | TT_RIGHTMASK));
-	type = newtype;
+	loctype = DBTransformDiagonal(type, transform);
+	loctype |= (loctype & TT_SIDE) ? (type & TT_LEFTMASK) << 14 :
+		(type & TT_LEFTMASK);
     }
+    else
 #endif
+	loctype = type;
 
     GeoTransRect(transform, rect, &newarea);
-    DBPaint(Select2Def, &newarea, type);
+    DBPaint(Select2Def, &newarea, loctype);
     return 0;
 }
 
@@ -327,6 +326,8 @@ selTransCellFunc(selUse, realUse, realTrans, transform)
     CellUse  *newUse;
     Transform newTrans;
 
+    if (selUse->cu_flags & CU_LOCKED) return 0;
+
     newUse = DBCellNewUse(selUse->cu_def, (char *) realUse->cu_id);
     if (!DBLinkCell(newUse, Select2Def))
     {
@@ -338,6 +339,7 @@ selTransCellFunc(selUse, realUse, realTrans, transform)
     DBSetArray(selUse, newUse);
     DBSetTrans(newUse, &newTrans);
     newUse->cu_expandMask = selUse->cu_expandMask;
+    newUse->cu_flags = selUse->cu_flags;
     DBPlaceCell(newUse, Select2Def);
 
     return 0;
@@ -360,13 +362,13 @@ selTransLabelFunc(label, cellUse, defTransform, transform)
 				 */
 {
     Rect rootArea, finalArea;
-    int rootPos, finalPos;
+    int rootJust, finalJust;
 
     GeoTransRect(defTransform, &label->lab_rect, &rootArea);
-    rootPos = GeoTransPos(defTransform, label->lab_pos);
+    rootJust = GeoTransPos(defTransform, label->lab_just);
     GeoTransRect(transform, &rootArea, &finalArea);
-    finalPos = GeoTransPos(transform, rootPos);
-    (void) DBPutLabel(Select2Def, &finalArea, finalPos, label->lab_text,
+    finalJust = GeoTransPos(transform, rootJust);
+    (void) DBPutLabel(Select2Def, &finalArea, finalJust, label->lab_text,
 	    label->lab_type, label->lab_flags);
     return 0;
 }
@@ -459,7 +461,7 @@ selExpandFunc(selUse, use, transform, mask)
     }
 
     /* Be sure to modify the expansion bit in the selection as well as
-     * the one in the layout in order to keep them consistend.
+     * the one in the layout in order to keep them consistent.
      */
 
     if (DBDescendSubcell(use, mask))
@@ -591,6 +593,7 @@ selArrayCFunc(selUse, use, transform, arrayInfo)
 	(void) DBLinkCell(newUse, Select2Def);
     }
     newUse->cu_expandMask = use->cu_expandMask;
+    newUse->cu_flags = use->cu_flags;
 
     DBSetTrans(newUse, transform);
     GeoInvertTrans(transform, &tinv);
@@ -632,7 +635,7 @@ selArrayLFunc(label, use, transform, arrayInfo)
     Transform *transform;	/* Transform from coords of def to root. */
     ArrayInfo *arrayInfo;	/* How to replicate. */
 {
-    int y, nx, ny, rootPos;
+    int y, nx, ny, rootJust;
     Rect original, current;
     int labx, laby, xi, yi, only1;		/* numeric parts of label */
 
@@ -642,7 +645,7 @@ selArrayLFunc(label, use, transform, arrayInfo)
     if (ny < 0) ny = -ny;
 
     GeoTransRect(transform, &label->lab_rect, &original);
-    rootPos = GeoTransPos(transform, label->lab_pos);
+    rootJust = GeoTransPos(transform, label->lab_just);
     current = original;
 
     /* get the existing numbers in the label */
@@ -665,9 +668,9 @@ selArrayLFunc(label, use, transform, arrayInfo)
 	     * the same text and position is considered a duplicate.
 	     */
 	    yi = ny - y;
-	    DBEraseLabelsByContent(Select2Def, &current, rootPos,
+	    DBEraseLabelsByContent(Select2Def, &current, rootJust,
 		-1, nmPutNums(label->lab_text, labx + xi, laby +yi));
-	    (void) DBPutLabel(Select2Def, &current, rootPos,
+	    (void) DBPutLabel(Select2Def, &current, rootJust,
 		nmPutNums(label->lab_text, labx + xi, laby + yi),
 		label->lab_type, label->lab_flags);
 	    current.r_ybot += arrayInfo->ar_ysep;
@@ -765,8 +768,19 @@ SelectStretch(x, y)
 
     while (selStretchList != NULL)
     {
-	DBPaint(EditCellUse->cu_def, &selStretchList->sa_area,
-		selStretchList->sa_type);
+	TileTypeBitMask tmask;
+	TileType type = selStretchList->sa_type;
+
+#ifdef NONMANHATTAN
+	TileType tloc;
+	tloc = (type & TT_DIAGONAL) ?  ((type & TT_SIDE) ?
+		(type & TT_RIGHTMASK) >> 14 : (type & TT_LEFTMASK)) : type;
+	TTMaskSetOnlyType(&tmask, tloc);
+#else
+	TTMaskSetOnlyType(&tmask, type);
+#endif
+
+	DBPaintValid(EditCellUse->cu_def, &selStretchList->sa_area, &tmask, type);
 	freeMagic((char *) selStretchList);
 	selStretchList = selStretchList->sa_next;
     }
@@ -916,6 +930,8 @@ selStretchEraseFunc(tile, plane)
         TTMaskClearType(&mask, t);
     }
 
+    TTMaskAndMask(&mask, &DBActiveLayerBits);
+
     /*
      *   Erase all tile types specified in the processed mask.
      */
@@ -935,11 +951,13 @@ selStretchEraseFunc(tile, plane)
        {
            for (t = TT_SPACE + 1; t < DBNumUserLayers; t++)
 	       if (TTMaskHasType(&mask, t) && DBIsContact(t))
-	           DBErase(EditCellUse->cu_def, &editArea, t);
+		   if (!LockedLayer(t))
+			DBErase(EditCellUse->cu_def, &editArea, t);
        }
        for (t = TT_SPACE + 1; t < DBNumUserLayers; t++)
            if (TTMaskHasType(&mask, t) && !DBIsContact(t))
-	        DBErase(EditCellUse->cu_def, &editArea, t);
+		if (!LockedLayer(t))
+		    DBErase(EditCellUse->cu_def, &editArea, t);
     }
 
     /*
@@ -1073,7 +1091,7 @@ selStretchFillFunc2(tile, area)
 	if (DBTypeOnPlane(selStretchType, pNum))
 	    (void) DBSrPaintArea((Tile *) NULL,
 			EditCellUse->cu_def->cd_planes[pNum],
-			&editArea, &DBAllButSpaceAndDRCBits, selStretchFillFunc3,
+			&editArea, &DBActiveLayerBits, selStretchFillFunc3,
 			(ClientData) &spaceArea);
     }
     return 0;
