@@ -833,7 +833,7 @@ SimCellTileSrFunc(scx, fp)
 	tp = fp->tf_tpath;
 	tnext = tp->tp_next;
 	tp->tp_next = DBPrintUseId(scx, tp->tp_next, tp->tp_last -
-		tp->tp_next);
+		tp->tp_next, FALSE);
 	if (tp->tp_next < tp->tp_last)
 	{
 	    *(tp->tp_next++) = '/';
@@ -978,7 +978,7 @@ SimPutLabel(cellDef, rect, align, text, type)
 	}
     }
 
-    lab->lab_pos = align;
+    lab->lab_just = align;
     lab->lab_type = type;
     lab->lab_rect = *rect;
     lab->lab_next = NULL;
