@@ -535,13 +535,17 @@ proc magic::techmanager {{option "update"}} {
 proc magic::caption {winpath {subcommand {}}} {
    global Opts
 
-   if {$subcommand != {} && $subcommand != "writeable"} {
-      return;
+   if {$subcommand != {} && $subcommand != "writeable" && $subcommand != "load"} {
+      return
+   }
+   if {$winpath == ""} {
+      return
    }
 
    set framename [winfo parent $winpath]
    set caption [$winpath windowcaption]
    set subcaption1 [lindex $caption 0]
+   set techname [tech name]
    if {[catch {set Opts(tool)}]} {
       set Opts(tool) unknown
    }
@@ -552,7 +556,8 @@ proc magic::caption {winpath {subcommand {}}} {
       set subcaption2 $caption
    }
    ${framename}.titlebar.caption configure -text \
-	"Loaded: ${subcaption1} Editing: ${subcaption2} Tool: $Opts(tool)"
+	"Loaded: ${subcaption1} Editing: ${subcaption2} Tool: $Opts(tool) \
+	Technology: ${techname}"
 }
 
 # Allow captioning in the title window by tagging the "load" and "edit" commands
@@ -571,7 +576,7 @@ magic::tag view "magic::scrollupdate %W"
 magic::tag zoom "magic::scrollupdate %W"
 magic::tag findbox "magic::scrollupdate %W"
 magic::tag see "magic::toolupdate %W %1 %2"
-magic::tag tech "magic::techrebuild %W %1"
+magic::tag tech "magic::techrebuild %W %1; magic::caption %W %1"
 magic::tag drc "magic::drcupdate %1"
 magic::tag path "magic::techmanager update"
 if {$MagicBLT} {
