@@ -19,7 +19,7 @@
  *	stdio.h
  *	magic.h
  *
- * rcsid $Header: /ufs/repository/magic/textio/textio.h,v 1.2 2001/01/12 22:13:26 jsolomon Exp $
+ * rcsid $Header: /usr/cvsroot/magic-7.5/textio/textio.h,v 1.1.1.1 2006/04/10 22:03:13 tim Exp $
  */
 
 #ifndef _TEXTIO_H
@@ -29,7 +29,12 @@
 
 #ifdef MAGIC_WRAPPER
 extern char *TxBuffer;
-extern bool TxInputRedirect;
+extern unsigned char TxInputRedirect;
+
+#define TX_INPUT_NORMAL 	0	/* keys translated as macros	   */
+#define TX_INPUT_REDIRECTED	1	/* keys redirected to terminal	   */
+#define TX_INPUT_PENDING_RESET	2	/* about to switch back to state 0 */
+
 #endif
 
 extern int TxCurButtons;
