@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/utils/geometry.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/utils/geometry.c,v 1.2 2008/01/31 07:33:31 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -69,11 +69,13 @@ global int GeoOppositePos[] =
 
 /*
  *-------------------------------------------------------------------
- *	Declarations of exported rectangles:
+ *	Declarations of exported variables:
  *-------------------------------------------------------------------
  */
 
-global Rect GeoNullRect = { 0, 0, 0, 0 };
+global Rect  GeoNullRect = { 0, 0, 0, 0 };
+global Point GeoOrigin = { 0, 0 };
+
 
 /*-------------------------------------------------------------------
  *	GeoTransPoint --
