@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/utils/path.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/utils/path.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -102,7 +102,7 @@ PaAppend(char **pathptr, char *newstring)
  *	If the first character of the string indicated by psource is a
  *	dollar sign ("$") then the subsequent name is converted to the
  *	expansion of the indicated environment variable.  If the environment
- *	variable does not exist and the indicated name is "CAD_HOME", then
+ *	variable does not exist and the indicated name is "CAD_ROOT", then
  *	we substitute the value of CAD_DIR passed down from the Makefile.
  *	Otherwise, we return error status -1.
  *
@@ -221,10 +221,10 @@ PaExpand(psource, pdest, size)
 
 	if (string == NULL)
 	{
-	    /* Check for CAD_HOME = CAD_DIR, the only internal variable */
+	    /* Check for CAD_ROOT = CAD_DIR, the only internal variable */
 	    /* recognized (this is passed down from the Makefile)	*/
 
-	    if (!strcmp(expandName, "CAD_HOME"))
+	    if (!strcmp(expandName, "CAD_ROOT"))
 		string = CAD_DIR;
 	    else
 	    {
