@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/wiring/wireOps.c,v 1.1.1.1 2006/04/10 22:03:13 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/wiring/wireOps.c,v 1.2 2008/01/17 19:41:06 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -412,7 +412,7 @@ WireAddLeg(rect, point, direction)
 
     GeoTransRect(&RootToEditTransform, &new, &editArea);
     TTMaskSetOnlyType(&mask, WireType);
-    DBPaint(EditCellUse->cu_def, &editArea, WireType);
+    DBPaintValid(EditCellUse->cu_def, &editArea, &mask, 0);
     DBAdjustLabels(EditCellUse->cu_def, &editArea);
     DBWAreaChanged(EditCellUse->cu_def, &editArea, DBW_ALLWINDOWS, &mask);
     DRCCheckThis(EditCellUse->cu_def, TT_CHECKPAINT, &editArea);
@@ -602,7 +602,7 @@ WireShowLeg()
     /* Paint the new leg into the selection box. */
 
     TTMaskSetOnlyType(&mask, WireType);
-    DBPaint(SelectDef, &new, WireType);
+    DBPaintValid(SelectDef, &new, &mask, 0);
     DBAdjustLabels(SelectDef, &new);
     DBWAreaChanged(SelectDef, &new, DBW_ALLWINDOWS, &mask);
     DBReComputeBbox(SelectDef);
@@ -646,7 +646,7 @@ WireAddContact(newType, newWidth)
     Rect oldLeg, contactArea, tmp, tmp2, editArea;
     CellDef *boxRootDef;
     TileType oldType;
-    TileTypeBitMask mask;
+    TileTypeBitMask mask, allmask;
     int oldOverlap, newOverlap, i, totalSize, oldDir;
     Contact *contact;
     SearchContext scx;
@@ -759,25 +759,29 @@ WireAddContact(newType, newWidth)
     /* Paint the contact and its surrounding areas. */
 
     GeoTransRect(&RootToEditTransform, &contactArea, &editArea);
-    TTMaskSetOnlyType(&mask, contact->con_type);
     GEO_EXPAND(&editArea, -oldOverlap, &tmp);
-    DBPaint(EditCellUse->cu_def, &tmp, contact->con_type);
+    TTMaskSetOnlyType(&mask, contact->con_type);
+    TTMaskSetOnlyType(&allmask, contact->con_type);
+    DBPaintValid(EditCellUse->cu_def, &tmp, &mask, 0);
     if (contact->con_surround1 != 0)
     {
+	TTMaskSetOnlyType(&mask, contact->con_layer1);
+	TTMaskSetType(&allmask, contact->con_layer1);
 	GEO_EXPAND(&tmp, contact->con_surround1, &tmp2);
 	(void) GeoInclude(&tmp2, &editArea);
-	DBPaint(EditCellUse->cu_def, &tmp2, contact->con_layer1);
-	TTMaskSetType(&mask, contact->con_layer1);
+	DBPaintValid(EditCellUse->cu_def, &tmp2, &mask, 0);
     }
     if (contact->con_surround2 != 0)
     {
+	TTMaskSetOnlyType(&mask, contact->con_layer2);
+	TTMaskSetType(&allmask, contact->con_layer2);
 	GEO_EXPAND(&tmp, contact->con_surround2, &tmp2);
 	(void) GeoInclude(&tmp2, &editArea);
-	DBPaint(EditCellUse->cu_def, &tmp2, contact->con_layer2);
-	TTMaskSetType(&mask, contact->con_layer2);
+	DBPaintValid(EditCellUse->cu_def, &tmp2, &mask, 0);
     }
+
     DBAdjustLabels(EditCellUse->cu_def, &editArea);
-    DBWAreaChanged(EditCellUse->cu_def, &editArea, DBW_ALLWINDOWS, &mask);
+    DBWAreaChanged(EditCellUse->cu_def, &editArea, DBW_ALLWINDOWS, &allmask);
     DRCCheckThis(EditCellUse->cu_def, TT_CHECKPAINT, &editArea);
     DBReComputeBbox(EditCellUse->cu_def);
 
